@@ -81,6 +81,9 @@
 #define SHOW_LATER 1
 #define SHOW_NOW 2
 
+extern int start_with_sfm;
+extern int initial_frames_loaded;
+
 /*  Function prototype for affirmation dialog when exiting application  */
 static void      really_quit_dialog (void);
 static Argument* quit_invoker       (Argument *args);
@@ -190,7 +193,6 @@ gimp_init (int    gimp_argc,
 	     gimp_argv++;
 	     serverLog = *gimp_argv;
 	 }
-
 	if (*gimp_argv)
 	  file_open (*gimp_argv, *gimp_argv);
 	gimp_argv++;
@@ -215,6 +217,10 @@ gimp_init (int    gimp_argc,
   if (enable_layer_dialog)
        lc_dialog_create (-1);
   */
+  if (start_with_sfm)
+    {
+      sfm_dialog_create();
+    }
 }
 
 
@@ -583,6 +589,7 @@ app_init (void)
   RESET_BAR();
   WireBufferOpen(wire_buffer); /* xcf uses wire buffer! */
   xcf_init ();             /*  initialize the xcf file format routines */
+  cms_init();              /*  initialize colour management */
 
   app_init_update_status ("Looking for data files", "Brushes", 0.00);
   brushes_init (no_data);         /*  initialize the list of gimp brushes  */
@@ -664,6 +671,7 @@ app_exit_finish (void)
 #endif 
   levels_free ();
   brush_select_dialog_free ();
+  file_temp_clear();
   pattern_select_dialog_free ();
   palette_free ();
   paint_funcs_area_free ();
@@ -671,6 +679,8 @@ app_exit_finish (void)
   procedural_db_free ();
   menus_quit ();
   tile_swap_exit ();
+  file_temp_clear();
+  cms_free();
 
   /*  Things to do only if there is an interface  */
   if (no_interface == FALSE)
