@@ -292,6 +292,9 @@ channels_dialog_flush ()
   GSList *list;
   int gimage_pos;
   int pos;
+  CMSProfile *profile = 0;
+  static char colour_sig[5] = {"Rgb"};
+  const char* colour_sig_neu = {"Rgb"};
 
   if (!channelsD)
     return;
@@ -299,6 +302,17 @@ channels_dialog_flush ()
   if (! (gimage = gimage_get_ID (channelsD->gimage_id)))
     return;
 
+  profile = gimage_get_cms_profile(gimage);
+  if(profile)
+      colour_sig_neu = cms_get_profile_info(profile)->color_space_name;
+
+  if(strstr(colour_sig,colour_sig_neu) == 0)
+    {
+      sprintf(colour_sig, colour_sig_neu);
+      channelsD->gimage_id = -1;
+      channels_dialog_update (gimage->ID);
+    }
+
   /*  Check if the gimage extents have changed  */
   if ((gimage->width != channelsD->gimage_width) ||
       (gimage->height != channelsD->gimage_height) ||
@@ -428,6 +442,7 @@ channels_dialog_update (int gimage_id)
 
   /*  The image components  */
   item_list = NULL;
+
   switch ((channelsD->format = gimage_format (gimage)))
     {
     case FORMAT_RGB:
@@ -1028,6 +1043,14 @@ create_channel_widget (GImage      *gimage,
   GtkWidget *hbox;
   GtkWidget *vbox;
   GtkWidget *alignment;
+  CMSProfile *profile = gimage_get_cms_profile(gimage);
+  static const char* color_space_channel_names_[8] = {"Red","Green","Blue",""};
+  const char** color_space_channel_names = color_space_channel_names_;
+  if(profile)
+    {
+      color_space_channel_names = cms_get_profile_info(profile)->color_space_channel_names;
+    }
+  
 
   list_item = gtk_list_item_new ();
 
@@ -1094,9 +1117,9 @@ create_channel_widget (GImage      *gimage,
   /*  the channel name label */
   switch (channel_widget->type)
     {
-    case Red:       channel_widget->label = gtk_label_new ("Red"); break;
-    case Green:     channel_widget->label = gtk_label_new ("Green"); break;
-    case Blue:      channel_widget->label = gtk_label_new ("Blue"); break;
+    case Red:       channel_widget->label = gtk_label_new (color_space_channel_names[0]); break;
+    case Green:     channel_widget->label = gtk_label_new (color_space_channel_names[1]); break;
+    case Blue:      channel_widget->label = gtk_label_new (color_space_channel_names[2]); break;
     case Gray:      channel_widget->label = gtk_label_new ("Gray"); break;
     case Indexed:   channel_widget->label = gtk_label_new ("Indexed"); break;
     case Matte:     channel_widget->label = gtk_label_new ("Matte"); break;
