@@ -25,6 +25,17 @@
 #include <dirent.h>
 #include <gtk/gtk.h>
 
+#ifdef __FreeBSD__ 
+# define USE_ALLOCA
+#else
+# undef USE_ALLOCA
+# define MAX_NUM_PROFILES 1024 /* ARGH */
+#endif
+
+#ifdef USE_ALLOCA
+# include <stdlib.h>
+#endif
+
 #include "../lib/version.h"
 #include "convert.h"
 #include "fileops.h"
@@ -36,7 +47,9 @@
 #include "rc.h"
 #include "tag.h"
 
-//#define DEBUG
+
+#define _(translate_string) translate_string
+
 /* parameter for the transform cache */
 /* how many transform cache entries can be kept without being used by anyone
  * (for efficiency, since likely to be used again soon)
@@ -130,6 +143,7 @@ const char* cms_get_long_profile_info (cmsHPROFILE hProfile);
 const char* cms_get_profile_keyname   (cmsHPROFILE hProfile, char* mem);
 const char* cms_get_pcs_name          (cmsHPROFILE hProfile);
 const char* cms_get_color_space_name  (cmsHPROFILE hProfile);
+const char** cms_get_color_space_channel_names (cmsHPROFILE hProfile);
 const char* cms_get_device_class_name (cmsHPROFILE hProfile);
 
 
@@ -208,13 +222,19 @@ erase_alpha_from_4th_color (CMSTransform *transform)
  * called from app_procs.c:app_init()
  */
 void cms_init()
-{   profile_cache = g_hash_table_new(g_str_hash, g_str_equal);
+{
+    GList *remove_settings = NULL;
+    GList *update_settings = NULL;
+    cmsHPROFILE test = NULL;
+
+    profile_cache = g_hash_table_new(g_str_hash, g_str_equal);
     transform_cache = g_hash_table_new(g_str_hash, g_str_equal);         
     profile_info_buffer = g_new(CMSProfileInfo, 1);
     profile_info_buffer->manufacturer = NULL;
     profile_info_buffer->description = NULL;
     profile_info_buffer->pcs = NULL;
     profile_info_buffer->color_space_name = NULL;
+    profile_info_buffer->color_space_channel_names = NULL;
     profile_info_buffer->device_class_name = NULL;
     profile_info_buffer->long_info = NULL;    
 
@@ -222,40 +242,36 @@ void cms_init()
     /* suppress lcms errors while checking */
     cmsErrorAction(LCMS_ERROR_IGNORE);     
 
-    GList *remove_settings = NULL;
-    GList *update_settings = NULL;
-    cmsHPROFILE test = NULL;
-
     /* 1. image profile */
     if (cms_default_image_profile_name != NULL)
     {   test = cmsOpenProfileFromFile(cms_default_image_profile_name, "r");
-	if (test == NULL) 
-	{   g_warning ("Cannot open selected ICC default image profile: %s", cms_default_image_profile_name);
-	    cms_default_image_profile_name = NULL;
-	    remove_settings = g_list_append(remove_settings, "cms-default-image-profile-name");
-	}
-	cmsCloseProfile(test);
+        if (test == NULL) 
+        {   g_warning ("Cannot open selected ICC assumed image profile: %s", cms_default_image_profile_name);
+            cms_default_image_profile_name = NULL;
+            remove_settings = g_list_append(remove_settings, "cms-default-image-profile-name");
+        }
+        cmsCloseProfile(test);
     }
 
-    /* 2. workspace profile */
+    /* 2. editing/workspace profile */
     if (cms_workspace_profile_name != NULL)
     {   test = cmsOpenProfileFromFile(cms_workspace_profile_name, "r");
-	if (test == NULL) 
-	{   g_warning ("Cannot open selected ICC workspace profile: %s", cms_workspace_profile_name);
-	    cms_workspace_profile_name = NULL;
-	    remove_settings = g_list_append(remove_settings, "cms-workspace-profile-name");
-	}
+        if (test == NULL) 
+        {   g_warning ("Cannot open selected ICC editing profile: %s", cms_workspace_profile_name);
+            cms_workspace_profile_name = NULL;
+            remove_settings = g_list_append(remove_settings, "cms-workspace-profile-name");
+        }
 	cmsCloseProfile(test);
     }
 
     /* 3. display profile */
     if (cms_display_profile_name != NULL)
     {   test = cmsOpenProfileFromFile(cms_display_profile_name, "r");
-	if (test == NULL) 
-	{   g_warning ("Cannot open selected ICC display profile: %s", cms_display_profile_name);
-	    cms_display_profile_name = NULL;
-	    remove_settings = g_list_append(remove_settings, "cms-display-profile-name");
-	}
+        if (test == NULL) 
+        {   g_warning ("Cannot open selected ICC display profile: %s", cms_display_profile_name);
+            cms_display_profile_name = NULL;
+            remove_settings = g_list_append(remove_settings, "cms-display-profile-name");
+        }
 	/* make sure it is a display profile */
 	else if (cmsGetDeviceClass (test) != icSigDisplayClass)
 	{   g_warning ("Selected display profile is not a display profile: %s", cms_display_profile_name);
@@ -270,6 +286,17 @@ void cms_init()
 	cmsCloseProfile(test);
     }
     
+    /* 4. proof profile */
+    if (cms_default_proof_profile_name != NULL)
+    {   test = cmsOpenProfileFromFile(cms_default_proof_profile_name, "r");
+        if (test == NULL) 
+        {   g_warning ("Cannot open selected ICC default proof profile: %s", cms_default_proof_profile_name);
+            cms_default_proof_profile_name = NULL;
+            remove_settings = g_list_append(remove_settings, "cms-default-proof-profile-name");
+        }
+        cmsCloseProfile(test);
+    }
+
     if (g_list_length(remove_settings) != 0)
     {   save_gimprc(&update_settings, &remove_settings);
     }
@@ -344,6 +371,82 @@ cms_get_color_space_name (cmsHPROFILE hProfile)
     return name;
 }
 
+const char**
+cms_get_color_space_channel_names (cmsHPROFILE hProfile)
+{   static char** name = 0;
+    const char** ret = 0;
+    if(!name)
+    { int i;
+      name=(char**)calloc(sizeof(char**),4);
+      for(i = 0; i < 4; ++i)
+        name[i] = (char*)calloc(sizeof(char*),36);
+    }
+    ret = name;
+    sprintf( name[3],_("Alpha"));
+
+    switch (cmsGetColorSpace (hProfile))
+    {
+    case icSigXYZData: sprintf( name[0],_("CIE X"));
+                       sprintf( name[1],_("CIE Y (Luminance)"));
+                       sprintf( name[2],_("CIE Z")); break;
+    case icSigLabData: sprintf( name[0],_("CIE *L"));
+                       sprintf( name[1],_("CIE *a"));
+                       sprintf( name[2],_("CIE *b")); break;
+    case icSigLuvData: sprintf( name[0],_("CIE *L"));
+                       sprintf( name[1],_("CIE *u"));
+                       sprintf( name[2],_("CIE *v")); break;
+    case icSigYCbCrData: sprintf( name[0],_("Luminance Y"));
+                       sprintf( name[1],_("Colour b"));
+                       sprintf( name[2],_("Colour r")); break;
+    case icSigYxyData: sprintf( name[0],_("CIE Y (Luminance)"));
+                       sprintf( name[1],_("CIE x"));
+                       sprintf( name[2],_("CIE y")); break;
+    case icSigRgbData: sprintf( name[0],_("Red"));
+                       sprintf( name[1],_("Green"));
+                       sprintf( name[2],_("Blue")); break;
+    case icSigGrayData: sprintf( name[0],_("Black")); break;
+    case icSigHsvData: sprintf( name[0],_("Hue"));
+                       sprintf( name[1],_("Saturation"));
+                       sprintf( name[2],_("Value")); break;
+    case icSigHlsData: sprintf( name[0],_("Hue"));
+                       sprintf( name[1],_("Lightness"));
+                       sprintf( name[2],_("Saturation")); break;
+    case icSigCmykData: sprintf( name[0],_("Cyan"));
+                       sprintf( name[1],_("Magenta"));
+                       sprintf( name[2],_("Yellow"));
+                       sprintf( name[3],_("Black")); break;
+    case icSigCmyData: sprintf( name[0],_("Cyan"));
+                       sprintf( name[1],_("Magenta"));
+                       sprintf( name[2],_("Yellow")); break;
+    case icSig2colorData: sprintf( name[0],_("1. Colour"));
+                       sprintf( name[1],_("2. Colour")); break;
+    case icSig3colorData: sprintf( name[0],_("1. Colour"));
+                       sprintf( name[1],_("2. Colour"));
+                       sprintf( name[2],_("3. Colour")); break;
+    case icSig4colorData:  sprintf( name[0],_("1. Colour"));
+                       sprintf( name[1],_("2. Colour"));
+                       sprintf( name[2],_("3. Colour"));
+                       sprintf( name[3],_("4. Colour")); break;
+    case icSig5colorData:
+    case icSig6colorData:
+    case icSig7colorData:
+    case icSig8colorData:
+    case icSig9colorData:
+    case icSig10colorData:
+    case icSig11colorData:
+    case icSig12colorData:
+    case icSig13colorData:
+    case icSig14colorData:
+    case icSig15colorData:
+    default:           sprintf( name[0],_("1. Colour"));
+                       sprintf( name[1],_("2. Colour"));
+                       sprintf( name[2],_("3. Colour"));
+                       sprintf( name[3],_("4. Colour"));
+    }
+
+    return ret;
+}
+
 const char*
 cms_get_device_class_name (cmsHPROFILE hProfile)
 {   static char class[15];
@@ -372,16 +475,18 @@ icUInt16Value (icUInt16Number val)
   unsigned char        *temp  = (unsigned char*) &val;
   static unsigned char  korb[KORB];
   int i;
-  for (i = 0; i < KORB ; i++ )
-    korb[i] = (int) 0;  // leeren
-    
   int klein = 0,
       gross = BYTES - 1;
+  icUInt16Number *erg;
+
+  for (i = 0; i < KORB ; i++ )
+    korb[i] = (int) 0;  /* leeren */
+    
   for (; klein < BYTES ; klein++ ) {
     korb[klein] = temp[gross--];
   } 
     
-  icUInt16Number *erg = (icUInt16Number*) &korb[0];
+  erg = (icUInt16Number*) &korb[0];
 
   return (icUInt16Number)*erg;
 #else
@@ -406,7 +511,6 @@ cms_get_profile_keyname (cmsHPROFILE  hProfile, char* mem)
 }
 
 
-#define _(x) x
 const char *
 cms_get_long_profile_info(cmsHPROFILE  hProfile)
 {   static char profile_info[2048];
@@ -525,11 +629,11 @@ cms_read_icc_profile_dir(gchar *path, icProfileClassSignature class)
     while ((entry = readdir (dir)))
     {   char *file_name = entry->d_name;
         cmsHPROFILE profile;
+        GString *file_path = g_string_new(NULL);
 	if ((strcmp (file_name, "..") == 0) || (strcmp (file_name, ".") == 0)) 
 	{   continue;
 	}
 
-	GString *file_path = g_string_new(NULL);
 	g_string_sprintf (file_path, "%s/%s", path, file_name);
 
 	/* Open the file and try to read it using the lcms library
@@ -570,6 +674,7 @@ cms_read_standard_profile_dirs(icProfileClassSignature class)
     /* process path by path, paths are separated by : */
     char *remaining_directories = directories;    
     char *token = xstrsep(&remaining_directories, ":");
+	GSList *sub_list = 0;
     while (token) 
     {   /* replace ~ by home dir */
         if (*token == '~')
@@ -580,7 +685,7 @@ cms_read_standard_profile_dirs(icProfileClassSignature class)
         {   path = strdup(token);
 	} 
 
-	GSList *sub_list = cms_read_icc_profile_dir(path, class);
+	sub_list = cms_read_icc_profile_dir(path, class);
 	return_list = g_slist_concat(return_list, sub_list);
 
 	g_free(path);
@@ -605,6 +710,11 @@ cms_get_profile_info(CMSProfile *profile)
       profile_info_buffer->description = cmsTakeProductDesc(profile->handle);       
       profile_info_buffer->pcs = cms_get_pcs_name(profile->handle);
       profile_info_buffer->color_space_name = cms_get_color_space_name(profile->handle);
+      profile_info_buffer->color_space_channel_names = cms_get_color_space_channel_names(profile->handle);
+#     ifdef DEBUG
+      printf("%s:%d ",__FILE__,__LINE__);
+      printf("%s\n",profile_info_buffer->color_space_channel_names[2]);
+#     endif      
       profile_info_buffer->device_class_name = cms_get_device_class_name(profile->handle);
       profile_info_buffer->long_info = cms_get_long_profile_info(profile->handle);
 
@@ -732,6 +842,7 @@ cms_get_profile_from_file(char *file_name)
     size_t size_order = 128;
     char *mem = 0;
     const char *keyname = 0; 
+    ProfileCacheEntry *cache_entry = 0; 
 
     /* get profile information */ 
     cmsHPROFILE profile = cmsOpenProfileFromFile (file_name, "r");
@@ -743,7 +854,7 @@ cms_get_profile_from_file(char *file_name)
     keyname = cms_get_profile_keyname (profile, mem);
  
     /* check hash table for profile */ 
-    ProfileCacheEntry *cache_entry = g_hash_table_lookup(profile_cache, 
+    cache_entry = g_hash_table_lookup(profile_cache, 
 							 (gpointer) cms_get_profile_keyname(profile,mem));
     if (cache_entry != NULL) 
     {   cache_entry->ref_count ++;
@@ -783,6 +894,7 @@ cms_get_profile_from_file(char *file_name)
 CMSProfile *
 cms_get_profile_from_mem(void *mem_pointer, DWORD size)
 {   CMSProfile *return_value;
+    ProfileCacheEntry *cache_entry = 0; 
     
     /* get profile information */ 
     cmsHPROFILE profile = cmsOpenProfileFromMem (mem_pointer, size);
@@ -792,7 +904,7 @@ cms_get_profile_from_mem(void *mem_pointer, DWORD size)
     }   
 
     /* check hash table for profile */ 
-    ProfileCacheEntry *cache_entry = g_hash_table_lookup(profile_cache, 
+    cache_entry = g_hash_table_lookup(profile_cache, 
 							 (gpointer) cms_get_profile_keyname(profile,mem_pointer));
     if (cache_entry != NULL) 
     {   cache_entry->ref_count ++;
@@ -852,7 +964,7 @@ cms_get_profile_data(CMSProfile *profile, size_t *size)
 CMSProfile *
 cms_get_lab_profile(LPcmsCIExyY white_point) 
 {   CMSProfile *return_value;
-
+    ProfileCacheEntry *cache_entry = 0;
     GString *hash_key = g_string_new(NULL); 
     if (white_point) 
     {   g_string_sprintf(hash_key, "###LAB%f%f%f###", white_point->x, white_point->y, white_point->Y);
@@ -861,7 +973,7 @@ cms_get_lab_profile(LPcmsCIExyY white_point)
     {   g_string_sprintf(hash_key, "###LAB###");
     }
 
-    ProfileCacheEntry *cache_entry = g_hash_table_lookup(profile_cache, (gpointer) hash_key);
+    cache_entry = g_hash_table_lookup(profile_cache, (gpointer) hash_key);
     if (cache_entry != NULL) 
     {   cache_entry->ref_count ++;
         return_value = cache_entry->profile;
@@ -1031,7 +1143,8 @@ cms_duplicate_profile(CMSProfile *profile)
  */ 
 gboolean
 cms_return_profile(CMSProfile *profile)
-{   /* search the cache for the profile 
+{   ProfileCacheEntry *entry = 0;
+   /* search the cache for the profile 
      * decreate ref_counter + possibly close profile 
      */  
     if (profile == NULL)
@@ -1039,7 +1152,7 @@ cms_return_profile(CMSProfile *profile)
         return FALSE;
     } 
 
-    ProfileCacheEntry *entry = g_hash_table_lookup(profile_cache, (gpointer)profile->cache_key);
+    entry = g_hash_table_lookup(profile_cache, (gpointer)profile->cache_key);
     if (entry == NULL)
     {   g_warning("cms_return_profile: profile not found in cache");
         return FALSE;
@@ -1072,7 +1185,29 @@ cms_get_transform(GSList *profiles,
 {   /* turn profiles into an array as needed by lcms + 
        check all profiles are registered with the profile cache + 
        create hash key to check transform cache */
+    /* profiles list order:
+       1. image profile
+       2. look profiles (abstact profiles)
+       3. proofing profile , only if proofing is set (cmsFLAGS_SOFTPROOFING)
+       4. display profile
+    */
     int num_profiles = g_slist_length(profiles);
+#ifdef USE_ALLOCA
+    cmsHPROFILE *profile_array;
+#else
+    cmsHPROFILE profile_array[MAX_NUM_PROFILES];
+#endif
+    GString *hash_key = g_string_new(NULL);
+    int i;
+    CMSProfile *current_profile;
+    GSList *iterator = profiles;
+    cmsHTRANSFORM transform;
+    TransformCacheEntry *cache_entry = 0;
+
+#ifdef USE_ALLOCA
+    profile_array = (cmsHPROFILE)alloca(num_profiles*sizeof(cmsHPROFILE));
+#endif
+
     if (num_profiles == 0)
     {   g_warning("cms_get_transform: profile list is empty, cannot create transfrom");
         return NULL;
@@ -1092,99 +1227,152 @@ cms_get_transform(GSList *profiles,
         return NULL;
     }
 
-    cmsHPROFILE profile_array[num_profiles];
-    GString *hash_key = g_string_new(NULL);
-
-    int i;
-    CMSProfile *current_profile;
-    GSList *iterator = profiles;
-    for (i=0; ((i<num_profiles) && (iterator !=NULL)); i++) 
+    /* collect profiles in a list */
+    for (i=0; ((i<num_profiles) && (iterator !=NULL)); i++)
     {   current_profile = (CMSProfile *)iterator->data;
-        if (g_hash_table_lookup(profile_cache, (gpointer)current_profile->cache_key) == NULL) 
-	{   g_warning("cms_get_transform: at least one of the profiles is not registered. Need to register profiles before creating transform");
-	    return NULL;
-	}
-
-        profile_array[i] = current_profile->handle;
-	g_string_sprintfa(hash_key,"%p", current_profile->handle);
-        iterator = g_slist_next(iterator);     
+        if (g_hash_table_lookup(profile_cache, (gpointer)current_profile->cache_key) == NULL)
+       {   g_warning("cms_get_transform: at least one of the profiles is not registered. Need to register profiles before creating transform");
+           return NULL;
+       }
+       profile_array[i] = current_profile->handle;
+       g_string_sprintfa(hash_key,"%p", current_profile->handle);
+       iterator = g_slist_next(iterator);
     }
     
-    g_string_sprintfa(hash_key,"%d%d%d%d", lcms_input_format,lcms_output_format,
-                                           lcms_intent, lcms_flags);
+    g_string_sprintfa(hash_key,"%d%d%d%d%d",
+                      lcms_input_format,lcms_output_format,
+                      lcms_intent, lcms_flags, num_profiles);
 
     /* now check the cache */
-    TransformCacheEntry *cache_entry = g_hash_table_lookup(transform_cache, (gpointer) hash_key->str);
+    cache_entry = g_hash_table_lookup(transform_cache, (gpointer) hash_key->str);
 
     /* if it was in the disc cache */
-    if (cache_entry != NULL) 
+    if (cache_entry != NULL && !(lcms_flags & cmsFLAGS_SOFTPROOFING))
     {   cache_entry->ref_count++;
 
-	/* if it is not in memory, load it */
-        if (cache_entry->transform->handle == NULL) 
-	{   cmsHPROFILE device_link = cmsOpenProfileFromFile(cache_entry->device_link_file, "r");
-	    cache_entry->transform->handle = cmsCreateTransform(device_link,
+        /* if it is not in memory, load it */
+        if (cache_entry->transform->handle == NULL)
+        {   cmsHPROFILE device_link = cmsOpenProfileFromFile(cache_entry->device_link_file, "r");
+            cache_entry->transform->handle = cmsCreateTransform(device_link,
 								lcms_input_format, 
 								NULL,
-								lcms_output_format, 
-								lcms_intent, 
-								lcms_flags);	   	    	    
-	    cmsCloseProfile(device_link);
-	}	
-	/* else, if we are the first to use it again
-	   remove it from the list of unused transforms in mem */
-	else if (cache_entry->ref_count == 1) 
-	{   unused_transforms_kept_in_mem = g_list_remove(unused_transforms_kept_in_mem,
-							  cache_entry);
-	}
+								lcms_output_format,
+								lcms_intent,
+								lcms_flags);
+            cmsCloseProfile(device_link);
+        }
+        /* else, if we are the first to use it again
+           remove it from the list of unused transforms in mem */
+        else if (cache_entry->ref_count == 1)
+        {   unused_transforms_kept_in_mem = g_list_remove(unused_transforms_kept_in_mem,
+                                                      cache_entry);
+        }
 
-	g_string_free(hash_key, TRUE);
-	return cache_entry->transform;	
+        g_string_free(hash_key, TRUE);
+        return cache_entry->transform;
     }
 
-    /* if no cache hit, create transform */
-    cmsHTRANSFORM transform;
+    /* if no cache hit, create transform , do a proofing exeption */
+    if( lcms_flags & cmsFLAGS_SOFTPROOFING )
+    {
+        /* For more than 3 profiles we have to create a new profile ... */
+        if(num_profiles > 3)
+        {
+            cmsHPROFILE look = 0;
+            transform = cmsCreateMultiprofileTransform  (profile_array,
+                             num_profiles - 2,
+                             lcms_input_format,
+                             NOCOLORSPACECHECK(lcms_output_format),
+                             lcms_intent,
+                             lcms_flags);
+
+            if (!transform) 
+            {   g_warning ("%s:%d ICC profile not valid.",__FILE__,__LINE__);
+                return NULL;
+            }							     
+            look = cmsTransform2DeviceLink( transform,
+                                                   cmsFLAGS_GUESSDEVICECLASS );
+            cmsDeleteTransform (transform);
+            /* ... and include the new profile in the proofing transform */
+            transform = cmsCreateProofingTransform(look, lcms_input_format,
+                                           profile_array[num_profiles-1], lcms_output_format,
+                                           profile_array[num_profiles-2],
+                                           lcms_intent,
+                                           INTENT_RELATIVE_COLORIMETRIC,
+                                           lcms_flags);
+            if (!transform) 
+            {   g_warning ("%s:%d ICC profile not valid.",__FILE__,__LINE__);
+                return NULL;
+            }							     
+        } else
+        {   /* just a proofing transform */
+            transform = cmsCreateProofingTransform(profile_array[0], lcms_input_format,
+                                           profile_array[2], lcms_output_format,
+                                           profile_array[1],
+                                           lcms_intent,
+                                           INTENT_RELATIVE_COLORIMETRIC,
+                                           lcms_flags);
+            if (!transform) 
+            {   g_warning ("%s:%d ICC profile not valid.",__FILE__,__LINE__);
+                return NULL;
+            }							     
+        }
+    } else
+    {   /* no proofing */
         transform = cmsCreateMultiprofileTransform  (profile_array,
 						     num_profiles,
 						     lcms_input_format,
 						     lcms_output_format,
 						     lcms_intent,
 						     lcms_flags);
+        if (!transform) 
+        {   g_warning ("%s:%d ICC profile not valid.",__FILE__,__LINE__);
+            return NULL;
+        }							     
+    }
 
-    if (!transform) 
-    {   g_warning ("%s:%d ICC profile not valid.",__FILE__,__LINE__);
-        return NULL;
-    }							     
 
-    /* save it to disk */
-    cmsHPROFILE devicelink = cmsTransform2DeviceLink(transform,0);
-    char *file_name = file_temp_name("icc");
-    _cmsSaveProfile(devicelink, file_name);
-    cmsCloseProfile(devicelink);
+    {
+        /* save it to disk */
+        cmsHPROFILE devicelink = cmsTransform2DeviceLink(transform,0);
+        char *file_name = file_temp_name("icc");
+        if(devicelink)
+        {
+#           ifdef DEBUG
+            printf("%s:%d %s() %x %x \n",__FILE__,__LINE__,__func__,(int)devicelink,(int)file_name);
+#           endif
+            _cmsSaveProfile( devicelink, file_name );
+            cmsCloseProfile( devicelink );
+        }
 
-    /* TODO debugging */
-    #ifdef DEBUG
-    printf ("%s:%d %s() %s written\n",__FILE__,__LINE__,__func__, file_name);
-    #endif
+        /* TODO debugging */
+        #ifdef DEBUG
+        printf ("%s:%d %s() %s written\n",__FILE__,__LINE__,__func__, file_name);
+        #endif
+        ;
+        /* and store a reference in the cache */
+        cache_entry = g_new(TransformCacheEntry, 1);
+        cache_entry->ref_count = 1;
+        cache_entry->transform = g_new(CMSTransform, 1);
+        cache_entry->transform->cache_key = hash_key->str;
+        cache_entry->transform->handle = transform;
+        /* CMYK hack: do we see 4 color data? */
+        if (_cmsChannelsOf (cmsGetColorSpace(profile_array[0])) >= 4
+         && _cmsChannelsOf (cmsGetColorSpace(profile_array[num_profiles-1])) <= 3) {
+            cache_entry->transform->erase_alpha_from_4th_color = TRUE;
+        } else
+            cache_entry->transform->erase_alpha_from_4th_color = FALSE;
+        cache_entry->device_link_file = file_name;
 
-    /* and store a reference in the cache */
-    cache_entry = g_new(TransformCacheEntry, 1);
-    cache_entry->ref_count = 1;
-    cache_entry->transform = g_new(CMSTransform, 1);
-    cache_entry->transform->cache_key = hash_key->str;
-    cache_entry->transform->handle = transform;
-    /* CMYK hack: do we see 4 color data? */
-    if (_cmsChannelsOf (cmsGetColorSpace(profile_array[0])) >= 4
-     && _cmsChannelsOf (cmsGetColorSpace(profile_array[num_profiles-1])) <= 3) {
-        cache_entry->transform->erase_alpha_from_4th_color = TRUE;
-    } else
-        cache_entry->transform->erase_alpha_from_4th_color = FALSE;
-    cache_entry->device_link_file = file_name;
-
-    g_hash_table_insert(transform_cache,
-			(gpointer) hash_key->str,
-			(gpointer) cache_entry);
+    /* TODO insert me in transform_cache and ignore + remove */
+    if (!(num_profiles == 3 &&
+          (lcms_flags & cmsFLAGS_SOFTPROOFING) &&
+          (lcms_flags & cmsFLAGS_GAMUTCHECK)     ))
+        g_hash_table_insert(transform_cache,
+    			(gpointer) hash_key->str,
+	    		(gpointer) cache_entry);
 
+    }
     g_string_free(hash_key, FALSE);
 
     return cache_entry->transform;    
@@ -1221,7 +1409,7 @@ cms_get_canvas_transform(GSList *profiles, Canvas *canvas,
 
     return cms_get_transform(profiles, lcms_format_in, lcms_format_out,
                                        lcms_intent, lcms_flags);
- }
+}
 
 
 gboolean 
@@ -1278,12 +1466,84 @@ cms_set_display_profile(CMSProfile *profile)
 }
 
 
+/* transform a pixel buffer, src and dest have to be same precision */
+void 
+cms_transform_buffer (CMSTransform *transform, void *src_data, void *dest_data,
+			  int num_pixels, Precision precision) 
+{   TransformFunc transform_func;
+
+    if (!transform || !transform->handle)
+    {   g_warning ("%s:%d %s(): transform empty",__FILE__,__LINE__,__func__);
+        return;
+    }
+    if (!src_data || !dest_data)
+    {   g_warning ("%s:%d %s(): buffer not allocated",__FILE__,__LINE__,__func__);
+        return;
+    }
+
+    switch (precision)
+    {
+
+    case PRECISION_U8:
+    case PRECISION_U16:  
+        transform_func = cms_transform_uint;
+        break;
+    case PRECISION_FLOAT:
+        transform_func = cms_transform_float;
+        break;
+    default:
+        g_warning ("cms_transform: precision not supported");
+        return;
+    }
+
+	(*transform_func) (transform, src_data, dest_data, num_pixels);
+        #ifdef DEBUG_
+        printf("%s:%d diff = %d\n",__FILE__,__LINE__,num_pixels);
+        #endif
+
+	/* part of the CMYK hack */
+	if ( transform->erase_alpha_from_4th_color )
+	{    int i;
+		 guint8  *u8;
+		 guint16 *u16;
+		 float   *f32;
+
+		 switch (precision)
+		 {
+			 case PRECISION_U8:
+				 u8 = dest_data;
+				 for (i = 3; i < num_pixels * 4 ; i += 4)
+					 u8[i] = 255;
+				 break;
+			 case PRECISION_U16:
+				 u16 = dest_data;
+				 for (i = 3; i < num_pixels * 4 ; i += 4)
+					 u16[i] = 65535;
+				 break;
+			 case PRECISION_FLOAT:
+				 f32 = dest_data;
+				 for (i = 3; i < num_pixels * 4 ; i += 4)
+					 f32[i] = 1.0;
+				 break;
+			 default:
+				 g_warning ("cms_transform: precision not supported");
+				 return;
+		 }
+	}
+}
+
 /* transforms a pixel area, src and dest have to be same precision */
 void 
 cms_transform_area(CMSTransform *transform, PixelArea *src_area, PixelArea *dest_area) 
 {   TransformFunc transform_func;
     Tag src_tag = pixelarea_tag(src_area); 
     Tag dest_tag = pixelarea_tag(dest_area); 
+    PixelRow src_row_buffer, dest_row_buffer;
+    guint h;
+    void *src_data, *dest_data;
+    guint num_pixels;
+    void *pag;
+    
     if (tag_precision(src_tag) != tag_precision(dest_tag)) 
     {   g_warning("cms_transform: src and dest area have to have same precision");
     }
@@ -1309,12 +1569,6 @@ cms_transform_area(CMSTransform *transform, PixelArea *src_area, PixelArea *dest
     }
 
 
-    PixelRow src_row_buffer, dest_row_buffer;
-    guint h;
-    void *src_data, *dest_data;
-    guint num_pixels;
-    void *pag;
-    
     for (pag = pixelarea_register (1, src_area, dest_area);
 	 pag != NULL;
 	 pag = pixelarea_process (pag))
@@ -1385,6 +1639,9 @@ void
 cms_transform_float(CMSTransform *transform, void *src_data, void *dest_data, int num_pixels) 
 {   /* need to convert data to double for lcms's convenience */
     int i;
+    float *src_fbuffer = (float *)src_data;
+    float *dest_fbuffer = (float *)dest_data;
+    double *dbuffer = 0; 
     if (!transform || !transform->handle)
         g_warning ("%s:%d %s() transform not allocated\n",
                    __FILE__,__LINE__,__func__);
@@ -1392,9 +1649,7 @@ cms_transform_float(CMSTransform *transform, void *src_data, void *dest_data, in
         g_warning ("%s:%d %s() array not allocated\n",
                    __FILE__,__LINE__,__func__);
 
-    float *src_fbuffer = (float *)src_data;
-    float *dest_fbuffer = (float *)dest_data;
-    double *dbuffer = malloc(sizeof(double) * num_pixels * 4); 
+    dbuffer = malloc(sizeof(double) * num_pixels * 4); 
     for (i=0; i < num_pixels * 4; i++) 
     {   dbuffer[i]=(double)src_fbuffer[i];
     }
@@ -1464,9 +1719,13 @@ static GtkWidget *cms_profile_menu_new(GtkTable *table, guint left, guint top,
     GtkWidget *menuitem;
     GtkWidget *optionmenu;
        
-    menu = gtk_menu_new (); 
     GSList *profile_file_names = cms_read_standard_profile_dirs(CMS_ANY_PROFILECLASS);      
     GSList *iterator = profile_file_names;
+    gchar *current_filename;
+    CMSProfile *current_profile;
+    CMSProfileInfo *current_profile_info;      
+
+    menu = gtk_menu_new (); 
 
     if (can_select_none)
     {   menuitem = gtk_menu_item_new_with_label("[none]");
@@ -1474,9 +1733,6 @@ static GtkWidget *cms_profile_menu_new(GtkTable *table, guint left, guint top,
 	gtk_object_set_data(GTK_OBJECT(menuitem), "value", NULL);      
     }
 
-    gchar *current_filename;
-    CMSProfile *current_profile;
-    CMSProfileInfo *current_profile_info;      
     while (iterator != NULL)
     {   current_filename = iterator->data;
         current_profile = cms_get_profile_from_file(current_filename);
@@ -1506,9 +1762,9 @@ static GtkWidget *cms_intent_menu_new(GtkTable *table, guint left, guint top)
     GtkWidget *menuitem = NULL;
     GtkWidget *optionmenu = NULL;
       
-    menu = gtk_menu_new ();	 
     guint8 *value = NULL;
 
+    menu = gtk_menu_new ();	 
     menuitem = gtk_menu_item_new_with_label ("Perceptual");
     gtk_menu_append (GTK_MENU (menu), menuitem);
     value = g_new(guint8, 1);
@@ -1672,6 +1928,7 @@ typedef struct _CMSAssignDialogData
     GtkWidget *profile_menu;
 
     GImage *image;
+    CMSProfileType type;
 } CMSAssignDialogData;
 
 /* private prototypes */    
@@ -1680,18 +1937,24 @@ static void cms_assign_dialog_cancel_callback(GtkWidget*, CMSAssignDialogData*);
 
 /* public functions */
 void 
-cms_assign_dialog(GImage *image)
+cms_assign_dialog(GImage *image, CMSProfileType type)
 {   GtkWidget *table = NULL;
     GtkWidget *label = NULL; 
     GtkWidget *vbox = NULL;
     GtkWidget *alignment = NULL;
+    CMSProfile *current_profile = NULL;
 
     CMSAssignDialogData *data=g_new(CMSAssignDialogData,1); 
+    char title[][32] = {"Select Image Profile",
+                        "Select Proof Profile"};
+    GString *profile_string = g_string_new(NULL);
+
     data->image = image;
     data->shell = gtk_dialog_new();
+    data->type = type;
 
     gtk_window_set_wmclass (GTK_WINDOW (data->shell), "cms_assign", PROGRAM_NAME);
-    gtk_window_set_title (GTK_WINDOW (data->shell), "Select Profile");
+    gtk_window_set_title (GTK_WINDOW (data->shell), title[type]);
     gtk_window_set_position(GTK_WINDOW(data->shell), GTK_WIN_POS_CENTER);
 
     vbox = gtk_vbox_new(FALSE, 0);   
@@ -1700,8 +1963,10 @@ cms_assign_dialog(GImage *image)
     table = gtk_table_new(3, 1, FALSE);
     gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
     
-    CMSProfile *current_profile = gimage_get_cms_profile(image);
-    GString *profile_string = g_string_new(NULL);
+    if (type ==  ICC_IMAGE_PROFILE)
+      current_profile = gimage_get_cms_profile(image);
+    else if (type == ICC_PROOF_PROFILE)
+      current_profile = gimage_get_cms_proof_profile(image);
     
     if (current_profile == NULL)
     {   g_string_sprintf (profile_string, "[The currently assigned profile is: [none]]");
@@ -1717,7 +1982,11 @@ cms_assign_dialog(GImage *image)
 		     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
 
     alignment = gtk_alignment_new(0.0, 1.0, 0.0, 0.0);
-    label = gtk_label_new("Please select the new profile to assign:");
+    
+    if (type ==  ICC_IMAGE_PROFILE)
+      label = gtk_label_new("Please select the new image profile to assign:");
+    else if (type == ICC_PROOF_PROFILE)
+      label = gtk_label_new("Please select the new proof profile to assign:");
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 1.0);
     gtk_container_add(GTK_CONTAINER(alignment), label);
     gtk_table_attach(GTK_TABLE(table), alignment, 0, 1, 1, 2, 
@@ -1745,13 +2014,25 @@ cms_assign_dialog_ok_callback(GtkWidget *widget, CMSAssignDialogData *data)
     
     /* assign it */
     if (profile_name != NULL) 
-    {   gimage_set_cms_profile(data->image, cms_get_profile_from_file(profile_name));
+    {
+        if (data->type ==  ICC_IMAGE_PROFILE)
+            gimage_set_cms_profile(data->image, cms_get_profile_from_file(profile_name));
+        else if (data->type == ICC_PROOF_PROFILE)
+            gimage_set_cms_proof_profile(data->image, cms_get_profile_from_file(profile_name));
     }
     else 
-    {   g_message("You set the image profile to [none]. Colormanagement will be\nswitched off for all displays of this image.");
-        gimage_set_cms_profile(data->image, NULL);
+    {
+        if        (data->type == ICC_IMAGE_PROFILE)
+        {   g_message("You set the image profile to [none]. Colormanagement will be\nswitched off for all displays of this image.");
+            gimage_set_cms_profile(data->image, NULL);
+        } else if (data->type == ICC_PROOF_PROFILE)
+        {   g_message("You set the proof profile to [none]. Proofing will be\nnot available off for all displays of this image.");
+            gimage_set_cms_proof_profile(data->image, NULL);
+        }
     }
-    
+    gdisplays_update_gimage (data->image->ID);
+    gdisplays_flush();
+
     /* destroy dialog */
     cms_assign_dialog_cancel_callback(widget, data);
 }
@@ -1793,7 +2074,9 @@ cms_open_assign_dialog(GImage *image)
     GtkWidget *vbox = NULL;
     GtkWidget *alignment = NULL;
 
+    GString *profile_string = g_string_new(NULL);    
     CMSOpenAssignDialogData *data=g_new(CMSOpenAssignDialogData,1); 
+
     data->image = image;
     data->shell = gtk_dialog_new();
 
@@ -1806,7 +2089,7 @@ cms_open_assign_dialog(GImage *image)
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG(data->shell)->vbox), vbox, TRUE, TRUE, 0);
     
     /* message */
-    label = gtk_label_new("Please select the profile to assign to the image:");
+    label = gtk_label_new("Please select the image profile to assign to the image:");
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 
@@ -1821,12 +2104,11 @@ cms_open_assign_dialog(GImage *image)
 						  
     data->profile_menu = cms_profile_menu_new(GTK_TABLE(table), 1, 0, TRUE);
 
-    GString *profile_string = g_string_new(NULL);    
     if (cms_default_image_profile_name == NULL)
-    {   g_string_sprintf (profile_string, "The default image profile: [none]");
+    {   g_string_sprintf (profile_string, "The assumed image profile: [none]");
     }
     else
-    {   g_string_sprintf (profile_string, "The default image profile: %s", 
+    {   g_string_sprintf (profile_string, "The assumed image profile: %s", 
 			  cmsTakeProductDesc(cms_get_profile_from_file(cms_default_image_profile_name)->handle));
     }
 
@@ -1888,9 +2170,11 @@ cms_open_assign_dialog_ok_callback(GtkWidget *widget, CMSOpenAssignDialogData *d
     
     /* update cms_open_action (global preference variable) */
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->always_default_check)))
-    {   cms_open_action = CMS_ASSIGN_DEFAULT;
+    {
         GList *update_settings = NULL;
         GList *remove_settings = NULL;
+
+        cms_open_action = CMS_ASSIGN_DEFAULT;
 	update_settings = g_list_append(update_settings, "cms-open-action");
 	save_gimprc(&update_settings, &remove_settings); 
 	g_list_free(update_settings);
@@ -1940,6 +2224,7 @@ cms_convert_on_open_prompt(GImage *image)
     GtkWidget *vbox;
     GtkWidget *alignment;
 
+    gboolean return_value;
     CMSConvertOnOpenPromptData *data = g_new(CMSConvertOnOpenPromptData, 1);
 
     data->shell = gtk_dialog_new();
@@ -1952,7 +2237,7 @@ cms_convert_on_open_prompt(GImage *image)
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG(data->shell)->vbox), vbox, TRUE, TRUE, 0);
 
     /* message */
-    label = gtk_label_new ("The profile that was assigned to this image does not match your workspace profile.");
+    label = gtk_label_new ("The profile that was assigned to this image does not match your editing profile.");
     gtk_widget_set_usize (label, 350, 25);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -1962,7 +2247,7 @@ cms_convert_on_open_prompt(GImage *image)
     table = gtk_table_new(2,2,FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 10);  
 
-    label = gtk_label_new("Workspace profile:");
+    label = gtk_label_new("Editing profile:");
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
 		     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);   
@@ -2010,7 +2295,7 @@ cms_convert_on_open_prompt(GImage *image)
     data->event_loop = g_main_new(FALSE);    
     g_main_run(data->event_loop);
 
-    gboolean return_value = data->return_value;
+    return_value = data->return_value;
     g_main_destroy(data->event_loop);
     g_free(data);
 
@@ -2023,9 +2308,10 @@ cms_convert_on_open_prompt_yes_callback (GtkWidget *widget, CMSConvertOnOpenProm
 {   data->return_value = TRUE;  
     if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->auto_convert_check)))
     {   /* cms_auto_convert is global preference variable */
-        cms_mismatch_action = CMS_MISMATCH_AUTO_CONVERT;
         GList *update_settings = NULL;
         GList *remove_settings = NULL;
+
+        cms_mismatch_action = CMS_MISMATCH_AUTO_CONVERT;
 	update_settings = g_list_append(update_settings, "cms-mismatch-action");
 	save_gimprc(&update_settings, &remove_settings); 
 	g_list_free(update_settings);
