@@ -246,12 +246,13 @@ static void _color_correction_gui_save_profile_ok_cb(_ColorCorrectionGuiSaveProf
 
 void 
 color_correction_gui (GDisplay *display)
-{   if (! drawable_color (gimage_active_drawable (display->gimage)))
+{  _ColorCorrectionGui *data = 0;
+   if (! drawable_color (gimage_active_drawable (display->gimage)))
     {   g_message ("Color correction operates only on RGB color drawables.");
         return;
     }
 
-    _ColorCorrectionGui *data = g_new(_ColorCorrectionGui, 1);
+    data = g_new(_ColorCorrectionGui, 1);
     data->display = display;
     data->active=TRUE;
     data->image_map = image_map_create (data->display, 
@@ -280,6 +281,13 @@ _color_correction_gui_create (_ColorCorrectionGui *data)
     GtkWidget *label =NULL;
     GtkWidget *notebook = NULL;
     GtkWidget *frame = NULL;
+	GtkObject *hue_adjustment = gtk_adjustment_new(0.0, 0.0, 360.0, 1.0, 0.0, 0.0);
+	GtkObject *saturation_adjustment = gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 0.0, 0.0);
+	GtkObject *lightness_adjustment = gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 0.0, 0.0);
+	_ColorCorrectionGuiSpin *spin_data = g_new(_ColorCorrectionGuiSpin, 1);
+	_ColorCorrectionGuiWheel *wheel_data=g_new(_ColorCorrectionGuiWheel, 1);
+	_ColorCorrectionGuiLSlider *lslider_data = g_new(_ColorCorrectionGuiLSlider, 1);
+    int i;
        
 
     /*** MAIN SHELL AND NOTEBOOK */
@@ -308,7 +316,6 @@ _color_correction_gui_create (_ColorCorrectionGui *data)
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, button);
 
     /*** WHEELS + SLIDERS + INPUTS ***/
-    int i;
     for (i=SHADOWS; i<=HIGHLIGHTS; i++)
     {   data->settings->values[i][HUE]=0.0;      
         data->settings->values[i][SATURATION]=0.0;      
@@ -325,16 +332,15 @@ _color_correction_gui_create (_ColorCorrectionGui *data)
 	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 3);
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	//	GtkStyle *style= gtk_style_new();
-	//style->fg[GTK_STATE_NORMAL]=style->dark[GTK_STATE_NORMAL];
-	//gtk_widget_set_style(hbox, style);
+	/*	GtkStyle *style= gtk_style_new(); */
+	/*style->fg[GTK_STATE_NORMAL]=style->dark[GTK_STATE_NORMAL]; */
+	/*gtk_widget_set_style(hbox, style); */
 
 	gtk_container_add(GTK_CONTAINER(frame), hbox);    
 
 	/* WHEEL */
 	data->color_wheels[i] = gtk_drawing_area_new ();
 	gtk_drawing_area_size(GTK_DRAWING_AREA(data->color_wheels[i]), WHEEL_RADIUS*2+2,WHEEL_RADIUS*2+2); 
-	_ColorCorrectionGuiWheel *wheel_data=g_new(_ColorCorrectionGuiWheel, 1);
 	wheel_data->range = i;
 	wheel_data->pixmap = NULL;
 	wheel_data->display_buffer = NULL;
@@ -363,10 +369,9 @@ _color_correction_gui_create (_ColorCorrectionGui *data)
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
 	gtk_widget_set_usize(frame, LSLIDER_WIDTH, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), frame, FALSE, FALSE, 0);
-	//	gtk_widget_set_style(frame, style);
+	/*	gtk_widget_set_style(frame, style); */
 
 	data->lsliders[i] = gtk_drawing_area_new();    
-	_ColorCorrectionGuiLSlider *lslider_data = g_new(_ColorCorrectionGuiLSlider, 1);
 	lslider_data->range= i;
 	lslider_data->gc = NULL;
 	lslider_data->pixmap = NULL;
@@ -383,16 +388,12 @@ _color_correction_gui_create (_ColorCorrectionGui *data)
 	gtk_container_add(GTK_CONTAINER(frame), data->lsliders[i]);
 
 
-	/* INNPUTS */
-	GtkObject *hue_adjustment = gtk_adjustment_new(0.0, 0.0, 360.0, 1.0, 0.0, 0.0);
-	GtkObject *saturation_adjustment = gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 0.0, 0.0);
-	GtkObject *lightness_adjustment = gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 0.0, 0.0);
+	/* INPUTS */
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 3);    
 
 
-	_ColorCorrectionGuiSpin *spin_data = g_new(_ColorCorrectionGuiSpin, 1);
 	spin_data->range = i;
 	spin_data->attribute = HUE;
 	label = gtk_label_new("H");
@@ -500,11 +501,11 @@ _color_correction_gui_preview(_ColorCorrectionGui *data)
 static void
 _color_correction_gui_update_wheel_handle (_ColorCorrectionGui *dialog,
 					   ColorCorrectionLightnessRange range)
-{   _ColorCorrectionGuiWheel *data;
-     data = (_ColorCorrectionGuiWheel *)gtk_object_get_data(GTK_OBJECT(dialog->color_wheels[range]), 
+{   _ColorCorrectionGuiWheel *data =
+     (_ColorCorrectionGuiWheel *)gtk_object_get_data(GTK_OBJECT(dialog->color_wheels[range]), 
 								"data");
-     
      gint new_pos[2]; 
+
      _color_correction_gui_wheel_hs_to_pos(dialog->settings->values[range][HUE],
 					   dialog->settings->values[range][SATURATION],
 					   data->radius, data->radius,
@@ -520,11 +521,11 @@ _color_correction_gui_update_wheel_handle (_ColorCorrectionGui *dialog,
 static void
 _color_correction_gui_update_lslider_handle (_ColorCorrectionGui *dialog,
 					     ColorCorrectionLightnessRange range)
-{   _ColorCorrectionGuiLSlider *data;
-    data = (_ColorCorrectionGuiLSlider *)gtk_object_get_data(GTK_OBJECT(dialog->lsliders[range]), 
+{   _ColorCorrectionGuiLSlider *data =
+    (_ColorCorrectionGuiLSlider *)gtk_object_get_data(GTK_OBJECT(dialog->lsliders[range]), 
 								  "data");
-
     gint new_position =  (0.5-dialog->settings->values[range][LIGHTNESS]) * data->height;
+
     if (new_position != data->position)
     {   _color_correction_gui_erase_lslider_handle(dialog->lsliders[range], data);
         data->position = new_position;
@@ -603,6 +604,7 @@ _color_correction_gui_wheel_events_cb (GtkWidget *widget,
 				   GdkEvent  *event,
 				   _ColorCorrectionGui *dialog)
 {   _ColorCorrectionGuiWheel *data = (_ColorCorrectionGuiWheel *)gtk_object_get_data(GTK_OBJECT(widget), "data");
+    _ColorCorrectionGuiLSlider *slider_data;
   
     switch (event->type)
     {
@@ -631,7 +633,6 @@ _color_correction_gui_wheel_events_cb (GtkWidget *widget,
 					      data->radius, data->radius, 
 					      &dialog->settings->values[data->range][HUE], 
 					      &dialog->settings->values[data->range][SATURATION]); 
-	     _ColorCorrectionGuiLSlider *slider_data;
 	     slider_data = (_ColorCorrectionGuiLSlider *)gtk_object_get_data(GTK_OBJECT(dialog->lsliders[data->range]), 
 										 "data");
 	     dialog->settings->values[data->range][LIGHTNESS] = (1-slider_data->position/(gfloat)slider_data->height) - 0.5;	     
@@ -697,6 +698,7 @@ _color_correction_gui_lslider_events_cb (GtkWidget *widget,
 				 GdkEvent  *event,
 				 _ColorCorrectionGui *dialog)
 {   _ColorCorrectionGuiLSlider *data = (_ColorCorrectionGuiLSlider *)gtk_object_get_data(GTK_OBJECT(widget), "data");
+    gfloat new_lightness;
 
     switch (event->type)
     {
@@ -716,7 +718,7 @@ _color_correction_gui_lslider_events_cb (GtkWidget *widget,
       _color_correction_gui_erase_lslider_handle(widget, data);
 
       data->position=event->button.y;
-      gfloat new_lightness = (1-data->position/(gfloat)data->height) -0.5;
+      new_lightness = (1-data->position/(gfloat)data->height) -0.5;
       if (dialog->settings->values[data->range][LIGHTNESS] != new_lightness) 
       {   if(dialog->settings->values[data->range][LIGHTNESS] == 0)
 	  {   dialog->settings->lightness_modified ++;
@@ -740,7 +742,7 @@ _color_correction_gui_lslider_events_cb (GtkWidget *widget,
       if (data->dragging)
       {   _color_correction_gui_erase_lslider_handle(widget, data);
           data->position=event->button.y;
-	  gfloat new_lightness = (1-data->position/(gfloat)data->height) -0.5;
+	  new_lightness = (1-data->position/(gfloat)data->height) -0.5;
 	  if (dialog->settings->values[data->range][LIGHTNESS] != new_lightness) 
           {   if(dialog->settings->values[data->range][LIGHTNESS] == 0)
 	      {   dialog->settings->lightness_modified ++;
@@ -843,9 +845,11 @@ _color_correction_gui_draw_wheel (GtkWidget *drawing_area, gfloat lightness)
     gfloat h,s,l;
     guchar color[3];
     _ColorCorrectionGuiWheel *data = (_ColorCorrectionGuiWheel *)gtk_object_get_data(GTK_OBJECT(drawing_area), "data");
+    int xmax, ymax;
     
     /* translate the marker to the new size */
     gint new_width, new_height;
+
     new_width = drawing_area->allocation.width;
     new_height = drawing_area->allocation.height;
     if (data->width != -1)
@@ -875,7 +879,6 @@ _color_correction_gui_draw_wheel (GtkWidget *drawing_area, gfloat lightness)
 		       TRUE, 0,0, data->width, data->height);
 
     /* iterate over wheel calculating hue and saturation */
-    int xmax, ymax;
     data->x = data->width/2-data->radius;
     data->y = data->height/2-data->radius;
     xmax = data->x + 2 * data->radius;
@@ -910,8 +913,10 @@ _color_correction_gui_draw_wheel_handle (GtkWidget *drawing_area, _ColorCorrecti
 		 10,10,0,23040); /* last two params are angle in 1/64th of degree */
     gdk_draw_point(data->display_buffer, data->gc, 
 		   data->position[X], data->position[Y]);
-    GdkRectangle update_area = {data->position[X]-5, data->position[Y]-5, 11, 11};
-    gtk_widget_draw(drawing_area, &update_area);
+    {
+      GdkRectangle update_area = {data->position[X]-5, data->position[Y]-5, 11, 11};
+      gtk_widget_draw(drawing_area, &update_area);
+    }
 }
 
 
@@ -965,9 +970,10 @@ _color_correction_gui_draw_wheel_toggle(GtkWidget *drawing_area)
 		      GTK_WIDGET(drawing_area)->style->light_gc[GTK_STATE_NORMAL], 5, 17, 
 		      color_correction_wheel_labels[data->range], 3);
     }
-
-    GdkRectangle update_area = {3,3,27,19};    
-    gtk_widget_draw(drawing_area, &update_area);
+    {
+      GdkRectangle update_area = {3,3,27,19};    
+      gtk_widget_draw(drawing_area, &update_area);
+    }
 }
 
 
@@ -1063,6 +1069,8 @@ static void
 _color_correction_gui_draw_lslider (GtkWidget *drawing_area)
 {   _ColorCorrectionGuiLSlider *data = (_ColorCorrectionGuiLSlider *)gtk_object_get_data(GTK_OBJECT(drawing_area), "data");
     gint y;
+    int step_size;
+    int gray;
     
     data->height = drawing_area->allocation.height;
 
@@ -1084,8 +1092,7 @@ _color_correction_gui_draw_lslider (GtkWidget *drawing_area)
 
     /* draw the slider */
     gdk_gc_set_function(data->gc, GDK_COPY);
-    int step_size = 255/data->height;
-    int gray;
+    step_size = 255/data->height;
     for (y = 0; y < data->height; y++)
     {   gray=255-(y*step_size);
         gdk_rgb_gc_set_foreground(data->gc, (gray<<16) | (gray<<8) | gray);
@@ -1111,9 +1118,10 @@ _color_correction_gui_draw_lslider_handle (GtkWidget *drawing_area, _ColorCorrec
     gdk_gc_set_foreground(data->gc, &drawing_area->style->dark[GTK_STATE_NORMAL]);
     gdk_draw_line(data->display_buffer, data->gc,
 		  0, data->position+2, LSLIDER_WIDTH, data->position+2);
-
-    GdkRectangle update_area = {0, data->position-2, LSLIDER_WIDTH, 5};
-    gtk_widget_draw(drawing_area, &update_area);		    
+    {
+      GdkRectangle update_area = {0, data->position-2, LSLIDER_WIDTH, 5};
+      gtk_widget_draw(drawing_area, &update_area);
+    }
 }
 
 
