@@ -40,8 +40,11 @@ void select_shrink_cmd_callback (GtkWidget *, gpointer);
 void select_by_color_cmd_callback (GtkWidget *, gpointer);
 void select_save_cmd_callback (GtkWidget *, gpointer);
 void view_colormanage_display_cmd_callback(GtkWidget *, gpointer);
+void view_proof_display_cmd_callback(GtkWidget *, gpointer);
+void view_gamutcheck_display_cmd_callback(GtkWidget *, gpointer);
 void view_rendering_intent_cmd_callback(GtkWidget *, gpointer, guint);
 void view_look_profiles_cmd_callback(GtkWidget *, gpointer);
+void view_expose_hdr_cmd_callback (GtkWidget *, gpointer);
 void view_zoomin_cmd_callback (GtkWidget *, gpointer);
 void view_zoomout_cmd_callback (GtkWidget *, gpointer);
 void view_zoom_16_1_callback (GtkWidget *, gpointer);
@@ -75,7 +78,6 @@ void image_color_balance_cmd_callback (GtkWidget *, gpointer);
 void image_color_correction_cmd_callback (GtkWidget *, gpointer);
 void image_brightness_contrast_cmd_callback (GtkWidget *, gpointer);
 void image_gamma_expose_cmd_callback (GtkWidget *, gpointer);
-void image_expose_image_cmd_callback (GtkWidget *, gpointer);
 void image_hue_saturation_cmd_callback (GtkWidget *, gpointer);
 void image_curves_cmd_callback (GtkWidget *, gpointer);
 void image_levels_cmd_callback (GtkWidget *, gpointer);
