@@ -306,6 +306,9 @@ void
 convert_image_colorspace (void *gimage_ptr, CMSProfile *new_profile,
                           int lcms_intent)
 {   GImage *image = (GImage *) gimage_ptr;
+    DWORD lcms_format_in;
+    DWORD lcms_format_out;
+    CMSTransform *transform;
 
     /*  get the profiles */
     GSList *profiles = NULL;
@@ -318,10 +321,10 @@ convert_image_colorspace (void *gimage_ptr, CMSProfile *new_profile,
     profiles = g_slist_append(profiles, (gpointer)new_profile);
 
     /* assuming all layers have homogenous alpha */
-    DWORD lcms_format_in  = cms_get_lcms_format(
+    lcms_format_in  = cms_get_lcms_format(
                               drawable_tag(GIMP_DRAWABLE(image->active_layer)),
                               gimage_get_cms_profile(image)  );
-    DWORD lcms_format_out = cms_get_lcms_format(
+    lcms_format_out = cms_get_lcms_format(
                               drawable_tag(GIMP_DRAWABLE(image->active_layer)),
                               new_profile  );
     if  (!(int)lcms_format_in || !(int)lcms_format_out)
@@ -330,7 +333,7 @@ convert_image_colorspace (void *gimage_ptr, CMSProfile *new_profile,
         return;
     } 
 
-    CMSTransform *transform = cms_get_transform(profiles, lcms_format_in, lcms_format_out, lcms_intent, cms_default_flags);
+    transform = cms_get_transform(profiles, lcms_format_in, lcms_format_out, lcms_intent, cms_default_flags);
     g_slist_free(profiles);
     gimage_transform_colors(image, transform, TRUE);
     cms_return_transform(transform);    
