@@ -654,7 +654,7 @@ crop_image (GImage *gimage,
     channel_invalidate_previews (gimage->ID);
     layer_invalidate_previews (gimage->ID);
     gimage_invalidate_preview (gimage);
-    gdisplays_update_full (gimage->ID);
+    gdisplays_update_gimage (gimage->ID);
     gdisplays_shrink_wrap (gimage->ID);
   }
 }
