@@ -856,6 +856,8 @@ display_brush_get_row_funcs (Tag t)
       return display_brush_get_row_float;
     case PRECISION_FLOAT16:
       return display_brush_get_row_float16;
+    case PRECISION_BFP:
+      return display_brush_get_row_bfp;
     case PRECISION_NONE:
     default:
       g_warning ("display_brush_get_row_funcs: bad precision");
@@ -990,6 +992,37 @@ display_brush_get_row_float16(
     }	
 }
 
+void  
+display_brush_get_row_bfp(  
+                        guchar *buf,
+			Canvas *brush_canvas,
+			gint row,
+                        gint width,
+			gint scale
+		  )
+{
+  Tag tag = canvas_tag (brush_canvas);
+  gint channels_per_row = tag_num_channels (tag) * canvas_width (brush_canvas);
+  guint16 * s = (guint16*)canvas_portion_data (brush_canvas,0,0) + row * channels_per_row;
+  guchar *b = buf; 
+  gint i;
+  Format f = tag_format (canvas_tag (brush_canvas));
+  
+  switch (f)
+    {
+    case FORMAT_GRAY:
+      /*  Invert the mask for display. */ 
+      for (i = 0; i < width; i++)
+        {
+	  *b++ = 255 - U16_TO_U8(*s);
+	   s += scale;
+        }
+      break;	
+    default:
+      break;
+    }	
+}
+
 static void
 display_setup (BrushSelectP bsp)
 {
