@@ -26,6 +26,7 @@
 #include "../drawable.h"
 #include "../errors.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../gimage_mask.h"
 #include "../layer.h"
 #include "../paint_funcs_area.h"
@@ -706,6 +707,15 @@ channel_value (Channel *mask, int x, int y)
         val = FLT (*data, u);
       }
       break;
+
+    case PRECISION_BFP:
+      {
+        guint16 * data = (guint16*) canvas_portion_data (canvas, x, y);
+        val = *data / ONE_BFP;
+      }
+      break;
+
+	default:
     case PRECISION_NONE:
       g_warning ("bad precision");
       break;
@@ -789,6 +799,7 @@ channel_bounds (Channel *mask, int *x1, int *y1, int *x2, int *y2)
 		  }
 		  break;
 
+	    case PRECISION_BFP:
 		case PRECISION_U16:
 		  {
 		    guint16 *d = (guint16*)data;
@@ -965,6 +976,7 @@ channel_is_empty (Channel *mask)
 	        }
 	}
 	break;
+	  case PRECISION_BFP:
       case PRECISION_U16:
 	{
 	  guint16* d = (guint16*) data;
@@ -1145,6 +1157,28 @@ channel_add_segment (Channel *mask, int x, int y, int width, gfloat value)
       }
       break;
 
+    case PRECISION_BFP:
+      {
+        guint16 * data;
+        int val;
+        
+        for (pr = pixelarea_register (1, &maskPR); 
+             pr != NULL; 
+             pr = pixelarea_process (pr))
+          {
+            data = (guint16*) pixelarea_data (&maskPR);
+            width = pixelarea_width (&maskPR);
+            while (width--)
+              {
+                val = *data + (value * ONE_BFP);
+                if (val > ONE_BFP)
+                  val = ONE_BFP;
+                *data++ = val;
+              }
+          }
+      }
+      break;
+
     default:
       g_warning ("channel_add_segment: bad precision");
       break;
@@ -1266,6 +1300,29 @@ channel_sub_segment (Channel *mask, int x, int y, int width, gfloat value)
           }
       }
       break;
+
+    case PRECISION_BFP:
+      {
+        guint16 * data;
+        int val;
+        
+        for (pr = pixelarea_register (1, &maskPR); 
+             pr != NULL; 
+             pr = pixelarea_process (pr))
+          {
+            data = (guint16*) pixelarea_data (&maskPR);
+            width = pixelarea_width (&maskPR);
+            while (width--)
+              {
+                val = *data - (value * ONE_BFP);
+                if (val < 0)
+                  val = 0;
+                *data++ = val;
+              }
+          }
+      }
+      break;
+
     case PRECISION_NONE:
       g_warning ("channel_sub_segment: bad precision");
       break;
@@ -1380,6 +1437,26 @@ channel_inter_segment (Channel *mask, int x, int y, int width, gfloat value)
       }
       break;
 
+    case PRECISION_BFP:
+      {
+        guint16 * data;
+        int val;
+        
+        for (pr = pixelarea_register (1, &maskPR); 
+             pr != NULL; 
+             pr = pixelarea_process (pr))
+          {
+            data = (guint16*) pixelarea_data (&maskPR);
+            width = pixelarea_width (&maskPR);
+            while (width--)
+              {
+                val = MINIMUM(*data, (value * ONE_BFP));
+                *data++ = val;
+              }
+          }
+      }
+      break;
+
     case PRECISION_NONE:
       g_warning ("channel_inter_segment: bad precision");
       break;
@@ -1849,6 +1926,19 @@ channel_sharpen (Channel *mask)
 	      }
 	  }
 	  break;
+
+	case PRECISION_BFP:
+	  {
+	    guint16 *d = (guint16*)data;
+	    while (size--)
+	      {
+		if (*d >  16384)
+		  *d++ = ONE_BFP;
+		else
+		  *d++ = 0;
+	      }
+	  }
+	  break;
 	
 	case PRECISION_NONE:
 	  g_warning ("bad precision");
