@@ -155,6 +155,9 @@ static void color_balance_row_float (PixelRow *, PixelRow *, double *, double *,
 
 static void color_balance_init_transfers_float16 (void); 
 static void color_balance_row_float16 (PixelRow *, PixelRow *, double *, double *, double *, gint);
+
+static void color_balance_init_transfers_bfp (void); 
+static void color_balance_row_bfp (PixelRow *, PixelRow *, double *, double *, double *, gint);
 /*  color balance machinery  */
 static void
 color_balance_funcs (Tag drawable_tag)
@@ -176,6 +179,10 @@ color_balance_funcs (Tag drawable_tag)
     color_balance_row = color_balance_row_float16;
     color_balance_init_transfers = color_balance_init_transfers_float16;
     break;
+  case PRECISION_BFP:
+    color_balance_row = color_balance_row_bfp;
+    color_balance_init_transfers = color_balance_init_transfers_bfp;
+    break;
   default:
     color_balance_row = NULL;
     color_balance_init_transfers = NULL;
@@ -593,6 +600,106 @@ color_balance_row_float16 (
     }
 }
 
+static void
+color_balance_row_bfp (
+			PixelRow *src_row,
+			PixelRow *dest_row,
+			double * cr,
+			double * mg,
+			double * yb,
+			gint preserve_luminosity
+		  )
+{
+  Tag src_tag = pixelrow_tag (src_row);
+  Tag dest_tag = pixelrow_tag (dest_row);
+  gint src_num_channels = tag_num_channels (src_tag);
+  gint dest_num_channels = tag_num_channels (dest_tag);
+  guint16 *s, *d;
+  int has_alpha;
+  int r, g, b;
+  int r_n, g_n, b_n;
+  double *cyan_red_transfer[3];
+  double *magenta_green_transfer[3];
+  double *yellow_blue_transfer[3];
+  gint w = pixelrow_width (src_row);
+  gfloat cr_shadows, cr_midtones, cr_highlights;
+  gfloat mg_shadows, mg_midtones, mg_highlights;
+  gfloat yb_shadows, yb_midtones, yb_highlights;
+
+  /*  Set the transfer arrays  (for speed)  */
+  cyan_red_transfer[SHADOWS] = (cr[SHADOWS] > 0) ? shadows_add_ptr : shadows_sub_ptr;
+  cyan_red_transfer[MIDTONES] = (cr[MIDTONES] > 0) ? midtones_add_ptr : midtones_sub_ptr;
+  cyan_red_transfer[HIGHLIGHTS] = (cr[HIGHLIGHTS] > 0) ? highlights_add_ptr : highlights_sub_ptr;
+  magenta_green_transfer[SHADOWS] = (mg[SHADOWS] > 0) ? shadows_add_ptr : shadows_sub_ptr;
+  magenta_green_transfer[MIDTONES] = (mg[MIDTONES] > 0) ? midtones_add_ptr : midtones_sub_ptr;
+  magenta_green_transfer[HIGHLIGHTS] = (mg[HIGHLIGHTS] > 0) ? highlights_add_ptr : highlights_sub_ptr;
+  yellow_blue_transfer[SHADOWS] = (yb[SHADOWS] > 0) ? shadows_add_ptr : shadows_sub_ptr;
+  yellow_blue_transfer[MIDTONES] = (yb[MIDTONES] > 0) ? midtones_add_ptr : midtones_sub_ptr;
+  yellow_blue_transfer[HIGHLIGHTS] = (yb[HIGHLIGHTS] > 0) ? highlights_add_ptr : highlights_sub_ptr;
+  
+  /* scale cr, mg, yb to [0-65535] */
+  cr_shadows = (cr[SHADOWS]/255.0) * 65535;
+  cr_midtones = (cr[MIDTONES]/255.0) * 65535;
+  cr_highlights = (cr[HIGHLIGHTS]/255.0) * 65535;
+  mg_shadows = (mg[SHADOWS]/255.0) * 65535;
+  mg_midtones = (mg[MIDTONES]/255.0) * 65535;
+  mg_highlights = (mg[HIGHLIGHTS]/255.0) * 65535;
+  yb_shadows = (yb[SHADOWS]/255.0) * 65535;
+  yb_midtones = (yb[MIDTONES]/255.0) * 65535;
+  yb_highlights = (yb[HIGHLIGHTS]/255.0) * 65535;
+
+  s = (guint16 *)pixelrow_data (src_row);
+  d = (guint16 *)pixelrow_data (dest_row);
+  has_alpha = tag_alpha (src_tag) == ALPHA_YES? TRUE: FALSE;
+  while (w--)
+    {
+      r = r_n = s[RED_PIX];
+      g = g_n = s[GREEN_PIX];
+      b = b_n = s[BLUE_PIX];
+
+      r_n += cr_shadows * cyan_red_transfer[SHADOWS][r_n];
+      r_n = BOUNDS (r_n, 0, 65535);
+      r_n += cr_midtones * cyan_red_transfer[MIDTONES][r_n];
+      r_n = BOUNDS (r_n, 0, 65535);
+      r_n += cr_highlights * cyan_red_transfer[HIGHLIGHTS][r_n];
+      r_n = BOUNDS (r_n, 0, 65535);
+
+      g_n += mg_shadows * magenta_green_transfer[SHADOWS][g_n];
+      g_n = BOUNDS (g_n, 0, 65535);
+      g_n += mg_midtones * magenta_green_transfer[MIDTONES][g_n];
+      g_n = BOUNDS (g_n, 0, 65535);
+      g_n += mg_highlights * magenta_green_transfer[HIGHLIGHTS][g_n];
+      g_n = BOUNDS (g_n, 0, 65535);
+
+      b_n += yb_shadows * yellow_blue_transfer[SHADOWS][b_n];
+      b_n = BOUNDS (b_n, 0, 65535);
+      b_n += yb_midtones * yellow_blue_transfer[MIDTONES][b_n];
+      b_n = BOUNDS (b_n, 0, 65535);
+      b_n += yb_highlights * yellow_blue_transfer[HIGHLIGHTS][b_n];
+      b_n = BOUNDS (b_n, 0, 65535);
+
+      if (preserve_luminosity)
+	{
+#if 0
+	  rgb_to_hls (&r, &g, &b);
+	  rgb_to_hls (&r_n, &g_n, &b_n);
+	  g_n = g;
+	  hls_to_rgb (&r_n, &g_n, &b_n);
+#endif
+	}
+
+      d[RED_PIX] = r_n;
+      d[GREEN_PIX] = g_n;
+      d[BLUE_PIX] = b_n;
+
+      if (has_alpha)
+	d[ALPHA_PIX] = s[ALPHA_PIX];
+
+      s += src_num_channels;
+      d += dest_num_channels;
+    }
+}
+
 static void 
 color_balance_allocate_transfer_arrays (gint size)
 {
@@ -668,6 +775,22 @@ color_balance_init_transfers_float16 (void)
 {
 }
 
+static void
+color_balance_init_transfers_bfp (void) 
+{
+  gdouble x;
+  gint i;
+  color_balance_allocate_transfer_arrays (65536);
+  for (i = 0; i < 65536; i++)
+    {
+      x = ((double)i/65535.0);   
+      highlights_add_ptr[i] = shadows_sub_ptr[65535 - i] =  f1(x);
+      midtones_add_ptr[i] = midtones_sub_ptr[i] = f2(x);
+      shadows_add_ptr[i] = highlights_sub_ptr[i] = f2(x);
+    }
+}
+
+
 #if 0
 /* Should this be used somewhere? --jcohen */
 static 
