@@ -142,6 +142,7 @@ color_picker_button_press (Tool           *tool,
 {
   GDisplay * gdisp;
   int x, y;
+  const char** color_space_channel_names = 0;
 
   gdisp = (GDisplay *) gdisp_ptr;
 
@@ -161,14 +162,15 @@ color_picker_button_press (Tool           *tool,
 					    "UNKNOWN");
       active_drawable = gimage_active_drawable (gdisp->gimage);
 
+      color_space_channel_names = gimage_get_channel_names(gdisp->gimage);
       /*  if the gdisplay is for a color image, the dialog must have RGB  */
       switch (prev_format=tag_format (drawable_tag (active_drawable)))
 	{
 	case FORMAT_RGB:
-	  info_dialog_add_field (color_picker_info, "Red", red_buf);
-	  info_dialog_add_field (color_picker_info, "Green", green_buf);
-	  info_dialog_add_field (color_picker_info, "Blue", blue_buf);
-	  info_dialog_add_field (color_picker_info, "Alpha", alpha_buf);
+	  info_dialog_add_field (color_picker_info, color_space_channel_names[RED_PIX], red_buf);
+	  info_dialog_add_field (color_picker_info, color_space_channel_names[GREEN_PIX], green_buf);
+	  info_dialog_add_field (color_picker_info, color_space_channel_names[BLUE_PIX], blue_buf);
+	  info_dialog_add_field (color_picker_info, color_space_channel_names[ALPHA_PIX], alpha_buf);
 	  info_dialog_add_field (color_picker_info, "Hex Triplet", hex_buf);
 	  break;
 
