@@ -39,6 +39,8 @@
 #include "../minimize.h"
 #include "../rc.h"
 
+#define _(translation_text) translation_text
+
 #define ROUND(x)  ((int) ((x) + 0.5))
 
 #define GRAPH              0x1
@@ -237,6 +239,7 @@ curves_funcs (
 	curves_convert_ui_value = curves_convert_ui_value_u8;
 	curves_calculate_free_curve = curves_calculate_free_curve_u8;
 	break;
+  case PRECISION_BFP:
   case PRECISION_U16:
 	curves = curves_u16;
 	curves_alloc_curves = curves_alloc_curves_u16;
@@ -665,7 +668,7 @@ void
 curves_initialize (void *gdisp_ptr)
 {
   GDisplay *gdisp;
-  int i, j;
+  int i, j, init = 0;
 
   gdisp = (GDisplay *) gdisp_ptr;
 
@@ -685,9 +688,10 @@ curves_initialize (void *gdisp_ptr)
   }
 
   /*  The curves dialog  */
-  if (!curves_dialog)
+  if (!curves_dialog) {
     curves_dialog = curves_new_dialog ();
-  else if (!GTK_WIDGET_VISIBLE (curves_dialog->shell))
+    init = 1;
+  } else if (!GTK_WIDGET_VISIBLE (curves_dialog->shell))
     gtk_widget_show (curves_dialog->shell);
 
   /* Look for Lab colorspace */ 
@@ -729,45 +733,75 @@ curves_initialize (void *gdisp_ptr)
   curves_dialog->color = drawable_color ( (curves_dialog->drawable));
   curves_dialog->image_map = image_map_create (gdisp_ptr, curves_dialog->drawable);
 
-  { if (curves_dialog->is_Lab) {
-      gtk_widget_hide( channel_items[0].widget);
-      gtk_widget_show( channel_items[5].widget);
-      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[1].widget)->child),
-                          "CIE Lightness");
-      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[2].widget)->child),
-                          "CIE a");
-      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[3].widget)->child),
-                          "CIE b");
-    } else {
-      gtk_widget_show( channel_items[0].widget);
-      gtk_widget_hide( channel_items[5].widget);
-      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[1].widget)->child),
-                          "Red");
-      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[2].widget)->child),
-                          "Green");
-      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[3].widget)->child),
-                          "Blue");
+  {
+    const char** color_space_channel_names = gimage_get_channel_names(gdisp->gimage);
+
+    GtkOptionMenu *option_menu = GTK_OPTION_MENU (curves_dialog->channel_menu);
+    GtkMenu *m = GTK_MENU(gtk_option_menu_get_menu(option_menu));
+    GtkWidget *w = gtk_menu_get_active(m);
+    MenuItem *items = channel_items;
+    int pos = -1;
+
+#   if 0
+    gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[1].widget)->child),
+                        color_space_channel_names[0]);
+    gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[2].widget)->child),
+                        color_space_channel_names[1]);
+    gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[3].widget)->child),
+                        color_space_channel_names[2]);
+    if (curves_dialog->is_Lab) {
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[4].widget)->child),
+                          _("Saturation"));
     }
+    else {
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[4].widget)->child),
+                          color_space_channel_names[3]);
+    }
+#   else
+
+    pos = update_color_menu_items (channel_items,
+                                   GTK_OPTION_MENU(curves_dialog->channel_menu),
+                                   color_space_channel_names);
+#   endif
+
+    if (curves_dialog->is_Lab) {
+      if (pos == 0)
+        ++pos;
+    }
+    else {
+      if (pos == 5)
+        pos = 0;
+    }
+
+    if (curves_dialog->is_Lab) {
+      gtk_widget_hide( items[0].widget);
+      gtk_widget_show( items[5].widget);
+    } else {
+      gtk_widget_show( items[0].widget);
+      gtk_widget_hide( items[5].widget);
+    }
+
+    if (!init || curves_dialog->is_Lab)
+      items[pos].callback (w, curves_dialog);
+    /* set the current selection */
+    /* the following line is causing segfault for some reason....*/
+    gtk_option_menu_set_history ( GTK_OPTION_MENU (option_menu), pos);
+
+    /* check for alpha channel */
+    if (drawable_has_alpha ( (curves_dialog->drawable)))
+      gtk_widget_show( channel_items[4].widget);
+    else 
+      gtk_widget_hide( channel_items[4].widget);
+
+    /*  hide or show the channel menu based on image type  */
+    if (curves_dialog->color)
+      for (i = 0; i < 4; i++) 
+         gtk_widget_set_sensitive( channel_items[i].widget, TRUE);
+    else 
+      for (i = 1; i < 4; i++) 
+         gtk_widget_set_sensitive( channel_items[i].widget, FALSE);
   }
   
-  /* check for alpha channel */
-  if (drawable_has_alpha ( (curves_dialog->drawable)))
-    gtk_widget_show( channel_items[4].widget);
-  else 
-    gtk_widget_hide( channel_items[4].widget);
-
-  /*  hide or show the channel menu based on image type  */
-  if (curves_dialog->color)
-    for (i = 0; i < 4; i++) 
-       gtk_widget_set_sensitive( channel_items[i].widget, TRUE);
-  else 
-    for (i = 1; i < 4; i++) 
-       gtk_widget_set_sensitive( channel_items[i].widget, FALSE);
-
-  /* set the current selection */
-  /* the following line is causing segfault for some reason....
-  gtk_option_menu_set_history ( GTK_OPTION_MENU (curves_dialog->channel_menu), 0);
-  */
 
   #ifdef USE_HISTOGRAM
   histogram_update (curves_dialog->histogram,
