@@ -47,6 +47,14 @@ void dodge_midtones_row_u16 ( PixelRow *src_row, PixelRow *dest_row,
 void dodge_shadows_row_u16 ( PixelRow *src_row, PixelRow *dest_row, 
 	gint type, gfloat exposure);
 
+//bfp
+void dodge_highlights_row_bfp ( PixelRow *src_row, PixelRow *dest_row, 
+	gint type, gfloat exposure);
+void dodge_midtones_row_bfp ( PixelRow *src_row, PixelRow *dest_row, 
+	gint type, gfloat exposure);
+void dodge_shadows_row_bfp ( PixelRow *src_row, PixelRow *dest_row, 
+	gint type, gfloat exposure);
+
 //float16
 void dodge_highlights_row_float16 ( PixelRow *src_row, PixelRow *dest_row, 
 	gint type, gfloat exposure);
@@ -553,6 +561,13 @@ dodgeburn_row_func (
                return dodge_midtones_row_float16; 
             else 
                return dodge_shadows_row_float16; 
+        case PRECISION_BFP:
+            if (mode == DODGEBURN_HIGHLIGHTS)
+               return dodge_highlights_row_bfp; 
+            else if (mode == DODGEBURN_MIDTONES)
+               return dodge_midtones_row_bfp; 
+            else 
+               return dodge_shadows_row_bfp; 
         case PRECISION_NONE:
         default:
           g_warning ("dodgeburn_row_func: bad precision");
@@ -1138,7 +1153,7 @@ void dodge_shadows_row_u8 (
   }
 }
 
-
+///////////////////////////////////////////////////////////
 // 16 bit dodge and burn code
 //
 void dodge_highlights_row_u16 (
@@ -1304,3 +1319,169 @@ void dodge_shadows_row_u16 (
 }
 
 
+///////////////////////////////////////////////////////////
+// bfp dodge and burn code
+//
+void dodge_highlights_row_bfp (
+			    PixelRow *src_row, 
+			    PixelRow *dest_row, 
+			    gint type, 
+			    gfloat exposure
+			   )
+{
+  gint    b;
+  guint32  s, t;
+  guint16 *src          = (guint16*) pixelrow_data (src_row);
+  guint16 *dest         = (guint16*) pixelrow_data (dest_row);
+  gint    num_channels  = tag_num_channels (pixelrow_tag (src_row));
+  gint    num_channels1 = tag_num_channels (pixelrow_tag (dest_row));
+  Tag     tag           = pixelrow_tag (dest_row);
+  Tag     tag2          = pixelrow_tag (src_row);
+  gint    width         = pixelrow_width (dest_row);  
+  ShortsFloat u;
+  gfloat factor = 1.0 + exposure * (.333333);
+  gint    num_color_chans;
+  gint    has_alpha = (tag_alpha(tag) == ALPHA_YES && tag_alpha(tag2) == ALPHA_YES) ? TRUE: FALSE;
+
+  if (has_alpha)
+    num_color_chans = num_channels-1;
+  else
+    num_color_chans = num_channels;
+
+#if 0
+  d_printf("num_color_chans is %d\n", num_color_chans);
+  d_printf("num_chans is %d\n", num_channels);
+#endif
+  while (width--)
+    {
+      for (b = 0; b < num_color_chans; b++)
+	{
+	  // added by IMAGEWORKS thedoug (01/02)
+          dest[b] = CLAMP(factor * src[b], 0, 65535);
+	}
+	
+      if(has_alpha){
+          dest[b] = src[b];
+      }
+
+      src += num_channels;
+      dest += num_channels1;
+    }
+}
+
+void dodge_midtones_row_bfp (
+			    PixelRow *src_row, 
+			    PixelRow *dest_row, 
+			    gint type, 
+			    gfloat exposure
+			   )
+{
+  gint    b;
+  guint16 *src          = (guint16*) pixelrow_data (src_row);
+  guint16 *dest         = (guint16*) pixelrow_data (dest_row);
+  gint    num_channels  = tag_num_channels (pixelrow_tag (src_row));
+  gint    num_channels1 = tag_num_channels (pixelrow_tag (dest_row));
+  Tag     tag           = pixelrow_tag (dest_row);
+  Tag     tag2          = pixelrow_tag (src_row);
+  gint    width         = pixelrow_width (dest_row);  
+  ShortsFloat u;
+  gfloat factor;
+  gint    num_color_chans;
+  gint    has_alpha = (tag_alpha(tag) == ALPHA_YES && tag_alpha(tag2) == ALPHA_YES) ? TRUE: FALSE;
+
+  if (has_alpha)
+    num_color_chans = num_channels-1;
+  else
+    num_color_chans = num_channels;
+
+  if (exposure < 0)
+    factor = 1.0 - exposure * (.333333);
+  else
+    factor = 1/(1.0 + exposure);
+ 
+  while (width--)
+    {
+      for (b = 0; b < num_color_chans; b++)
+	{
+	  // added by IMAGEWORKS thedoug (01/02)
+          dest[b] = CLAMP(65535 * pow(src[b]/65535., factor), 0, 65535);
+	}
+
+      if(has_alpha)
+          dest[b] = src[b];
+
+      src += num_channels;
+      dest += num_channels1;
+    }
+}
+
+void dodge_shadows_row_bfp (
+			    PixelRow *src_row, 
+			    PixelRow *dest_row, 
+			    gint type, 
+			    gfloat exposure
+			   )
+{
+  gint    b;
+  guint32  s, t;
+  guint16 *src          = (guint16*) pixelrow_data (src_row);
+  guint16 *dest         = (guint16*) pixelrow_data (dest_row);
+  gint    num_channels  = tag_num_channels (pixelrow_tag (src_row));
+  gint    num_channels1 = tag_num_channels (pixelrow_tag (dest_row));
+  Tag     tag           = pixelrow_tag (dest_row);
+  Tag     tag2          = pixelrow_tag (src_row);
+  gint    width         = pixelrow_width (dest_row);  
+  gfloat factor;
+  gint    num_color_chans;
+  gint    has_alpha = (tag_alpha(tag) == ALPHA_YES && tag_alpha(tag2) == ALPHA_YES) ? TRUE: FALSE;
+
+  if (has_alpha)
+    num_color_chans = num_channels-1;
+  else
+    num_color_chans = num_channels;
+
+  if (exposure >= 0)
+  {
+    factor = .333333 * exposure;
+
+    while (width--)
+      {
+	for (b = 0; b < num_color_chans; b++)
+	  {
+	    // added by IMAGEWORKS thedoug (01/02)
+	    dest[b] = CLAMP(65535 * (factor + src[b]/65535. - factor*src[b]/65535.), 0, 65535);
+	  }
+
+	if(has_alpha)
+	    dest[b] = src[b];
+
+	src += num_channels;
+	dest += num_channels1;
+      }
+  }
+  else   /*exposure < 0*/ 
+  {
+    factor = -.333333 * exposure;
+
+    while (width--)
+      {
+	for (b = 0; b < num_color_chans; b++)
+	  {
+ 	    if (src[b]/65535. < factor)
+	      dest[b] = 0;
+	    else if (src[b]/65535. < 1)
+	      dest[b] = CLAMP(65535 * (src[b]/65535. - factor)/(1-factor), 0, 65535); 
+	    else 
+	      dest[b] = src[b];
+	  }
+
+	if(has_alpha)
+	    dest[b] = src[b];
+
+	src += num_channels;
+	dest += num_channels1;
+      }
+  }
+}
+
+
