@@ -67,9 +67,13 @@ struct GammaExposeDialog
 
   double       gamma;
   double       expose;
+  double       old_gamma;
+  double       old_expose;
 
   gint         preview;
   gint         forceexpose;
+
+  GDisplay    *gdisp;
 };
 
 /* gamma and expose transfer luts */
@@ -84,7 +88,7 @@ static void   gamma_expose_motion         (Tool *, GdkEventMotion *, gpointer);
 static void   gamma_expose_cursor_update  (Tool *, GdkEventMotion *, gpointer);
 static void   gamma_expose_control        (Tool *, int, gpointer);
 
-static GammaExposeDialog *  gamma_expose_new_dialog  (void);
+static void   gamma_expose_new_dialog              (GammaExposeDialog *);
 static void   gamma_expose_update                  (GammaExposeDialog *, int);
 static void   gamma_expose_preview                 (GammaExposeDialog *);
 static void   gamma_expose_ok_callback             (GtkWidget *, gpointer);
@@ -239,6 +243,7 @@ gamma_expose_init_transfers_u16 (void * user_data)
 static void 
 gamma_expose_init_transfers_float (void * user_data)
 {
+#if 0
   gint i;
   GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
   gfloat *gamma_data = (gfloat*) pixelrow_data (&gamma_lut);
@@ -261,7 +266,7 @@ gamma_expose_init_transfers_float (void * user_data)
   
   pixelrow_init (&gamma_lut, lut_tag, (guchar*)gamma_data, 1); 
   pixelrow_init (&expose_lut, lut_tag, (guchar*)expose_data, 1); 
-    
+#endif  
 }
 
 static void 
@@ -292,8 +297,8 @@ gamma_expose_init_transfers_bfp (void * user_data)
   guint16 *gamma_data = (guint16*) pixelrow_data (&gamma_lut);
   guint16 *expose_data = (guint16*) pixelrow_data (&expose_lut);
   Tag lut_tag = tag_new (PRECISION_BFP, FORMAT_GRAY, ALPHA_NO);
-  gdouble gamma = bcd->gamma;
-  gdouble expose = bcd->expose;
+  gfloat gamma = bcd->gamma;
+  gfloat expose = bcd->expose;
  
   /* allocate memory for the transfers if we dont have it yet */
   if (!gamma_data && !expose_data)
@@ -328,8 +333,8 @@ gamma_expose_u8 (PixelArea *src_area,
   int alpha;
   int w, h, b;
   GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
-  gdouble gamma = (gfloat)bcd->gamma;
-  gdouble expose = (gfloat)bcd->expose;
+  gfloat gamma = 1.f/(gfloat)bcd->gamma;
+  gfloat expose = pow(2.0,(gfloat)bcd->expose);
 
   src = (guchar *)pixelarea_data (src_area);
   dest = (guchar *)pixelarea_data (dest_area);
@@ -346,8 +351,8 @@ gamma_expose_u8 (PixelArea *src_area,
 	{
 	  // Added by IMAGEWORKS thedoug (01/02)
 	  for (b = 0; b < alpha; b++)
-	    d[b] = CLAMP(255 * expose_func(gamma_func(s[b]/255., 
-		   2.*gamma), 2.*expose), 0, 255);
+	    d[b] = CLAMP(255 * pow(s[b]/255.*expose, 
+		   gamma), 0, 255);
 
 	  if (has_alpha)
 	    d[alpha] = s[alpha];
@@ -376,8 +381,8 @@ gamma_expose_u16 (PixelArea *src_area,
   int alpha;
   int w, h, b;
   GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
-  gdouble gamma = (gfloat)bcd->gamma;
-  gdouble expose = (gfloat)bcd->expose;
+  gfloat gamma = 1.f/(gfloat)bcd->gamma;
+  gfloat expose = pow(2.0,(gfloat)bcd->expose);
 
   src = (guchar *)pixelarea_data (src_area);
   dest = (guchar *)pixelarea_data (dest_area);
@@ -394,8 +399,8 @@ gamma_expose_u16 (PixelArea *src_area,
 	{
 	  // Added by IMAGEWORKS thedoug (01/02)
 	  for (b = 0; b < alpha; b++)
-	    d[b] = CLAMP(65535 * expose_func(gamma_func(s[b]/65535., 
-	           2.*gamma), 2.*expose), 0, 65535);
+	    d[b] = CLAMP(65535 * pow(s[b]/65535.*expose, 
+	           gamma), 0, 65535);
 
 	  if (has_alpha)
 	    d[alpha] = s[alpha];
@@ -424,8 +429,8 @@ gamma_expose_float (PixelArea *src_area,
   int alpha;
   int w, h, b;
   GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
-  gdouble gamma = (gfloat)bcd->gamma;
-  gdouble expose = (gfloat)bcd->expose;
+  gfloat gamma = 1.f/(gfloat)bcd->gamma;
+  gfloat expose = pow(2.0,(gfloat)bcd->expose);
 
   src = (guchar *)pixelarea_data (src_area);
   dest = (guchar *)pixelarea_data (dest_area);
@@ -441,8 +446,9 @@ gamma_expose_float (PixelArea *src_area,
       while (w--)
 	{
 	  for (b = 0; b < alpha; b++) {
-          d[b] = expose_func (gamma_func(s[b], gamma), expose);
-          if (bcd->forceexpose) {
+          d[b] = pow(s[b] * expose, gamma);
+          //expose_func (gamma_func(s[b], gamma), expose);
+          if (bcd->forceexpose) { /* clamp */
               if (d[b] > 1.0f) d[b] = 1.0f;
               else if (d[b] < 0.0f) d[b] = 0.0f;
           }
@@ -475,8 +481,8 @@ gamma_expose_float16 (PixelArea *src_area,
   int alpha;
   int w, h, b;
   GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
-  gdouble gamma = (gfloat)bcd->gamma;
-  gdouble expose = (gfloat)bcd->expose;
+  gfloat gamma = 1.f/(gfloat)bcd->gamma;
+  gfloat expose = pow(2.0,(gfloat)bcd->expose);
   ShortsFloat u;
   gfloat sb, db;
 
@@ -496,7 +502,7 @@ gamma_expose_float16 (PixelArea *src_area,
 	  for (b = 0; b < alpha; b++)
 	  {
 	    sb = FLT (s[b], u);
-	    db = expose_func (gamma_func(sb, 2.0*gamma), 2.0*expose);
+	    db = pow (sb*expose, gamma);
 	    d[b] = FLT16 (db, u);
 	  }
 
@@ -527,8 +533,8 @@ gamma_expose_bfp (PixelArea *src_area,
   int alpha;
   int w, h, b;
   GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
-  gdouble gamma = (gfloat)bcd->gamma;
-  gdouble expose = (gfloat)bcd->expose;
+  gfloat gamma = 1.f/(gfloat)bcd->gamma;
+  gfloat expose = pow(2.0,(gfloat)bcd->expose);
 
   src = (guchar *)pixelarea_data (src_area);
   dest = (guchar *)pixelarea_data (dest_area);
@@ -545,8 +551,8 @@ gamma_expose_bfp (PixelArea *src_area,
 	{
 	  // Added by IMAGEWORKS thedoug (01/02)
 	  for (b = 0; b < alpha; b++)
-	    d[b] = CLAMP(65535 * expose_func(gamma_func(s[b]/65535., 
-	           2.*gamma), 2.*expose), 0, 65535);
+	    d[b] = CLAMP(32768.f * pow(s[b]/32768.f*expose, 
+	           gamma), 0, 65535);
 
 	  if (has_alpha)
 	    d[alpha] = s[alpha];
@@ -672,6 +678,7 @@ gamma_expose_initialize (void *gdisp_ptr)
 {
   GDisplay *gdisp;
   CanvasDrawable *drawable;
+  int init_dialog = gamma_expose_dialog ? 0:1;
 
   gdisp = (GDisplay *) gdisp_ptr;
 
@@ -681,13 +688,6 @@ gamma_expose_initialize (void *gdisp_ptr)
       return;
     }
 
-  /*  The gamma-expose dialog  */
-  if (!gamma_expose_dialog)
-    gamma_expose_dialog = gamma_expose_new_dialog ();
-  else
-    if (!GTK_WIDGET_VISIBLE (gamma_expose_dialog->shell))
-      gtk_widget_show (gamma_expose_dialog->shell);
-
   drawable = gimage_active_drawable (gdisp->gimage);
   
   /* Set up the function pointers for our data type */
@@ -696,16 +696,40 @@ gamma_expose_initialize (void *gdisp_ptr)
   pixelrow_init (&gamma_lut, tag_null(), NULL , 0); 
   pixelrow_init (&expose_lut, tag_null(), NULL, 0); 
   
-  /*  Initialize dialog fields  */
-  gamma_expose_dialog->image_map = NULL;
-  gamma_expose_dialog->gamma = 1.0;
-  gamma_expose_dialog->expose = 0.0;
+
+  /*  The gamma-expose dialog  */
+  if (init_dialog)
+  { gamma_expose_dialog = g_malloc_zero (sizeof (GammaExposeDialog));
+    /*  Initialize dialog fields  */
+    gamma_expose_dialog->preview = TRUE;
+    gamma_expose_dialog->forceexpose = FALSE;
+    gamma_expose_dialog->image_map = NULL;
+    gamma_expose_dialog->gdisp = gdisp;
+  }
+  else
+  {
+    gamma_expose_dialog->gdisp = gdisp;
+
+    if (!GTK_WIDGET_VISIBLE (gamma_expose_dialog->shell))
+      gtk_widget_show (gamma_expose_dialog->shell);
+  }
+
+    gamma_expose_dialog->gamma = gamma_expose_dialog->old_gamma = gdisp->gamma;
+    gamma_expose_dialog->expose = gamma_expose_dialog->old_expose = gdisp->expose;
+    gdisp->gamma = 1.0;
+    gdisp->expose = 0.0;
 
   gamma_expose_dialog->drawable = drawable;
   gamma_expose_dialog->image_map = image_map_create (gdisp_ptr,
 							    gamma_expose_dialog->drawable);
 
+  if (init_dialog)
+    gamma_expose_new_dialog (gamma_expose_dialog);
+
   gamma_expose_update (gamma_expose_dialog, ALL);
+
+  if (gamma_expose_dialog->preview)
+    gamma_expose_preview (gamma_expose_dialog);
 }
 
 
@@ -720,10 +744,9 @@ static ActionAreaItem action_items[] =
   { "Cancel", gamma_expose_cancel_callback, NULL, NULL }
 };
 
-static GammaExposeDialog *
-gamma_expose_new_dialog ()
+static void
+gamma_expose_new_dialog (GammaExposeDialog * bcd)
 {
-  GammaExposeDialog *bcd;
   GtkWidget *vbox;
   GtkWidget *hbox;
   GtkWidget *table;
@@ -733,10 +756,6 @@ gamma_expose_new_dialog ()
   GtkWidget *forceexpose;
   GtkObject *data;
 
-  bcd = g_malloc_zero (sizeof (GammaExposeDialog));
-  bcd->preview = TRUE;
-  bcd->forceexpose = TRUE;
-
   /*  The shell and main vbox  */
   bcd->shell = gtk_dialog_new ();
   gtk_window_set_wmclass (GTK_WINDOW (bcd->shell), "gamma_expose",PROGRAM_NAME);
@@ -755,14 +774,14 @@ gamma_expose_new_dialog ()
   table = gtk_table_new (2, 3, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 
-  /*  Create the gamma scale widget  */
-  label = gtk_label_new ("gamma");
+  /*  Create the expose scale widget  */
+  label = gtk_label_new ("expose");
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
 		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
 
-  data = gtk_adjustment_new (2.2, -10, 10, .001, .01, 0.0);
-  bcd->gamma_data = GTK_ADJUSTMENT (data);
+  data = gtk_adjustment_new (0, -20, 20, .01, .01, 0.0);
+  bcd->expose_data = GTK_ADJUSTMENT (data);
   slider = gtk_hscale_new (GTK_ADJUSTMENT (data));
   gtk_widget_set_usize (slider, SLIDER_WIDTH, SLIDER_HEIGHT);
   gtk_scale_set_digits (GTK_SCALE (slider), 3);
@@ -772,47 +791,13 @@ gamma_expose_new_dialog ()
 		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 		    2, 2);
-  gtk_signal_connect (GTK_OBJECT (data), "value_changed",
-		      (GtkSignalFunc) gamma_expose_gamma_scale_update,
-		      bcd);
-
-  bcd->gamma_text = gtk_entry_new ();
-  gtk_widget_set_usize (bcd->gamma_text, TEXT_WIDTH, TEXT_HEIGHT);
-  gtk_table_attach (GTK_TABLE (table), bcd->gamma_text, 2, 3, 0, 1,
-		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
-  gtk_signal_connect (GTK_OBJECT (bcd->gamma_text), "changed",
-		      (GtkSignalFunc) gamma_expose_gamma_text_update,
-		      bcd);
-
-  gtk_widget_show (label);
-  gtk_widget_show (bcd->gamma_text);
-  gtk_widget_show (slider);
-
-
-  /*  Create the expose scale widget  */
-  label = gtk_label_new ("expose");
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
-		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
-
-  data = gtk_adjustment_new (0, -10, 10, .01, .01, 0.0);
-  bcd->expose_data = GTK_ADJUSTMENT (data);
-  slider = gtk_hscale_new (GTK_ADJUSTMENT (data));
-  gtk_widget_set_usize (slider, SLIDER_WIDTH, SLIDER_HEIGHT);
-  gtk_scale_set_digits (GTK_SCALE (slider), 3);
-  gtk_scale_set_value_pos (GTK_SCALE (slider), GTK_POS_TOP);
-  gtk_range_set_update_policy (GTK_RANGE (slider), GTK_UPDATE_DELAYED);
-  gtk_table_attach (GTK_TABLE (table), slider, 1, 2, 1, 2,
-		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
-		    2, 2);
   gtk_signal_connect (GTK_OBJECT (data), "value_changed",
 		      (GtkSignalFunc) gamma_expose_expose_scale_update,
 		      bcd);
 
   bcd->expose_text = gtk_entry_new ();
   gtk_widget_set_usize (bcd->expose_text, TEXT_WIDTH, TEXT_HEIGHT);
-  gtk_table_attach (GTK_TABLE (table), bcd->expose_text, 2, 3, 1, 2,
+  gtk_table_attach (GTK_TABLE (table), bcd->expose_text, 2, 3, 0, 1,
 		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
   gtk_signal_connect (GTK_OBJECT (bcd->expose_text), "changed",
 		      (GtkSignalFunc) gamma_expose_expose_text_update,
@@ -823,25 +808,59 @@ gamma_expose_new_dialog ()
   gtk_widget_show (slider);
 
 
+  /*  Create the gamma scale widget  */
+  label = gtk_label_new ("gamma");
+  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
+		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
+
+  data = gtk_adjustment_new (2.2, 0, 10, .001, .01, 0.0);
+  bcd->gamma_data = GTK_ADJUSTMENT (data);
+  slider = gtk_hscale_new (GTK_ADJUSTMENT (data));
+  gtk_widget_set_usize (slider, SLIDER_WIDTH, SLIDER_HEIGHT);
+  gtk_scale_set_digits (GTK_SCALE (slider), 3);
+  gtk_scale_set_value_pos (GTK_SCALE (slider), GTK_POS_TOP);
+  gtk_range_set_update_policy (GTK_RANGE (slider), GTK_UPDATE_DELAYED);
+  gtk_table_attach (GTK_TABLE (table), slider, 1, 2, 1, 2,
+		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+		    2, 2);
+  gtk_signal_connect (GTK_OBJECT (data), "value_changed",
+		      (GtkSignalFunc) gamma_expose_gamma_scale_update,
+		      bcd);
+
+  bcd->gamma_text = gtk_entry_new ();
+  gtk_widget_set_usize (bcd->gamma_text, TEXT_WIDTH, TEXT_HEIGHT);
+  gtk_table_attach (GTK_TABLE (table), bcd->gamma_text, 2, 3, 1, 2,
+		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
+  gtk_signal_connect (GTK_OBJECT (bcd->gamma_text), "changed",
+		      (GtkSignalFunc) gamma_expose_gamma_text_update,
+		      bcd);
+
+  gtk_widget_show (label);
+  gtk_widget_show (bcd->gamma_text);
+  gtk_widget_show (slider);
+
+
   /*  Horizontal box for preview and preserve luminosity toggle buttons  */
   hbox = gtk_hbox_new (TRUE, 2);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
   /*  The preview toggle  */
   toggle = gtk_check_button_new_with_label ("Preview");
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle), bcd->preview);
   gtk_box_pack_start (GTK_BOX (hbox), toggle, TRUE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
 		      (GtkSignalFunc) gamma_expose_preview_update,
 		      bcd);
+  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle), bcd->preview);
 
   gtk_widget_show (label);
   gtk_widget_show (toggle);
   gtk_widget_show (hbox);
 
   /*  The expose toggle  */
-  forceexpose = gtk_check_button_new_with_label ("ForceExpose");
-  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (toggle), bcd->forceexpose);
+  forceexpose = gtk_check_button_new_with_label ("Clamp");
+  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (forceexpose), bcd->forceexpose);
   gtk_box_pack_start (GTK_BOX (hbox), forceexpose, TRUE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (forceexpose), "toggled",
 		      (GtkSignalFunc) gamma_expose_force_update,
@@ -859,8 +878,6 @@ gamma_expose_new_dialog ()
   gtk_widget_show (table);
   gtk_widget_show (vbox);
   gtk_widget_show (bcd->shell);
-
-  return bcd;
 }
 
 static void
@@ -958,8 +975,10 @@ gamma_expose_cancel_callback (GtkWidget *widget,
       image_map_abort (bcd->image_map);
       gamma_expose_free_transfers();
       active_tool->preserve = FALSE;
-      gdisplays_flush ();
     }
+  bcd->gdisp->gamma = bcd->old_gamma;
+  bcd->gdisp->expose = bcd->old_expose;
+  gdisplays_flush ();
 
   bcd->image_map = NULL;
 }
@@ -1022,9 +1041,9 @@ gamma_expose_expose_scale_update (GtkAdjustment *adjustment,
 					   gpointer       data)
 {
   GammaExposeDialog *bcd;
-  int i;
+
   bcd = (GammaExposeDialog *) data;
-  i=4;
+
   if (bcd->expose != adjustment->value)
     {
       bcd->expose = adjustment->value;
