@@ -338,10 +338,10 @@ histogram_update (Histogram         *histogram,
   
     /*  hide or show the channel menu based on image type  */
     if (histogram_p->color)
-      for (i = 0; i < 4; i++) 
+      for (i = 0; i < 5; ++i) 
 	gtk_widget_set_sensitive( histogram->color_option_items[i].widget, TRUE);
     else 
-      for (i = 1; i < 4; i++) 
+      for (i = 1; i < 5; ++i) 
 	gtk_widget_set_sensitive( histogram->color_option_items[i].widget, FALSE);
 
     /* set the current selection */
@@ -480,7 +480,7 @@ histogram_histogram_funcs (Tag tag)
 /*  histogram machinery  */
 
 static void
-histogram_histogram_info_u8 (PixelArea       *src_area,
+histogram_histogram_info_u8 (  PixelArea       *src_area,
 			       PixelArea       *mask_area,
 			       HistogramValues  values,
 			       void            *user_data)
@@ -489,11 +489,11 @@ histogram_histogram_info_u8 (PixelArea       *src_area,
   int                  w, h;
   guchar *src, *mask = NULL;
   guint8 *s, *m = NULL;
-  gint    value, red, green, blue;
+  gint    value, red, green, blue, alpha=128;
   gfloat  scale = 1./255.;
   Tag                  src_tag = pixelarea_tag (src_area);
   gint                 s_num_channels = tag_num_channels (src_tag);
-  gint                 alpha = tag_alpha (src_tag);
+  gint    has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   Tag                  mask_tag = pixelarea_tag (mask_area);
   gint                 m_num_channels = tag_num_channels (mask_tag);
   HistogramPrivate *histogram_p;
@@ -523,6 +523,8 @@ histogram_histogram_info_u8 (PixelArea       *src_area,
 	       red   = s[RED_PIX];
 	       green = s[GREEN_PIX];
 	       blue  = s[BLUE_PIX];
+               if(has_alpha)
+                   alpha = s[ALPHA_PIX];
 
 	       if (mask_area)
 		 {
@@ -530,6 +532,8 @@ histogram_histogram_info_u8 (PixelArea       *src_area,
 		   values[HISTOGRAM_RED][red]     += (double) *m * scale;
 		   values[HISTOGRAM_GREEN][green] += (double) *m * scale;
 		   values[HISTOGRAM_BLUE][blue]   += (double) *m * scale;
+	           if(has_alpha)
+        	       values[HISTOGRAM_ALPHA][alpha]   += (double) *m * scale;
 		 }
 	       else
 		 {
@@ -537,6 +541,8 @@ histogram_histogram_info_u8 (PixelArea       *src_area,
 		   values[HISTOGRAM_RED][red]     += 1.0;
 		   values[HISTOGRAM_GREEN][green] += 1.0;
 		   values[HISTOGRAM_BLUE][blue]   += 1.0;
+	           if(has_alpha)
+        	       values[HISTOGRAM_ALPHA][alpha]   += 1.0;
 		 }
 	     }
 	   else
@@ -562,7 +568,7 @@ histogram_histogram_info_u8 (PixelArea       *src_area,
 }
 
 static void
-histogram_histogram_info_u16 (PixelArea       *src_area,
+histogram_histogram_info_u16 ( PixelArea       *src_area,
 			       PixelArea       *mask_area,
 			       HistogramValues  values,
 			       void            *user_data)
@@ -572,14 +578,14 @@ histogram_histogram_info_u16 (PixelArea       *src_area,
   int                  w, h;
   guchar *src, *mask = NULL;
   guint16 *s, *m = NULL;
-  gint    value, red, green, blue;
+  gint    value, red, green, blue, alpha=0;
   gfloat  scale = 1./65535.;
   Tag                  src_tag = pixelarea_tag (src_area);
   gint                 s_num_channels = tag_num_channels (src_tag);
-  gint                 alpha = tag_alpha (src_tag);
+  gint    has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   Tag                  mask_tag = pixelarea_tag (mask_area);
   gint                 m_num_channels = tag_num_channels (mask_tag);
-  gint 		       value_bin=0, red_bin, green_bin, blue_bin;
+  gint 		       value_bin=0, red_bin, green_bin, blue_bin, alpha_bin=0;
   HistogramPrivate *histogram_p = (HistogramPrivate *) htd->private_part;
   bins = histogram_p->bins;
   h = pixelarea_height (src_area);
@@ -605,6 +611,8 @@ histogram_histogram_info_u16 (PixelArea       *src_area,
 	       red   = s[RED_PIX];
 	       green = s[GREEN_PIX];
 	       blue  = s[BLUE_PIX];
+               if(has_alpha)
+                   alpha = s[ALPHA_PIX];
 
 	       /* figure out which bin to place it in */ 
   
@@ -612,6 +620,8 @@ histogram_histogram_info_u16 (PixelArea       *src_area,
 	       red_bin = bins*(double)red*scale;   	
 	       green_bin = bins*(double)green*scale;
 	       blue_bin = bins*(double)blue*scale;
+               if(has_alpha)
+                   alpha_bin = bins*(double)alpha*scale;
 
 	       if (mask_area)
 		 {
@@ -619,6 +629,8 @@ histogram_histogram_info_u16 (PixelArea       *src_area,
 		   values[HISTOGRAM_RED][red_bin]     += (double) *m * scale;
 		   values[HISTOGRAM_GREEN][green_bin] += (double) *m * scale;
 		   values[HISTOGRAM_BLUE][blue_bin]   += (double) *m * scale;
+	           if(has_alpha)
+        	       values[HISTOGRAM_ALPHA][alpha_bin]   += (double) *m * scale;
 		 }
 	       else
 		 {
@@ -626,6 +638,8 @@ histogram_histogram_info_u16 (PixelArea       *src_area,
 		   values[HISTOGRAM_RED][red_bin]     += 1.0;
 		   values[HISTOGRAM_GREEN][green_bin] += 1.0;
 		   values[HISTOGRAM_BLUE][blue_bin]   += 1.0;
+                   if(has_alpha)
+                       values[HISTOGRAM_ALPHA][alpha_bin]   += 1.0;
 		 }
 	     }
 	   else
@@ -662,13 +676,13 @@ histogram_histogram_info_float (PixelArea *src_area,
   int                  w, h;
   guchar *src, *mask = NULL;
   gfloat *s, *m = NULL;
-  gfloat    value, red, green, blue;
+  gfloat    value, red, green, blue, alpha=0;
   Tag                  src_tag = pixelarea_tag (src_area);
   gint                 s_num_channels = tag_num_channels (src_tag);
-  gint                 alpha = tag_alpha (src_tag);
+  gint    has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   Tag                  mask_tag = pixelarea_tag (mask_area);
   gint                 m_num_channels = tag_num_channels (mask_tag);
-  gint value_bin=0, red_bin, green_bin, blue_bin;
+  gint value_bin=0, red_bin, green_bin, blue_bin, alpha_bin=0;
   HistogramPrivate *histogram_p = (HistogramPrivate *) htd->private_part;
 
   bins = histogram_p->bins;
@@ -693,11 +707,15 @@ histogram_histogram_info_float (PixelArea *src_area,
 	       red   = s[RED_PIX];
 	       green = s[GREEN_PIX];
 	       blue  = s[BLUE_PIX];
-		
+               if(has_alpha)
+                   alpha  = s[ALPHA_PIX];
+#if 1
 	       red = CLAMP (red, 0.0, 1.0);
 	       green = CLAMP (green, 0.0, 1.0);
 	       blue = CLAMP (blue, 0.0, 1.0);
-
+               if(has_alpha)
+                   alpha = CLAMP (alpha, 0.0, 1.0);
+#endif
 	       value = MAX (red, green);
 	       value = MAX (value, blue);
 
@@ -705,6 +723,8 @@ histogram_histogram_info_float (PixelArea *src_area,
 	       red_bin = (int)(red * (bins-1));   	
 	       green_bin = (int)(green * (bins-1));
 	       blue_bin = (int)(blue * (bins-1));
+               if(has_alpha)
+                   alpha_bin = (int)(alpha * (bins-1));
 
 	       if (mask_area)
 		 {
@@ -712,6 +732,8 @@ histogram_histogram_info_float (PixelArea *src_area,
 		   values[HISTOGRAM_RED][red_bin] += (double) *m;
 		   values[HISTOGRAM_GREEN][green_bin] += (double) *m;
 		   values[HISTOGRAM_BLUE][blue_bin]   += (double) *m;
+	           if(has_alpha)
+	               values[HISTOGRAM_ALPHA][alpha_bin]   += (double) *m;
 		 }
 	       else
 		 {
@@ -719,6 +741,8 @@ histogram_histogram_info_float (PixelArea *src_area,
 		   values[HISTOGRAM_RED][red_bin]     += 1.0;
 		   values[HISTOGRAM_GREEN][green_bin] += 1.0;
 		   values[HISTOGRAM_BLUE][blue_bin]   += 1.0;
+                   if(has_alpha)
+	               values[HISTOGRAM_ALPHA][alpha_bin]   += 1.0;
 		 }
 	     }
 	   else
@@ -755,13 +779,13 @@ histogram_histogram_info_float16 (PixelArea *src_area,
   int                  w, h;
   guchar *src, *mask = NULL;
   guint16 *s, *m = NULL;
-  gfloat    value, red, green, blue;
+  gfloat    value, red, green, blue, alpha=0;
   Tag                  src_tag = pixelarea_tag (src_area);
   gint                 s_num_channels = tag_num_channels (src_tag);
-  gint                 alpha = tag_alpha (src_tag);
+  gint    has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   Tag                  mask_tag = pixelarea_tag (mask_area);
   gint                 m_num_channels = tag_num_channels (mask_tag);
-  gint value_bin = 0, red_bin, green_bin, blue_bin;
+  gint value_bin = 0, red_bin, green_bin, blue_bin, alpha_bin=0;
   ShortsFloat u;
   gfloat m_val;
   HistogramPrivate *histogram_p;
@@ -789,10 +813,14 @@ histogram_histogram_info_float16 (PixelArea *src_area,
 	       red   = FLT (s[RED_PIX], u);
 	       green = FLT (s[GREEN_PIX], u);
 	       blue  = FLT (s[BLUE_PIX], u);
+               if(has_alpha)
+	           alpha  = FLT (s[ALPHA_PIX], u);
 #if 1
 	       red = CLAMP (red, -0.01, 1.01);
 	       green = CLAMP (green, -0.01, 1.01);
 	       blue = CLAMP (blue, -0.01, 1.01);
+               if(has_alpha)
+	           alpha = CLAMP (alpha, -0.01, 1.01);
 #endif
 	       value = MAX (red, green);
 	       value = MAX (value, blue);
@@ -801,6 +829,8 @@ histogram_histogram_info_float16 (PixelArea *src_area,
 	       red_bin = (int)((red) * 100 + 1);   	
 	       green_bin = (int)((green) * 100 + 1);
 	       blue_bin = (int)((blue) * 100 + 1);
+               if(has_alpha)
+                   alpha_bin = (int)((alpha) * 100 + 1);
 
 	       if (mask_area)
 		 {
@@ -809,6 +839,8 @@ histogram_histogram_info_float16 (PixelArea *src_area,
 		   values[HISTOGRAM_RED][red_bin] += (double)m_val;
 		   values[HISTOGRAM_GREEN][green_bin] += (double)m_val;
 		   values[HISTOGRAM_BLUE][blue_bin]   += (double)m_val;
+                   if(has_alpha)
+                       values[HISTOGRAM_ALPHA][alpha_bin]   += (double)m_val;
 		 }
 	       else
 		 {
@@ -817,6 +849,8 @@ histogram_histogram_info_float16 (PixelArea *src_area,
 		   values[HISTOGRAM_RED][red_bin]     += 1.0;
 		   values[HISTOGRAM_GREEN][green_bin] += 1.0;
 		   values[HISTOGRAM_BLUE][blue_bin]   += 1.0;
+                   if(has_alpha)
+                       values[HISTOGRAM_ALPHA][alpha_bin]   += 1.0;
 		 }
 	     }
 	   else
@@ -853,14 +887,14 @@ histogram_histogram_info_bfp (PixelArea       *src_area,
   int                  w, h;
   guchar *src, *mask = NULL;
   guint16 *s, *m = NULL;
-  gint    value, red, green, blue;
+  gint    value, red, green, blue, alpha=0;
   gfloat  scale = 1./65535.;
   Tag                  src_tag = pixelarea_tag (src_area);
   gint                 s_num_channels = tag_num_channels (src_tag);
-  gint                 alpha = tag_alpha (src_tag);
+  gint    has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   Tag                  mask_tag = pixelarea_tag (mask_area);
   gint                 m_num_channels = tag_num_channels (mask_tag);
-  gint 		       value_bin=0, red_bin, green_bin, blue_bin;
+  gint 		       value_bin=0, red_bin, green_bin, blue_bin, alpha_bin=0;
   HistogramPrivate *histogram_p = (HistogramPrivate *) htd->private_part;
   bins = histogram_p->bins;
   h = pixelarea_height (src_area);
@@ -886,6 +920,7 @@ histogram_histogram_info_bfp (PixelArea       *src_area,
 	       red   = s[RED_PIX];
 	       green = s[GREEN_PIX];
 	       blue  = s[BLUE_PIX];
+	       alpha  = s[ALPHA_PIX];
 
 	       /* figure out which bin to place it in */ 
   
@@ -893,6 +928,8 @@ histogram_histogram_info_bfp (PixelArea       *src_area,
 	       red_bin = bins*(double)red*scale;   	
 	       green_bin = bins*(double)green*scale;
 	       blue_bin = bins*(double)blue*scale;
+               if(has_alpha)
+	           alpha_bin = bins*(double)alpha*scale;
 
 	       if (mask_area)
 		 {
@@ -900,6 +937,8 @@ histogram_histogram_info_bfp (PixelArea       *src_area,
 		   values[HISTOGRAM_RED][red_bin]     += (double) *m * scale;
 		   values[HISTOGRAM_GREEN][green_bin] += (double) *m * scale;
 		   values[HISTOGRAM_BLUE][blue_bin]   += (double) *m * scale;
+                   if(has_alpha)
+		       values[HISTOGRAM_ALPHA][alpha_bin]   += (double) *m * scale;
 		 }
 	       else
 		 {
@@ -907,6 +946,8 @@ histogram_histogram_info_bfp (PixelArea       *src_area,
 		   values[HISTOGRAM_RED][red_bin]     += 1.0;
 		   values[HISTOGRAM_GREEN][green_bin] += 1.0;
 		   values[HISTOGRAM_BLUE][blue_bin]   += 1.0;
+                   if(has_alpha)
+		       values[HISTOGRAM_ALPHA][alpha_bin]   += 1.0;
 		 }
 	     }
 	   else
