@@ -151,6 +151,10 @@ static void hue_saturation_float16       (PixelArea *, PixelArea *, void *);
 static void hue_saturation_calculate_transfers_float16 (gint, gint, gint);
 static void hue_saturation_alloc_transfers_float16 (void);
 
+static void hue_saturation_bfp       (PixelArea *, PixelArea *, void *);
+static void hue_saturation_calculate_transfers_bfp (gint, gint, gint);
+static void hue_saturation_alloc_transfers_bfp (void);
+
 static void hue_saturation_init_transfers (void *);
 static void hue_saturation_free_transfers (void);
 
@@ -205,6 +209,11 @@ hue_saturation_funcs (Tag drawable_tag)
     hue_saturation_calculate_transfers = hue_saturation_calculate_transfers_float16;
     hue_saturation = hue_saturation_float16;
     break;
+  case PRECISION_BFP:
+    hue_saturation_alloc_transfers = hue_saturation_alloc_transfers_bfp;
+    hue_saturation_calculate_transfers = hue_saturation_calculate_transfers_bfp;
+    hue_saturation = hue_saturation_bfp;
+    break;
   default:
     hue_saturation_alloc_transfers = NULL;
     hue_saturation_calculate_transfers = NULL;
@@ -276,6 +285,32 @@ hue_saturation_alloc_transfers_float16 (void)
 {
 }
 
+static void
+hue_saturation_alloc_transfers_bfp (void)
+{
+  gint i;
+  Tag tag = tag_new (PRECISION_BFP, FORMAT_GRAY, ALPHA_NO);
+  
+  for (i = 0; i < 6; i++)
+  { 
+       guint16* hue_data = (guint16*) g_malloc (sizeof(guint16) * 65536 );
+       guint16* lightness_data = (guint16*) g_malloc (sizeof(guint16) * 65536 );
+       guint16* saturation_data = (guint16*) g_malloc (sizeof(guint16) * 65536 );
+       pixelrow_init (&hue_trans[i], tag, (guchar*)hue_data, 65536);
+       pixelrow_init (&lightness_trans[i], tag, (guchar*)lightness_data, 65536);
+       pixelrow_init (&saturation_trans[i], tag, (guchar*)saturation_data, 65536);
+  }
+  
+  /* the values for the transfer arrays */ 
+  for (i = 0; i < 7; i++)
+  {
+    hue[i] = 0.0;
+    lightness[i] = 0.0;
+    saturation[i] = 0.0;
+  }
+  hue_saturation_calculate_transfers_u16 ( TRUE, TRUE, TRUE);
+}
+
 static void
 hue_saturation_free_transfers (void)
 {
@@ -471,6 +506,52 @@ hue_saturation_calculate_transfers_float16 (
 {
 }
 
+static void
+hue_saturation_calculate_transfers_bfp (
+					gint do_hue,
+					gint do_lightness,
+					gint do_saturation
+					)
+{
+  int i, j;
+  
+  /* hue transfer */
+  if (do_hue)
+    for (j = 0; j < 6; j++)
+    {
+      guint16* hue_trans_data = (guint16*)pixelrow_data (&hue_trans[j]);
+      for (i = 0; i < 65536; i++)
+	{
+            hue_trans_data[i] = (guint16)(65535.0 * hue_value ((gdouble)i/65535.0, 
+					hue[0], hue[j + 1]) + .5);
+	}
+    }
+ 
+  /* lightness transfer */ 
+  if (do_lightness)
+    for (j = 0; j < 6; j++)
+    {
+      guint16* lightness_trans_data = (guint16*)pixelrow_data (&lightness_trans[j]);
+      for (i = 0; i < 65536; i++)
+	{
+           lightness_trans_data[i] = (guint16)( 65535.0 * lightness_value ((gdouble)i/65535.0, 
+					lightness[0], lightness[j + 1]) + .5);
+	}
+    }
+  
+  /*  saturation  transfer */
+  if (do_saturation)
+    for (j = 0; j < 6; j++)
+    {
+      guint16* saturation_trans_data = (guint16*)pixelrow_data (&saturation_trans[j]);
+      for (i = 0; i < 65536; i++)
+	{
+          saturation_trans_data[i] = (guint16)( 65535.0 * saturation_value ((gdouble)i/65535.0, 
+						saturation[0], saturation[j + 1]) + .5);
+	}
+    }
+}
+
 
 static void
 hue_saturation_u8 (PixelArea *src_area,
@@ -748,6 +829,79 @@ hue_saturation_float16 (PixelArea *src_area,
     }
 }
 
+static void
+hue_saturation_bfp (PixelArea *src_area,
+		PixelArea *dest_area,
+		void        *user_data)
+{
+  Tag src_tag = pixelarea_tag (src_area);
+  Tag dest_tag = pixelarea_tag (dest_area);
+  gint src_num_channels = tag_num_channels (src_tag);
+  gint dest_num_channels = tag_num_channels (dest_tag);
+  HueSaturationDialog *hsd;
+  guchar *src, *dest;
+  guint16 *s, *d;
+  int has_alpha;
+  int w, h;
+  gdouble r, g, b;
+  gint j;
+  guint16* hue_trans_data[6];
+  guint16* lightness_trans_data[6];
+  guint16* saturation_trans_data[6];
+  hsd = (HueSaturationDialog *) user_data;
+
+  /* get pointers to the transfer tables for speed */ 
+  for (j = 0 ; j < 6; j++)
+  { 
+    hue_trans_data[j] = (guint16*)pixelrow_data (&hue_trans[j]);
+    lightness_trans_data[j] = (guint16*)pixelrow_data (&lightness_trans[j]);
+    saturation_trans_data[j] = (guint16*)pixelrow_data (&saturation_trans[j]);
+  } 
+  
+  src = (guchar *)pixelarea_data (src_area);
+  dest = (guchar *)pixelarea_data (dest_area);
+  has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE : FALSE;
+  h = pixelarea_height (src_area);
+  while (h--)
+    {
+      s = (guint16*)src;
+      d = (guint16*)dest;
+      w = pixelarea_width (src_area);
+      while (w--)
+	{
+	  r = s[RED_PIX]/65535.0;
+	  g = s[GREEN_PIX]/65535.0;
+	  b = s[BLUE_PIX]/65535.0;
+
+	  calc_rgb_to_hls (&r, &g, &b);
+
+          r /= 360.0;
+          j = (int)(r * 6.0);
+	  j = BOUNDS (j, 0 , 5);
+
+	  r = hue_trans_data[j][(guint16)(r * 65535.0)] / 65535.0;
+	  g = lightness_trans_data[j][(guint16)(g * 65535.0)] / 65535.0;
+	  b = saturation_trans_data[j][(guint16)(b * 65535.0)] / 65535.0;
+           
+          r *= 360;
+	  calc_hls_to_rgb (&r, &g, &b);
+
+	  d[RED_PIX] = r * 65535.0;
+	  d[GREEN_PIX] = g * 65535.0;
+	  d[BLUE_PIX] = b * 65535.0;
+
+	  if (has_alpha)
+	    d[ALPHA_PIX] = s[ALPHA_PIX];
+
+	  s += src_num_channels;
+	  d += dest_num_channels;
+	}
+
+      src += pixelarea_rowstride (src_area);
+      dest += pixelarea_rowstride (dest_area);
+    }
+}
+
 
 static void
 hue_saturation_button_press (Tool           *tool,
