@@ -39,9 +39,6 @@
 #include "../trace.h"
 #include "displaylut.h"
 
-static double gamma_func (double, double);
-static double expose_func (double, double);
-
 #define MAX_CHANNELS 4
 
 #define ALL_INTEN_VISIBLE 0
@@ -83,6 +80,7 @@ struct RenderInfo
   GSList *channels;
   Channel *single_visible_channel;
   int aux_channels_visible;  
+  CMSTransform *transform_buffer;
 };
 
 
@@ -106,45 +104,27 @@ static guchar  check_combos[6][2] =
   { 0, 0 }
 };
 
-static gfloat gamma_value = 1.0;
-static gfloat expose_value = 0.0;
 static guint force_expose = 0;
-static guint force_clamp = 0;
+//static guint force_clamp = 0;
 
-float image_render_get_gamma()
-{
-    return (gamma_value);
-}
-float image_render_get_expose()
-{
-    return (expose_value);
-}
 float image_render_get_force_expose()
 {
     return (force_expose);
 }
-float image_render_get_clamp()
+/*float image_render_get_clamp()
 {
     return (force_clamp);
-}
+}*/
 
 
-void image_render_set_gamma(gfloat val)
-{
-    gamma_value = val;
-}
-void image_render_set_expose(gfloat val)
-{
-    expose_value = val;
-}
 void image_render_set_force_expose(guint val)
 {
     force_expose = val;
 }
-void image_render_set_force_clamp(guint val)
+/*void image_render_set_force_clamp(guint val)
 {
     force_clamp = val;
-}
+}*/
 
 void
 render_setup (int check_type,
@@ -393,6 +373,9 @@ render_image (GDisplay *gdisp,
         break;
       }
   }
+
+  if(info.transform_buffer)
+    cms_return_transform(info.transform_buffer);
 }
 
 
@@ -846,6 +829,7 @@ render_image_init_info (RenderInfo *info,
   gint num_visible_aux_chans; 
   Channel *single_visible_channel = NULL;
   GSList * list;
+  CMSTransform *         transform_buffer=NULL;
 
   info->gdisp = gdisp;
   info->x = x + gdisp->offset_x;
@@ -858,7 +842,11 @@ render_image_init_info (RenderInfo *info,
   info->src_y = UNSCALE (gdisp, info->y);
   info->src_w = UNSCALE (gdisp, (info->x + info->w)) - UNSCALE (gdisp, info->x) + 1;
 
+  #if 0
   info->src_canvas = gdisplay_get_projection (gdisp);
+  #else
+  info->src_canvas = gimage_projection (gdisp->gimage);
+  #endif
   info->channels = gimage_channels (gdisp->gimage);
   info->src_width = canvas_width (info->src_canvas);
   src_canvas_tag = canvas_tag (info->src_canvas);
@@ -926,6 +914,59 @@ render_image_init_info (RenderInfo *info,
 
   if (tag_alpha (canvas_tag (info->src_canvas)) == ALPHA_YES)
     info->alpha = render_image_init_alpha (gimage_projection_opacity (gdisp->gimage));
+
+  /* if colormanaged, get the required transform */
+
+  if (info->gdisp->colormanaged)
+  {   /* combine profiles */
+      GSList *profiles = NULL;
+      CMSProfile *image_profile = gimage_get_cms_profile(info->gdisp->gimage);
+      CMSProfile *proof_profile = NULL;
+      CMSProfile *display_profile = cms_get_display_profile();
+      if (gdisplay_is_proofed(info->gdisp))
+      {
+          proof_profile = gimage_get_cms_proof_profile(info->gdisp->gimage);
+      }
+      if (image_profile == NULL)
+      {   g_warning("gdisplay_project: cannot colourmanage, image has no profile assigned");
+          transform_buffer = NULL;
+      }
+      else if (display_profile == NULL)
+      {   g_warning("gdisplay_project: cannot colourmanage, no display profile set");
+          transform_buffer = NULL;
+      }
+      else
+      {   profiles = g_slist_append(profiles, (gpointer)image_profile);
+          if(info->gdisp->look_profile_pipe)
+              profiles = g_slist_concat(profiles, g_slist_copy(info->gdisp->look_profile_pipe));
+          if (proof_profile)
+              profiles = g_slist_append(profiles, (gpointer)proof_profile);
+          profiles = g_slist_append(profiles, (gpointer)display_profile);
+
+          transform_buffer = cms_get_canvas_transform(profiles,
+                               info->src_canvas,
+	                           info->gdisp->cms_intent, info->gdisp->cms_flags);
+          g_slist_free(profiles);
+      }
+  }
+  /* if we have got look profiles ... */
+  else if (!info->gdisp->colormanaged && (info->gdisp->look_profile_pipe != NULL))
+  {   GSList *profiles = NULL;
+      CMSProfile *image_profile = gimage_get_cms_profile(info->gdisp->gimage);
+      if (image_profile == NULL)
+      {   image_profile = cms_get_srgb_profile();
+      }
+      profiles = g_slist_append(profiles, (gpointer)image_profile);
+      profiles = g_slist_concat(profiles, g_slist_copy(info->gdisp->look_profile_pipe));
+      profiles = g_slist_append(profiles, (gpointer)image_profile);
+      transform_buffer = cms_get_canvas_transform(profiles,
+                               info->src_canvas, 
+	                           info->gdisp->cms_intent, info->gdisp->cms_flags);
+      g_slist_free(profiles);
+  }
+
+  info->transform_buffer = transform_buffer;
+  
 }
 
 static guint*
@@ -978,26 +1019,25 @@ render_image_accelerate_scaling (int width,
 void
 set_visible_channels_row_u8(
              guchar * buffer,
-	     int num_pixels,
-	     int *visible,
-	     Tag tag
+             int num_pixels,
+             int *vmask,
+             int *smask,
+	         int num_channels,
+             int n,
+             int is_lab
              )
 {
   gint    b;
   guint8 *data         = (guint8*) buffer;
-  gint    num_channels = tag_num_channels (tag);
-  gint    n;
-  Alpha   alpha = tag_alpha (tag);
+  gint    dark_color = 0;
 
-  if (alpha == ALPHA_YES) 
-    n = num_channels - 1; 
-  else
-    n = num_channels;
+  if (is_lab)
+      dark_color = 128;
 
   while (num_pixels--)
     {
       for (b = 0; b < n; b++)
-        data[b] =  visible[b] ? data[b] : 0;
+        data[b] =  vmask[b] ? data[smask[b]] : dark_color;
       data += num_channels;
     }
 }
@@ -1005,26 +1045,25 @@ set_visible_channels_row_u8(
 void
 set_visible_channels_row_u16(
              guchar * buffer,
-	     int num_pixels,
-	     int *visible,
-	     Tag tag
+             int num_pixels,
+             int *vmask,
+             int *smask,
+	         int num_channels,
+             int n,
+             int is_lab
              )
 {
   gint    b;
   guint16 *data         = (guint16*) buffer;
-  gint    num_channels = tag_num_channels (tag);
-  gint    n;
-  Alpha   alpha = tag_alpha (tag);
+  gint    dark_color = 0;
 
-  if (alpha == ALPHA_YES) 
-    n = num_channels - 1; 
-  else
-    n = num_channels;
+  if (is_lab)
+      dark_color = 32768;
 
   while (num_pixels--)
     {
       for (b = 0; b < n; b++)
-        data[b] =  visible[b] ? data[b] : 0.0;
+        data[b] =  vmask[b] ? data[smask[b]] : dark_color;
       data += num_channels;
     }
 }
@@ -1032,26 +1071,25 @@ set_visible_channels_row_u16(
 void
 set_visible_channels_row_float(
              guchar * buffer,
-	     int num_pixels,
-	     int *visible,
-	     Tag tag
+             int num_pixels,
+             int *vmask,
+             int *smask,
+	         int num_channels,
+             int n,
+             int is_lab
              )
 {
   gint    b;
   gfloat *data         = (gfloat*) buffer;
-  gint    num_channels = tag_num_channels (tag);
-  gint    n;
-  Alpha   alpha = tag_alpha (tag);
+  gfloat  dark_color = 0.0;
 
-  if (alpha == ALPHA_YES) 
-    n = num_channels - 1; 
-  else
-    n = num_channels;
+  if (is_lab)
+      dark_color = 0.5;
 
   while (num_pixels--)
     {
       for (b = 0; b < n; b++)
-        data[b] =  visible[b] ? data[b] : ZERO_FLOAT16;
+        data[b] =  vmask[b] ? data[smask[b]] : dark_color;
       data += num_channels;
     }
 }
@@ -1059,26 +1097,21 @@ set_visible_channels_row_float(
 void
 set_visible_channels_row_float16(
              guchar * buffer,
-	     int num_pixels,
-	     int *visible,
-	     Tag tag
+             int num_pixels,
+             int *vmask,
+             int *smask,
+	         int num_channels,
+             int n,
+             int is_lab
              )
 {
   gint    b;
   guint16 *data         = (guint16*) buffer;
-  gint    num_channels = tag_num_channels (tag);
-  gint    n;
-  Alpha   alpha = tag_alpha (tag);
-
-  if (alpha == ALPHA_YES) 
-    n = num_channels - 1; 
-  else
-    n = num_channels;
 
   while (num_pixels--)
     {
       for (b = 0; b < n; b++)
-        data[b] =  visible[b] ? data[b] : ZERO_FLOAT16;
+        data[b] =  vmask[b] ? data[smask[b]] : ZERO_FLOAT16;
       data += num_channels;
     }
 }
@@ -1086,32 +1119,27 @@ set_visible_channels_row_float16(
 void
 set_visible_channels_row_bfp(
              guchar * buffer,
-	     int num_pixels,
-	     int *visible,
-	     Tag tag
+             int num_pixels,
+             int *vmask,
+             int *smask,
+	         int num_channels,
+             int n,
+             int is_lab
              )
 {
   gint    b;
   guint16 *data         = (guint16*) buffer;
-  gint    num_channels = tag_num_channels (tag);
-  gint    n;
-  Alpha   alpha = tag_alpha (tag);
-
-  if (alpha == ALPHA_YES) 
-    n = num_channels - 1; 
-  else
-    n = num_channels;
 
   while (num_pixels--)
     {
       for (b = 0; b < n; b++)
-        data[b] =  visible[b] ? data[b] : 0;
+        data[b] =  vmask[b] ? data[smask[b]] : 0;
       data += num_channels;
     }
 }
  
 
-typedef void (*SetVisibleChannelsFunc) (guchar*, int, int*, Tag);
+typedef void (*SetVisibleChannelsFunc) (guchar*, int, int*, int*, int,int, int);
 static SetVisibleChannelsFunc 
 set_visible_channels_func (
                    Tag tag
@@ -1150,6 +1178,17 @@ image_render_set_visible_channels(GImage *gimage,
     int num_inten = tag_num_channels(projection_tag);
     int visible[MAX_CHANNELS];
     SetVisibleChannelsFunc func=set_visible_channels_func(projection_tag);
+    gint is_lab = 0, is_cmyk = 0;
+    gint  smask[4] = {0,1,2,3},
+          vmask[4] = {0,1,2,3};
+
+    if (gimage->cms_profile)
+    {   if (strstr("Lab",cms_get_profile_info(gimage->cms_profile)->color_space_name))
+            is_lab = 1;
+        else
+        if (strstr("Cmyk",cms_get_profile_info(gimage->cms_profile)->color_space_name))
+            is_cmyk = 1;
+    }
 
     if (tag_alpha (projection_tag) == ALPHA_YES )
     {   num_inten--;
@@ -1159,6 +1198,30 @@ image_render_set_visible_channels(GImage *gimage,
     {   visible[i] = gimage_get_component_visible (gimage, i); 
     }	
    
+    if (is_lab &&
+      num_inten == 3 &&
+      !visible[0] && 
+      (visible[1] + visible[2] == 1) )
+    {
+      smask[0] = visible[1] ? 1 : 2;
+      vmask[0] = 1;
+      vmask[1] = vmask[2] = 0;
+    }
+    else
+    if (!is_lab && !is_cmyk &&
+      num_inten == 3 &&
+      (visible[0] + visible[1] + visible[2]) == 1 )
+    { for(i = 0; i < num_inten; ++i)
+        {
+          vmask[i] = 1;
+          smask[i] = visible[0] ? 0 : (visible[1] ? 1 : 2);
+        }
+    }
+    else
+    { for(i = 0; i < num_inten; ++i)
+          vmask[i] = visible[i];
+    }
+
     pixelarea_init (&projectionPA, projection, x, y, w, h, TRUE);
     for (pag = pixelarea_register (1, &projectionPA);
 	 pag != NULL;
@@ -1172,7 +1235,8 @@ image_render_set_visible_channels(GImage *gimage,
         {   pixelarea_getdata(&projectionPA, &row_buffer, h);
    	    data = pixelrow_data(&row_buffer);
 	    num_pixels = pixelrow_width(&row_buffer);
-	    (*func)(data,num_pixels,visible,projection_tag);	    
+	    (*func)(data,num_pixels,vmask,smask,
+                tag_num_channels (projection_tag),num_inten,is_lab);
 	}
     }
 }
@@ -1204,6 +1268,9 @@ image_render_set_visible_channels(GImage *gimage,
 
 
 #if 1
+/* TODO make the function smaller and put the expose and
+        ICC code somewhere else
+ */
 static guchar*
 render_image_tile_fault (RenderInfo *info)
 {
@@ -1217,6 +1284,21 @@ render_image_tile_fault (RenderInfo *info)
   int step;
   int x;
   int src_bpp;
+  int bps = info->src_bpp/info->src_num_channels;
+  int dest_sample_width = info->w*info->src_num_channels;
+  Tag tag = canvas_tag (info->src_canvas);
+  int has_alpha = tag_alpha (tag) == ALPHA_YES ? 1:0;
+  int i;
+  float   *f;
+  guint16 *u16;
+  guint8  *u8;
+  float    sb, db;
+  ShortsFloat u;
+
+  float offset = pow(2.0,info->gdisp->offset )-1;
+  float gamma = 1.0/info->gdisp->gamma;
+  float expose = pow(2.0,info->gdisp->expose);
+
   /* the first portion x and y */
   x_portion = info->src_x;
   y_portion = info->src_y;
@@ -1256,7 +1338,72 @@ render_image_tile_fault (RenderInfo *info)
 	    {
 	      canvas_portion_unref (info->src_canvas, x_portion, y_portion);
               if (x >= CAST(int) info->src_width)
-		return tile_buf;
+              { f = (float*)tile_buf;
+                u16 = (guint16*)tile_buf;
+                u8 = (guint8*)tile_buf;
+                if(info->gdisp->expose != EXPOSE_DEFAULT || info->gdisp->offset != OFFSET_DEFAULT || info->gdisp->gamma != GAMMA_DEFAULT)
+                {
+                  switch (tag_precision(tag))
+                  {
+                  case PRECISION_FLOAT:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *f = pow((*f-offset) * expose, gamma);
+                      ++f;
+                    }
+                    break;
+                  case PRECISION_FLOAT16:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                      {
+                        sb = FLT (*u16, u);
+                        db = pow (sb*expose, gamma);
+                        *u16 = FLT16 (db, u);
+                      }
+                      ++u16;
+                    }
+                    break;
+                  case PRECISION_BFP:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *u16 = CLAMP(32768.f * pow(*u16/32768.f*expose, 
+                                     gamma), 0, 65535);
+                      ++u16;
+                    }
+                    break;
+                  case PRECISION_U8:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *u8 =  CLAMP(255.f * pow(*u8/255.f * expose, gamma),
+                                     0, 255);
+                      ++u8;
+                    }
+                    break;
+                  case PRECISION_U16:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *u16 = CLAMP(65535.f * pow(*u16/65535.f * expose,gamma),
+                                     0, 65535);
+                      ++u16;
+                    }
+                    break;
+                  case PRECISION_NONE:
+                  default:
+                    break;
+                  }
+                }
+                   
+                if(info->transform_buffer)
+                  cms_transform_buffer(info->transform_buffer,
+                                 tile_buf, tile_buf, info->w,
+                                 tag_precision(gimage_tag(info->gdisp->gimage)) );
+                return tile_buf;
+              }
 	      x_portion += portion_width;
               canvas_portion_refro (info->src_canvas, x_portion, y_portion ); 
               data = canvas_portion_data (info->src_canvas, x_portion, y_portion );
@@ -1268,9 +1415,75 @@ render_image_tile_fault (RenderInfo *info)
 	      portion_width = canvas_portion_width ( info->src_canvas, 
                                               x_portion,
                                               y_portion );
-	    }
-	}
+            }
+        }
     }
+
+  if(info->gdisp->gamma != 1.0 || info->gdisp->expose != 0.0)
+    {
+                f = (float*)tile_buf;
+                u16 = (guint16*)tile_buf;
+                u8 = (guint8*)tile_buf;
+
+                  switch (tag_precision(tag))
+                  {
+                  case PRECISION_FLOAT:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *f = pow((*f-offset) * expose, gamma);
+                      ++f;
+                    }
+                    break;
+                  case PRECISION_FLOAT16:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                      {
+                        sb = FLT (*u16, u);
+                        db = pow (sb*expose, gamma);
+                        *u16 = FLT16 (db, u);
+                      }
+                      ++u16;
+                    }
+                    break;
+                  case PRECISION_BFP:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *u16 = CLAMP(32768.f * pow(*u16/32768.f*expose, 
+                                     gamma), 0, 65535);
+                      ++u16;
+                    }
+                    break;
+                  case PRECISION_U8:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *u8 =  CLAMP(255.f * pow(*u8/255.f * expose, gamma),
+                                     0, 255);
+                      ++u8;
+                    }
+                    break;
+                  case PRECISION_U16:
+                    for(i = 0; i < dest_sample_width; ++i)
+                    { if ((has_alpha && (i+1)%info->src_num_channels) ||
+                          !has_alpha)
+                        *u16 = CLAMP(65535.f * pow(*u16/65535.f * expose,gamma),
+                                     0, 65535);
+                      ++u16;
+                    }
+                    break;
+                  case PRECISION_NONE:
+                  default:
+                    break;
+                  }
+    }
+
+  if(info->transform_buffer)
+     cms_transform_buffer(info->transform_buffer, tile_buf, tile_buf, info->w,
+                          tag_precision(gimage_tag(info->gdisp->gimage)) );
+
   canvas_portion_unref (info->src_canvas, x_portion, y_portion);
   return tile_buf;
 }
@@ -1704,7 +1917,6 @@ render_image_rgb_float (RenderInfo *info)
   int initial;
   float error;
   float step;
-  float value;
 
   y = info->y;
   ye = info->y + info->h;
@@ -1734,20 +1946,9 @@ render_image_rgb_float (RenderInfo *info)
               dest[1] = (guint8) (src[GREEN_PIX] * 255);
               dest[2] = (guint8) (src[BLUE_PIX] * 255);
 #else
-        if(force_expose){
-          value = (gfloat) expose_func (gamma_func ( src[0], gamma_value) , expose_value);
-	  dest[0] = float_to_8bit(value);
-          
-          value = (gfloat) expose_func (gamma_func ( src[1], gamma_value) , expose_value);
-	  dest[1] = float_to_8bit(value);
-
-          value = (gfloat) expose_func (gamma_func ( src[2], gamma_value) , expose_value);
-	  dest[2] = float_to_8bit(value);
-        }else{
 	    dest[0] = float_to_8bit(src[0]);
 	    dest[1] = float_to_8bit(src[1]);
 	    dest[2] = float_to_8bit(src[2]);
-        }
 #endif 
 	      src += 3;
 	      dest += 3;
@@ -1768,16 +1969,6 @@ render_image_rgb_float (RenderInfo *info)
     }
 }
 
-static double gamma_func (double x, double b)
-{
-    return pow(x, 1 / b);
-}
-
-static double expose_func (double x, double c)
-{
-    return x * pow(2.0f, c);
-}
-
 
 static void
 render_image_rgb_a_float (RenderInfo *info)
