@@ -33,6 +33,7 @@
 #include "../tag.h"
 
 #define MAX_BUF 256
+#define SHOW_ALPHA 1
 
 typedef struct InfoWinData InfoWinData;
 struct InfoWinData
@@ -214,7 +215,7 @@ info_window_update_xy (InfoDialog *info_win,
   guchar *channel_data; 
   Alpha alpha;
   Tag tag;
-  ShortsFloat u, v, s;
+  ShortsFloat u, v, s, t;
   gint offset_x, offset_y;
   Channel *channel = NULL;
   Channel *active_channel = gimage_get_active_channel(gdisp->gimage);
@@ -289,7 +290,7 @@ info_window_update_xy (InfoDialog *info_win,
       {
 	guint8* data = (guint8*)d;
 	if (format == FORMAT_RGB)
-#if 0
+#if (SHOW_ALPHA == 1)
 	  if (alpha == ALPHA_YES)
 	    sprintf (iwd->xy_color_values_str, "%3d, %3d, %3d, %3d",
 		 (int)data[0], (int)data[1], (int)data[2], (int)data[3]);
@@ -298,7 +299,7 @@ info_window_update_xy (InfoDialog *info_win,
 	    sprintf (iwd->xy_color_values_str, "%3d, %3d, %3d",
 		 (int)data[0], (int)data[1], (int)data[2]);
 	else if (format == FORMAT_GRAY)
-#if 0
+#if (SHOW_ALPHA == 1)
 	  if (alpha == ALPHA_YES)
 	    sprintf (iwd->xy_color_values_str, "%3d, %3d",
 		 (int)data[0], (int)data[1]);
@@ -316,22 +317,22 @@ info_window_update_xy (InfoDialog *info_win,
       {
 	guint16* data = (guint16*)d;
 	if (format == FORMAT_RGB)
-#if 0
+#if (SHOW_ALPHA == 1)
 	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%d, %d, %d, %d",
+	    sprintf (iwd->xy_color_values_str, "%5d, %5d, %5d, %5d",
 		 (int)data[0], (int)data[1], (int)data[2], (int)data[3]);
 	  else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%d, %d, %d",
+	    sprintf (iwd->xy_color_values_str, "%5d, %5d, %5d",
 		 (int)data[0], (int)data[1], (int)data[2]);
 	else if (format == FORMAT_GRAY)
-#if 0
+#if (SHOW_ALPHA == 1)
 	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%d, %d",
+	    sprintf (iwd->xy_color_values_str, "%5d, %5d",
 		 (int)data[0], (int)data[1]);
 	  else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%d",
+	    sprintf (iwd->xy_color_values_str, "%5d",
 		 (int)data[0]);
 	else if (format == FORMAT_INDEXED)
       iwd->xy_color_values_str[0] = '\0'; //truncate string
@@ -341,77 +342,103 @@ info_window_update_xy (InfoDialog *info_win,
  
     case PRECISION_FLOAT:
       {
-	gfloat* data = (gfloat*)d;
-	if (format == FORMAT_RGB)
-#if 0
-	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%f, %f, %f, %f",
-		 data[0], data[1], data[2], data[3]);
-	  else
+        gfloat* data = (gfloat*)d;
+        if (format == FORMAT_RGB)
+        {
+#if (SHOW_ALPHA == 1)
+          if (alpha == ALPHA_YES)
+          {
+            sprintf (iwd->xy_color_values_str, "%10f, %10f, %10f, %10f",
+            data[0], data[1], data[2], data[3]);
+          } else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%f, %f, %f",
-		 data[0], data[1], data[2]);
-	else if (format == FORMAT_GRAY)
-#if 0
-	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%f, %f",
-		 data[0], data[1]);
-	  else
+          {
+            sprintf (iwd->xy_color_values_str, "%10f, %10f, %10f",
+            data[0], data[1], data[2]);
+          }
+        } else if (format == FORMAT_GRAY)
+        {
+#if (SHOW_ALPHA == 1)
+          if (alpha == ALPHA_YES)
+          {
+            sprintf (iwd->xy_color_values_str, "%10f, %10f",
+            data[0], data[1]);
+          } else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%f",
-		 data[0]);
+          {
+            sprintf (iwd->xy_color_values_str, "%10f",
+            data[0]);
+          }
+        }
       }
       break;
 
     case PRECISION_FLOAT16:
       {
-	guint16* data = (guint16*)d;
-	if (format == FORMAT_RGB)
-#if 0
-	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%f, %f, %f, %f",
-		 FLT (data[0],u), FLT (data[1],v), FLT (data[2],s), FLT (data[3],t) );
-	  else
+        guint16* data = (guint16*)d;
+        if (format == FORMAT_RGB)
+        {
+#if (SHOW_ALPHA == 1)
+          if (alpha == ALPHA_YES)
+          {
+            sprintf (iwd->xy_color_values_str, "%10f, %10f, %10f, %10f",
+            FLT(data[0],u), FLT (data[1],v), FLT (data[2],s), FLT (data[3],t) );
+          } else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%f, %f, %f",
-		 FLT (data[0], u), FLT (data[1], v), FLT (data[2], s));
-	else if (format == FORMAT_GRAY)
-#if 0
-	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%f, %f",
-		 FLT (data[0], u), FLT (data[1], s));
-	  else
+          {
+            sprintf (iwd->xy_color_values_str, "%10f, %10f, %10f",
+        	 FLT (data[0], u), FLT (data[1], v), FLT (data[2], s));
+          }
+        } else if (format == FORMAT_GRAY)
+        {
+#if (SHOW_ALPHA == 1)
+          if (alpha == ALPHA_YES)
+          {
+            sprintf (iwd->xy_color_values_str, "%10f, %10f",
+        	 FLT (data[0], u), FLT (data[1], s));
+          } else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%f",
-		 FLT (data[0], u));
+          {
+            sprintf (iwd->xy_color_values_str, "%10f",
+        	 FLT (data[0], u));
+          }
+        }
       }
       break;
 
     case PRECISION_BFP:
       {
-	guint16* data = (guint16*)d;
-	if (format == FORMAT_RGB)
-#if 0
-	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%f, %f, %f, %f",
-		 BFP_TO_FLOAT(data[0]), BFP_TO_FLOAT(data[1]), BFP_TO_FLOAT(data[2]), BFP_TO_FLOAT(data[3]));
-	  else
+        guint16* data = (guint16*)d;
+        if (format == FORMAT_RGB)
+        {
+#if (SHOW_ALPHA == 1)
+          if (alpha == ALPHA_YES)
+          {
+            sprintf (iwd->xy_color_values_str, "%9f, %9f, %9f, %9f",
+            BFP_TO_FLOAT(data[0]), BFP_TO_FLOAT(data[1]), BFP_TO_FLOAT(data[2]), BFP_TO_FLOAT(data[3]));
+          } else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%f, %f, %f",
-		 BFP_TO_FLOAT(data[0]), BFP_TO_FLOAT(data[1]), BFP_TO_FLOAT(data[2]));
-	else if (format == FORMAT_GRAY)
-#if 0
-	  if (alpha == ALPHA_YES)
-	    sprintf (iwd->xy_color_values_str, "%f, %f",
-		 BFP_TO_FLOAT(data[0]), BFP_TO_FLOAT(data[1]));
-	  else
+          {
+            sprintf (iwd->xy_color_values_str, "%9f, %9f, %9f",
+        	BFP_TO_FLOAT(data[0]), BFP_TO_FLOAT(data[1]), BFP_TO_FLOAT(data[2]));
+          }
+        } else if (format == FORMAT_GRAY)
+        {
+#if (SHOW_ALPHA == 1)
+          if (alpha == ALPHA_YES)
+          {
+            sprintf (iwd->xy_color_values_str, "%9f, %9f",
+            BFP_TO_FLOAT(data[0]), BFP_TO_FLOAT(data[1]));
+          } else
 #endif
-	    sprintf (iwd->xy_color_values_str, "%f",
-		 BFP_TO_FLOAT(data[0]));
-	else if (format == FORMAT_INDEXED)
-      iwd->xy_color_values_str[0] = '\0'; //truncate string
+          {
+            sprintf (iwd->xy_color_values_str, "%9f",
+            BFP_TO_FLOAT(data[0]));
+          }
+        } else if (format == FORMAT_INDEXED)
+          iwd->xy_color_values_str[0] = '\0'; //truncate string
 	   //sprintf (iwd->xy_color_values_str, "");
-     }
+      }
       break;
 
     case PRECISION_NONE:
