@@ -197,6 +197,7 @@ layer_new  (
             )
 {
   Layer * layer;
+  SplineTool *st;
 
   if (width == 0 || height == 0) {
     g_message ("Zero width or height layers not allowed.");
@@ -233,7 +234,6 @@ layer_new  (
   layer->fs.num_segs = 0;
 
   /* spline layer variables */
-  SplineTool *st;
   layer->sl.splines=(DL_list*)malloc(sizeof(DL_list));
   DL_init((DL_node*)layer->sl.splines);
   st=(SplineTool*)malloc(sizeof(SplineTool));
@@ -809,7 +809,12 @@ layer_scale (layer, new_width, new_height, local_origin)
                   FALSE);
 
   /*  Allocate the new layer, configure dest region  */
-  new_canvas = canvas_new (layer_tag, new_width, new_height, STORAGE_FLAT);
+  new_canvas = canvas_new (layer_tag, new_width, new_height,
+#ifdef NO_TILES						  
+						  STORAGE_FLAT);
+#else
+						  STORAGE_TILED);
+#endif
   pixelarea_init (&destPR, new_canvas, 0, 0, new_width, new_height, TRUE);
 
   /*  Scale the layer -
@@ -822,9 +827,6 @@ layer_scale (layer, new_width, new_height, local_origin)
   else
     scale_area (&srcPR, &destPR);
 
-  /*  Push the layer on the undo stack  */
-  undo_push_layer_mod (gimage_get_ID (GIMP_DRAWABLE(layer)->gimage_ID), layer);
-
   /*  Configure the new layer  */
   if (local_origin)
     {
@@ -846,6 +848,10 @@ layer_scale (layer, new_width, new_height, local_origin)
       GIMP_DRAWABLE(layer)->offset_x = (int) (xrat * GIMP_DRAWABLE(layer)->offset_x);
       GIMP_DRAWABLE(layer)->offset_y = (int) (yrat * GIMP_DRAWABLE(layer)->offset_y);
     }
+
+  /*  Push the layer on the undo stack  */
+  undo_push_layer_mod (gimage_get_ID (GIMP_DRAWABLE(layer)->gimage_ID), layer);
+
   GIMP_DRAWABLE(layer)->tiles = new_canvas;
 
   /*  If there is a layer mask, make sure it gets scaled also  */
@@ -917,7 +923,12 @@ layer_resize (layer, new_width, new_height, offx, offy)
 	x1, y1, w, h, FALSE);
 
   /*  Allocate the new layer, configure dest region  */
-  new_canvas = canvas_new (layer_tag, new_width, new_height, STORAGE_FLAT);
+  new_canvas = canvas_new (layer_tag, new_width, new_height,
+#ifdef NO_TILES						  
+						  STORAGE_FLAT);
+#else
+						  STORAGE_TILED);
+#endif
   pixelarea_init (&destPR, new_canvas, 0, 0, new_width, new_height, TRUE);
 
   /*  fill with the fill color  */
