@@ -28,6 +28,7 @@
 #include "../drawable.h"
 #include "../errors.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../gdisplay.h"
 #include "../gimage.h"
 #include "../gimage_mask.h"
@@ -47,10 +48,12 @@
 Canvas *  canvas_16_to_16                (Canvas *); /*IMAGEWORKS aland*/
 Canvas *  canvas_16_to_8                 (Canvas *); /*IMAGEWORKS aland*/
 Canvas *  canvas_16_to_float             (Canvas *); /*IMAGEWORKS thedoug*/
+Canvas *  canvas_16_to_bfp               (Canvas *); 
 
 
 /*  global variables--for use in the various paint tools  */
 PaintCore16  non_gui_paint_core_16;
+PaintCore16 spline_paint_core;
 
 /*  local function prototypes  */
 static void      paint_core_16_button_press    (Tool *, GdkEventButton *, gpointer);
@@ -90,9 +93,6 @@ brush_to_canvas_buf(
 static void      painthit_finish               (CanvasDrawable *, PaintCore16 *,
                                                 Canvas *);
 
-static int       paint_core_16_init_linked  (PaintCore16 *, 
-			CanvasDrawable *, CanvasDrawable *, double, double);
-
 static Canvas *  brush_mask_get                (PaintCore16 *, Canvas *, int, gdouble);
 #ifdef BRUSH_WITH_BORDER 
 static Canvas *  brush_mask_subsample          (Canvas *, double, double);
@@ -194,7 +194,7 @@ paint_core_16_free  (Tool * tool)
   g_free (paint_core);
 }
 
-static int 
+int 
 paint_core_16_init_linked  (
                      PaintCore16 * paint_core,
                      CanvasDrawable *drawable,
@@ -211,7 +211,7 @@ paint_core_16_init_linked  (
   paint_core->curx = x;
   paint_core->cury = y;
 
-  if (paint_core == &non_gui_paint_core)
+  if (paint_core == &non_gui_paint_core || paint_core==&spline_paint_core)
     {
       paint_core->startpressure = paint_core->lastpressure = paint_core->curpressure = 0.4;
       paint_core->startxtilt = paint_core->lastxtilt = paint_core->curxtilt = 0.1;
@@ -347,9 +347,12 @@ paint_core_16_interpolate  (PaintCore16 * paint_core,
   
   /* bail if we haven't moved */
   if (!dx && !dy && !dpressure && !dxtilt && !dytilt)
-{
-    return;
-}
+    {
+#if 0
+      printf("no movement occured!\n");
+#endif
+      return;
+    }
 
   dist = sqrt (SQR (dx) + SQR (dy));
   total = dist + paint_core->distance;
@@ -370,7 +373,6 @@ paint_core_16_interpolate  (PaintCore16 * paint_core,
 	  paint_core->curpressure = paint_core->lastpressure + dpressure * t;
 	  paint_core->curxtilt    = paint_core->lastxtilt + dxtilt * t;
 	  paint_core->curytilt    = paint_core->lastytilt + dytilt * t;
-
 	  (* paint_core->paint_func) (paint_core, drawable, MOTION_PAINT);
 	}
     }
@@ -702,7 +704,7 @@ paint_core_16_cursor_update  (
   
   if ((layer = gimage_get_active_layer (gdisp->gimage)))
     {
-      double off_x, off_y;
+      int off_x, off_y;
       drawable_offsets (GIMP_DRAWABLE(layer), &off_x, &off_y);
       if (x >= off_x && y >= off_y &&
           x < (off_x + drawable_width (GIMP_DRAWABLE(layer))) &&
@@ -837,6 +839,7 @@ paint_core_16_area_setup  (
 #endif
   paint_core->setup_mode = NORMAL_SETUP;
   paint_core->pre_mult = drawable_has_alpha(drawable) ? UNPRE_MULT_ALPHA : PRE_MULT_ALPHA;
+
   (*paint_core->painthit_setup) (paint_core, paint_core->canvas_buf); 
 
   /* configure the linked_canvas_buffer if there is one */
@@ -1070,6 +1073,8 @@ paint_core_16_area_paste  (
       brush_mask2 = canvas_16_to_16 (brush_mask);
   }else if(tag_precision(drawable_tag(drawable)) == PRECISION_FLOAT){
       brush_mask2 = canvas_16_to_float (brush_mask);
+  }else if(tag_precision(drawable_tag(drawable)) == PRECISION_BFP){
+      brush_mask2 = canvas_16_to_bfp (brush_mask);
   }else{
       brush_mask2 = brush_mask;
   }
@@ -1089,6 +1094,9 @@ paint_core_16_area_paste  (
       break;
     }
   
+  if (brush_mask2 != brush_mask)
+    canvas_delete (brush_mask2);
+
   gimage_apply_painthit (drawable_gimage (drawable), drawable,
 			 undo_canvas, paint_core->canvas_buf,
 			 0, 0,
@@ -1178,6 +1186,8 @@ paint_core_16_area_replace  (
       brush_mask2 = canvas_16_to_16 (brush_mask);
   else if(tag_precision(drawable_tag(drawable)) == PRECISION_FLOAT)
       brush_mask2 = canvas_16_to_float (brush_mask);
+  else if(tag_precision(drawable_tag(drawable)) == PRECISION_BFP)
+      brush_mask2 = canvas_16_to_bfp (brush_mask);
   else
       brush_mask2 = brush_mask;
 
@@ -1211,6 +1221,9 @@ paint_core_16_area_replace  (
 			  FALSE, image_opacity, NORMAL_MODE,
 			  paint_core->x, paint_core->y);
 #endif
+  if (brush_mask2 != brush_mask)
+    canvas_delete (brush_mask2);
+
   painthit_finish (drawable, paint_core, paint_core->canvas_buf);
 }
 
@@ -2020,6 +2033,58 @@ static BrushSolidifyMaskFunc brush_solidify_mask_funcs[] =
   brush_solidify_mask_float16
 };
 
+Canvas *
+canvas_16_to_bfp (
+            Canvas *canvas
+            )
+{
+  Canvas *c;
+  ShortsFloat u;
+  gfloat src_val;
+  gint i, j;
+
+  c = canvas_new (
+              tag_new(PRECISION_BFP, canvas_format(canvas), canvas_alpha(canvas)),
+              canvas_width(canvas),
+              canvas_height(canvas),
+              canvas_storage (canvas)
+              );
+      // Convert 16 bit float to 16 bit
+
+
+      canvas_portion_refro (canvas, 0, 0);
+      canvas_portion_refrw (c, 0, 0);
+ 
+    {  
+      /* get the data and advance one line into it  */
+      guint16* data = (guint16*)canvas_portion_data (c, 0, 0);
+      guint16* src = (guint16*)canvas_portion_data (canvas, 0, 0);
+
+      for (i = 0; i < canvas_height (c); i++)
+        {
+#ifdef BRUSH_WITH_BORDER
+      data++;
+#endif
+            for (j = 0; j < canvas_width (c); j++)
+            {
+	      src_val = FLT(*src, u);
+              *data = src_val*ONE_BFP;
+
+              src++;
+              data++;
+            }
+#ifdef BRUSH_WITH_BORDER
+      data++;
+#endif
+        }
+    }
+
+  canvas_portion_unref (canvas, 0, 0);
+  canvas_portion_unref (c, 0, 0);
+
+  return c;
+}
+
 static Canvas * 
 brush_mask_solidify  (
 		      PaintCore16 * paint_core,	
@@ -2172,6 +2237,35 @@ static void brush_solidify_mask_float16 (
     }
 }
 
+static void brush_solidify_mask_bfp (
+			Canvas *brush_mask,
+			Canvas *solid_brush
+		      )
+{  
+  /* get the data and advance one line into it  */
+#ifdef BRUSH_WITH_BORDER
+  guint16* data = (guint16*)canvas_portion_data (solid_brush, 0, 1);
+#else
+  guint16* data = (guint16*)canvas_portion_data (solid_brush, 0, 0);
+#endif
+  guint16* src = (guint16*)canvas_portion_data (brush_mask, 0, 0);
+  gint i, j;
+
+  for (i = 0; i < canvas_height (brush_mask); i++)
+    {
+#ifdef BRUSH_WITH_BORDER
+      data++;
+#endif
+      for (j = 0; j < canvas_width (brush_mask); j++)
+	{
+	  *data++ = (*src++) ? ONE_BFP : 0;
+	}
+#ifdef BRUSH_WITH_BORDER
+      data++;
+#endif
+    }
+}
+
 static void
 paint_core_calculate_brush_size (Canvas *mask,
 				 gdouble  scale,
