@@ -21,6 +21,7 @@
 #include <math.h>
 #include "../appenv.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../paint_funcs_area.h"
 #include "paint_funcs_row_float.h"
 #include "../pixelrow.h"
@@ -2348,6 +2349,116 @@ copy_row_rgb_to_float16_gray  (
 }
 
 
+static void 
+copy_row_rgb_to_bfp_rgb  (
+                          PixelRow * src_row,
+                          PixelRow * dest_row
+                          )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  gfloat * s = (gfloat*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = CLAMP(s[0], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[1] = CLAMP(s[1], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[2] = CLAMP(s[2], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[3] = CLAMP(s[3], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            s += 4;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = CLAMP(s[0], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[1] = CLAMP(s[1], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[2] = CLAMP(s[2], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            s += 4;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = CLAMP(s[0], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[1] = CLAMP(s[1], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[2] = CLAMP(s[2], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[3] = ONE_BFP;
+            s += 3;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = CLAMP(s[0], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[1] = CLAMP(s[1], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            d[2] = CLAMP(s[2], 0.0, BFP_MAX_IN_FLOAT) * ONE_BFP;
+            s += 3;
+            d += 3;
+          }
+    }
+}
+
+
+static void 
+copy_row_rgb_to_bfp_gray  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  gfloat * s = (gfloat*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = INTENSITY ((s[0]*ONE_BFP), (s[1]*ONE_BFP), (s[2]*ONE_BFP));
+            d[1] = s[3] * ONE_BFP;
+            s += 4;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = INTENSITY ((s[0]*ONE_BFP), (s[1]*ONE_BFP), (s[2]*ONE_BFP));
+            s += 4;
+            d += 1;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = INTENSITY ((s[0]*ONE_BFP), (s[1]*ONE_BFP), (s[2]*ONE_BFP));
+            d[1] = ONE_BFP;
+            s += 3;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = INTENSITY ((s[0]*ONE_BFP), (s[1]*ONE_BFP), (s[2]*ONE_BFP));
+            s += 3;
+            d += 1;
+          }
+    }
+}
+
+
 static void 
 copy_row_gray_to_u8_rgb  (
                           PixelRow * src_row,
@@ -2754,10 +2865,111 @@ copy_row_gray_to_float16_gray  (
 }
 
 
+static void 
+copy_row_gray_to_bfp_rgb  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  gfloat * s = (gfloat*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] * ONE_BFP;
+            d[3] = s[1] * ONE_BFP;
+            s += 2;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] * ONE_BFP;
+            s += 2;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] * ONE_BFP;
+            d[3] = 65535;
+            s += 1;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] * ONE_BFP;
+            s += 1;
+            d += 3;
+          }
+    }
+}
+
+
+static void
+copy_row_gray_to_bfp_gray (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  gfloat * s = (gfloat*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] * ONE_BFP;
+            d[1] = s[1] * ONE_BFP;
+            s += 2;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] * ONE_BFP;
+            s += 2;
+            d += 1;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] * ONE_BFP;
+            d[1] = 65535;
+            s += 1;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] * ONE_BFP;
+            s += 1;
+            d += 1;
+          }
+    }
+}
+
+
 typedef void (*CopyRowFunc) (PixelRow *, PixelRow *);
 
-static CopyRowFunc funcs[2][4][2] =
-{
+static CopyRowFunc funcs[2][5][2] = {
   {
     {
       copy_row_rgb_to_u8_rgb,
@@ -2774,6 +2986,10 @@ static CopyRowFunc funcs[2][4][2] =
     {
       copy_row_rgb_to_float16_rgb,
       copy_row_rgb_to_float16_gray
+    },
+    {
+      copy_row_rgb_to_bfp_rgb,
+      copy_row_rgb_to_bfp_gray
     }
   },
   
@@ -2793,11 +3009,13 @@ static CopyRowFunc funcs[2][4][2] =
     {
       copy_row_gray_to_float16_rgb,
       copy_row_gray_to_float16_gray
+    },
+    {
+      copy_row_gray_to_bfp_rgb,
+      copy_row_gray_to_bfp_gray
     }
   }
 };
-
-
 void
 copy_row_float (
              PixelRow * src_row,
@@ -2823,6 +3041,7 @@ copy_row_float (
     case PRECISION_U16:   y = 1; break;
     case PRECISION_FLOAT: y = 2; break;
     case PRECISION_FLOAT16: y = 3; break;
+    case PRECISION_BFP: y = 4; break;
     case PRECISION_NONE:
     default:
       g_warning ("unsupported dest precision in copy_row_float()");
