@@ -22,6 +22,7 @@
 #include <math.h>
 #include "../appenv.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../paint_funcs_area.h"
 #include "paint_funcs_row_float16.h"
 #include "../pixelrow.h"
@@ -3060,6 +3061,146 @@ copy_row_rgb_to_float16_gray  (
 }
 
 
+static void 
+copy_row_rgb_to_bfp_rgb  (
+                          PixelRow * src_row,
+                          PixelRow * dest_row
+                          )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+  gfloat s0, s1, s2, s3;
+  ShortsFloat u;
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+	    s3 = FLT (s[3], u);
+            d[0] = s0 * ONE_BFP;
+            d[1] = s1 * ONE_BFP;
+            d[2] = s2 * ONE_BFP;
+            d[3] = s3 * ONE_BFP;
+            s += 4;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+            d[0] = s0 * ONE_BFP;
+            d[1] = s1 * ONE_BFP;
+            d[2] = s2 * ONE_BFP;
+            s += 4;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+            d[0] = s0 * ONE_BFP;
+            d[1] = s1 * ONE_BFP;
+            d[2] = s2 * ONE_BFP;
+            d[3] = ONE_BFP;
+            s += 3;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+            d[0] = s0 * ONE_BFP;
+            d[1] = s1 * ONE_BFP;
+            d[2] = s2 * ONE_BFP;
+            s += 3;
+            d += 3;
+          }
+    }
+}
+
+
+static void 
+copy_row_rgb_to_bfp_gray  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+  gfloat s0, s1, s2, s3;
+  ShortsFloat u;
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+	    s3 = FLT (s[3], u);
+            d[0] = INTENSITY (s0*ONE_BFP, s1*ONE_BFP, s2*ONE_BFP);
+            d[1] = s3 * ONE_BFP;
+            s += 4;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+            d[0] = INTENSITY (s0*ONE_BFP, s1*ONE_BFP, s2*ONE_BFP);
+            s += 4;
+            d += 1;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+            d[0] = INTENSITY (s0*ONE_BFP, s1*ONE_BFP, s2*ONE_BFP);
+            d[1] = 65535;
+            s += 3;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+	    s0 = FLT (s[0], u);
+	    s1 = FLT (s[1], u);
+	    s2 = FLT (s[2], u);
+            d[0] = INTENSITY (s0*ONE_BFP, s1*ONE_BFP, s2*ONE_BFP);
+            s += 3;
+            d += 1;
+          }
+    }
+}
+
+
 static void 
 copy_row_gray_to_u8_rgb  (
                           PixelRow * src_row,
@@ -3470,10 +3611,113 @@ copy_row_gray_to_float16_gray  (
 }
 
 
+static void 
+copy_row_gray_to_bfp_rgb  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+  ShortsFloat u;
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = FLT (s[0], u) * ONE_BFP;
+            d[3] = FLT (s[1], u) * ONE_BFP;
+            s += 2;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = FLT (s[0], u) * ONE_BFP;
+            s += 2;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = FLT (s[0], u) * ONE_BFP;
+            d[3] = 65535;
+            s += 1;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = FLT (s[0], u) * ONE_BFP;
+            s += 1;
+            d += 3;
+          }
+    }
+}
+
+
+static void
+copy_row_gray_to_bfp_gray (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+  ShortsFloat u;
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = FLT (s[0], u) * ONE_BFP;
+            d[1] = FLT (s[1], u) * ONE_BFP;
+            s += 2;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = FLT (s[0], u) * ONE_BFP;
+            s += 2;
+            d += 1;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = FLT (s[0], u) * ONE_BFP;
+            d[1] = 65535;
+            s += 1;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = FLT (s[0], u) * ONE_BFP;
+            s += 1;
+            d += 1;
+          }
+    }
+}
+
+
 typedef void (*CopyRowFunc) (PixelRow *, PixelRow *);
 
-static CopyRowFunc funcs[2][4][2] =
-{
+static CopyRowFunc funcs[2][5][2] = {
   {
     {
       copy_row_rgb_to_u8_rgb,
@@ -3490,6 +3734,10 @@ static CopyRowFunc funcs[2][4][2] =
     {
       copy_row_rgb_to_float16_rgb,
       copy_row_rgb_to_float16_gray
+    },
+    {
+      copy_row_rgb_to_bfp_rgb,
+      copy_row_rgb_to_bfp_gray
     }
   },
   
@@ -3509,11 +3757,14 @@ static CopyRowFunc funcs[2][4][2] =
     {
       copy_row_gray_to_float16_rgb,
       copy_row_gray_to_float16_gray
+    },
+    {
+      copy_row_gray_to_bfp_rgb,
+      copy_row_gray_to_bfp_gray
     }
   }
 };
 
-
 void
 copy_row_float16 (
              PixelRow * src_row,
@@ -3539,6 +3790,7 @@ copy_row_float16 (
     case PRECISION_U16:   y = 1; break;
     case PRECISION_FLOAT: y = 2; break;
     case PRECISION_FLOAT16: y = 3; break;
+    case PRECISION_BFP: y = 4; break;
     case PRECISION_NONE:
     default:
       g_warning ("unsupported dest precision in copy_row_float16()");
