@@ -20,6 +20,7 @@
 #include <math.h>
 #include "../appenv.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../paint_funcs_area.h"
 #include "paint_funcs_row_u16.h"
 #include "../pixelrow.h"
@@ -448,8 +449,8 @@ hsv_only_row_u16 (
           int       mode
           )
 {
-  gint r1, g1, b1;
-  gint r2, g2, b2;
+  gfloat r1, g1, b1;
+  gfloat r2, g2, b2;
   Tag     src1_tag      = pixelrow_tag (src1_row); 
   Tag     src2_tag      = pixelrow_tag (src2_row); 
   gint    ha1           = (tag_alpha (src1_tag)==ALPHA_YES)? TRUE: FALSE;
@@ -1122,7 +1123,7 @@ combine_mask_and_alpha_channel_row_u16  (
   gint    width        = pixelrow_width (src_row);
   guint16 opac = opacity * 65535;
 
-  alpha = num_channels - 1;
+	  alpha = num_channels - 1;
   while (width --)
     {
       mask_val = INT_MULT_16 (*mask++, opac , t );
@@ -2969,6 +2970,116 @@ static void copy_row_rgb_to_float16_gray (PixelRow* src_row, PixelRow* dest_row)
    }
 
 
+static void 
+copy_row_rgb_to_bfp_rgb  (
+                          PixelRow * src_row,
+                          PixelRow * dest_row
+                          )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            d[1] = s[1] >> 1;
+            d[2] = s[2] >> 1;
+            d[3] = s[3] >> 1;
+            s += 4;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            d[1] = s[1] >> 1;
+            d[2] = s[2] >> 1;
+            s += 4;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            d[1] = s[1] >> 1;
+            d[2] = s[2] >> 1;
+            d[3] = ONE_BFP;
+            s += 3;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            d[1] = s[1] >> 1;
+            d[2] = s[2] >> 1;
+            s += 3;
+            d += 3;
+          }
+    }
+}
+
+
+static void 
+copy_row_rgb_to_bfp_gray  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] >> 1, s[1] >> 1, s[2] >> 1);
+            d[1] = s[3] >> 1;
+            s += 4;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] >> 1, s[1] >> 1, s[2] >> 1);
+            s += 4;
+            d += 1;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] >> 1, s[1] >> 1, s[2] >> 1);
+            d[1] = 65535;
+            s += 3;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] >> 1, s[1] >> 1, s[2] >> 1);
+            s += 3;
+            d += 1;
+          }
+    }
+}
+
+
 static void 
 copy_row_gray_to_u8_rgb  (
                           PixelRow * src_row,
@@ -3376,9 +3487,111 @@ static void copy_row_gray_to_float16_gray (PixelRow* src_row, PixelRow* dest_row
       }
    }
 
+
+static void 
+copy_row_gray_to_bfp_rgb  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] >> 1;
+            d[3] = s[1] >> 1;
+            s += 2;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] >> 1;
+            s += 2;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] >> 1;
+            d[3] = 65535;
+            s += 1;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] >> 1;
+            s += 1;
+            d += 3;
+          }
+    }
+}
+
+
+static void
+copy_row_gray_to_bfp_gray (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint16 * s = (guint16*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            d[3] = s[1] >> 1;
+            s += 2;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            s += 2;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            d[1] = 65535;
+            s += 1;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] >> 1;
+            s += 1;
+            d += 1;
+          }
+    }
+}
+
 typedef void (*CopyRowFunc) (PixelRow *, PixelRow *);
 
-static CopyRowFunc funcs[2][4][2] = {
+static CopyRowFunc funcs[2][5][2] = {
   {
     {
       copy_row_rgb_to_u8_rgb,
@@ -3391,10 +3604,14 @@ static CopyRowFunc funcs[2][4][2] = {
     {
       copy_row_rgb_to_float_rgb,
       copy_row_rgb_to_float_gray
-      },
-      {
-         copy_row_rgb_to_float16_rgb,
-         copy_row_rgb_to_float16_gray,
+    },
+    {
+      copy_row_rgb_to_float16_rgb,
+      copy_row_rgb_to_float16_gray
+    },
+    {
+      copy_row_rgb_to_bfp_rgb,
+      copy_row_rgb_to_bfp_gray
     }
   },
   
@@ -3410,10 +3627,14 @@ static CopyRowFunc funcs[2][4][2] = {
     {
       copy_row_gray_to_float_rgb,
       copy_row_gray_to_float_gray
-      },
-      {
-         copy_row_gray_to_float16_rgb,
-         copy_row_gray_to_float16_gray
+    },
+    {
+      copy_row_gray_to_float16_rgb,
+      copy_row_gray_to_float16_gray
+    },
+    {
+      copy_row_gray_to_bfp_rgb,
+      copy_row_gray_to_bfp_gray
     }
   }
 };
@@ -3455,6 +3676,9 @@ copy_row_u16 (
       case PRECISION_FLOAT16:
          y = 3;
          break;
+      case PRECISION_BFP:
+         y = 4; 
+         break;         
     case PRECISION_NONE:
     default:
         g_warning ("unsupported dest precision in copy_row_u16() of [%d]", tag_precision (pixelrow_tag (dest_row)));
