@@ -81,6 +81,7 @@ static void posterize_u8 (PixelArea *, PixelArea *, void *);
 static void posterize_u16 (PixelArea *, PixelArea *, void *);
 static void posterize_float (PixelArea *, PixelArea *, void *);
 static void posterize_float16 (PixelArea *, PixelArea *, void *);
+static void posterize_bfp (PixelArea *, PixelArea *, void *);
 
 
 /*  posterize machinery  */
@@ -97,7 +98,9 @@ posterize_func (Tag dest_tag)
   case PRECISION_FLOAT:
     return posterize_float; 
   case PRECISION_FLOAT16:
-    return posterize_float16; 
+    return posterize_float16;   
+  case PRECISION_BFP:
+    return posterize_bfp; 
   default:
     return NULL;
   } 
@@ -327,6 +330,63 @@ posterize_float16 (PixelArea *src_area,
     }
 }
 
+static void
+posterize_bfp (PixelArea *src_area,
+	   PixelArea *dest_area,
+	   void        *user_data)
+{
+  PosterizeDialog *pd;
+  Tag src_tag = pixelarea_tag (src_area);
+  Tag dest_tag = pixelarea_tag (dest_area);
+  guchar *src, *dest;
+  guint16 *s, *d;
+  gint alpha = tag_alpha (src_tag);
+  gint has_alpha;
+  gint s_num_channels = tag_num_channels (src_tag);
+  gint d_num_channels = tag_num_channels (dest_tag);
+  int w, h, b, i;
+  double interval, half_interval;
+  guint16 transfer[65536];
+
+  pd = (PosterizeDialog *) user_data;
+
+  /*  Set the transfer array  */
+  interval = 65535.0 / (double) (pd->levels - 1);
+  half_interval = interval / 2.0;
+
+  for (i = 0; i < 65536; i++)
+    transfer[i] = (guint16) ((int) (((double) i + half_interval) / interval) * interval);
+
+  h = pixelarea_height (src_area);
+  src = (guchar*)pixelarea_data (src_area);
+  dest = (guchar*)pixelarea_data (dest_area);
+
+  has_alpha = (alpha == ALPHA_YES) ? TRUE: FALSE;
+  alpha = has_alpha ? s_num_channels - 1 : s_num_channels;
+  
+ 
+  while (h--)
+    {
+      w = pixelarea_width (src_area);
+      s = (guint16*)src;
+      d = (guint16*)dest;
+      while (w--)
+	{
+	  for (b = 0; b < alpha; b++)
+	    d[b] = transfer[s[b]];
+
+	  if (has_alpha)
+	    d[alpha] = s[alpha];
+	   
+	  s+= s_num_channels;
+	  d+= d_num_channels;
+	}
+
+      src += pixelarea_rowstride (src_area);
+      dest += pixelarea_rowstride (dest_area); 
+    }
+}
+
 /*  posterize select action functions  */
 
 static void
