@@ -1907,6 +1907,55 @@ static void text_gdk_image_to_region_float16 (GdkImage* image, int scale, PixelA
      pixelarea_write_row (textPR, &row, pixelarea_x(textPR), y, width);
    }
 }
+
+static void
+text_gdk_image_to_region_bfp (GdkImage    *image,
+			  int          scale,
+			  PixelArea   *textPR)
+{
+   int black_pixel;
+   int pixel;
+   int value;
+   int scalex, scaley;
+   int scale2;
+   int x, y;
+   int i, j;
+   PixelRow row;
+   guint16 *outdata, *outd;
+   int width = pixelarea_width (textPR);
+
+   scale2 = scale * scale;
+#ifdef WIN32
+	black_pixel=0;
+#else
+   black_pixel = BlackPixel (DISPLAY, DefaultScreen (DISPLAY));
+#endif
+   outdata = (guint16 *) g_malloc (width*sizeof(guint16));
+
+  for (y = 0, scaley = 0; y < pixelarea_height (textPR); y++, scaley += scale)
+    {
+      outd = outdata;
+      for (x = 0, scalex = 0; x < pixelarea_width (textPR); x++, scalex += scale)
+	{
+           value = 0;
+
+	  for (i = scaley; i < scaley + scale; i++)
+	    for (j = scalex; j < scalex + scale; j++)
+	      {
+                   pixel = gdk_image_get_pixel (image, j, i);
+                   if (pixel == black_pixel)
+		  value ++;
+               }
+
+           /*  store the alpha value in the data  */
+           *outd++= (guint16) ((value * 65535) / scale2);
+
+	}
+      pixelrow_init (&row, pixelarea_tag (textPR), (guchar*) outdata, width);
+      pixelarea_write_row (textPR, &row, pixelarea_x(textPR), y, width);
+
+      }
+}
  
 static Layer *
 text_render (GImage *gimage,
@@ -2083,6 +2132,8 @@ text_render (GImage *gimage,
               break;
            case PRECISION_FLOAT16:
               text_gdk_image_to_region_float16(image, antialias, &maskPR);
+           case PRECISION_BFP:
+              text_gdk_image_to_region_bfp(image, antialias, &maskPR);
               /*
                * g_message("Currently unsupported for this format");
                */
