@@ -25,6 +25,7 @@
 #include "../general.h"
 #include "../gdisplay.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../histogram.h"
 #include "../image_map.h"
 #include "../interface.h"
@@ -82,6 +83,7 @@ static void               threshold_low_threshold_text_update_u8  (GtkWidget *,
 static void               threshold_low_threshold_text_update_u16  (GtkWidget *, gpointer);
 static void               threshold_low_threshold_text_update_float  (GtkWidget *, gpointer);
 static void               threshold_low_threshold_text_update_float16  (GtkWidget *, gpointer);
+static void               threshold_low_threshold_text_update_bfp  (GtkWidget *, gpointer);
 
 typedef void (*ThresholdHighThresholdTextUpdateFunc)(GtkWidget *, gpointer);
 static ThresholdHighThresholdTextUpdateFunc threshold_high_threshold_text_update;
@@ -89,6 +91,7 @@ static void               threshold_high_threshold_text_update_u8 (GtkWidget *,
 static void               threshold_high_threshold_text_update_u16 (GtkWidget *, gpointer);
 static void               threshold_high_threshold_text_update_float (GtkWidget *, gpointer);
 static void               threshold_high_threshold_text_update_float16 (GtkWidget *, gpointer);
+static void               threshold_high_threshold_text_update_bfp (GtkWidget *, gpointer);
 
 static void *threshold_options = NULL;
 static ThresholdDialog *threshold_dialog = NULL;
@@ -100,6 +103,7 @@ static void       threshold_u8 (PixelArea *, PixelArea *, void *);
 static void       threshold_u16 (PixelArea *, PixelArea *, void *);
 static void       threshold_float (PixelArea *, PixelArea *, void *);
 static void       threshold_float16 (PixelArea *, PixelArea *, void *);
+static void       threshold_bfp (PixelArea *, PixelArea *, void *);
 
 typedef void (*ThresholdHistogramInfoFunc)(PixelArea *, PixelArea *, HistogramValues, void *);
 static ThresholdHistogramInfoFunc threshold_histogram_info;
@@ -107,6 +111,7 @@ static void       threshold_histogram_info_u8 (PixelArea *, PixelArea *, Histogr
 static void       threshold_histogram_info_u16 (PixelArea *, PixelArea *, HistogramValues, void *);
 static void       threshold_histogram_info_float (PixelArea *, PixelArea *, HistogramValues, void *);
 static void       threshold_histogram_info_float16 (PixelArea *, PixelArea *, HistogramValues, void *);
+static void       threshold_histogram_info_bfp (PixelArea *, PixelArea *, HistogramValues, void *);
 
 typedef void (*ThresholdHistogramRangeFunc)(int, int, int, HistogramValues, void *);
 static ThresholdHistogramRangeFunc threshold_histogram_range;
@@ -114,6 +119,7 @@ static void       threshold_histogram_range_u8 (int, int, int, HistogramValues,
 static void       threshold_histogram_range_u16 (int, int, int, HistogramValues, void *);
 static void       threshold_histogram_range_float (int, int, int, HistogramValues, void *);
 static void       threshold_histogram_range_float16 (int, int, int, HistogramValues, void *);
+static void       threshold_histogram_range_bfp (int, int, int, HistogramValues, void *);
 static Argument * threshold_invoker (Argument *);
 
 /*  threshold machinery  */
@@ -151,6 +157,13 @@ threshold_funcs (Tag tag)
      threshold_high_threshold_text_update = threshold_high_threshold_text_update_float16;
      threshold_low_threshold_text_update = threshold_low_threshold_text_update_float16;
      break;
+  case PRECISION_BFP:
+     threshold = threshold_bfp;
+     threshold_histogram_info = threshold_histogram_info_bfp;
+     threshold_histogram_range = threshold_histogram_range_bfp;
+     threshold_high_threshold_text_update = threshold_high_threshold_text_update_bfp;
+     threshold_low_threshold_text_update = threshold_low_threshold_text_update_bfp;
+     break;
   default:
      threshold = NULL;
      threshold_histogram_info = NULL;
@@ -397,6 +410,64 @@ threshold_float16 ( PixelArea *src_area,
     }
 }
 
+static void
+threshold_bfp ( PixelArea *src_area,
+	       PixelArea *dest_area,
+	       void      *user_data)
+{
+  ThresholdDialog *td;
+  gint             has_alpha;
+  gint             w, h, b;
+  guchar	   *src, *dest;
+  guint16          *s, *d;
+  gint             value;
+  Tag              src_tag = pixelarea_tag (src_area);
+  gint             s_num_channels = tag_num_channels (src_tag);
+  gint             alpha = tag_alpha (src_tag);
+  Tag              dest_tag = pixelarea_tag (dest_area);
+  gint             d_num_channels = tag_num_channels (dest_tag);
+
+  td = (ThresholdDialog *) user_data;
+
+  h = pixelarea_height (src_area);
+
+  has_alpha = (alpha == ALPHA_YES) ? TRUE: FALSE;
+  alpha = has_alpha ? s_num_channels - 1 : s_num_channels;
+
+  src = (guchar*)pixelarea_data (src_area);
+  dest = (guchar*)pixelarea_data (dest_area);
+
+  while (h--)
+    {
+       w = pixelarea_width (src_area);
+       s = (guint16 *)src;
+       d = (guint16 *)dest;
+       while (w--)
+	 {
+	   if (td->color)
+	     {
+	       value = MAX (s[RED_PIX], s[GREEN_PIX]);
+	       value = MAX (value, s[BLUE_PIX]);
+	       value = (value >= td->low_threshold && value <= td->high_threshold ) ? ONE_BFP: 0;
+	     }
+	   else
+	     value = (s[GRAY_PIX] >= td->low_threshold && s[GRAY_PIX] <= td->high_threshold) ? ONE_BFP : 0;
+
+	   for (b = 0; b < alpha; b++)
+	     d[b] = value;
+
+	   if (has_alpha)
+	     d[alpha] = s[alpha];
+
+	   s += s_num_channels;
+	   d += d_num_channels;
+	 }
+
+	 src += pixelarea_rowstride (src_area);
+	 dest += pixelarea_rowstride (dest_area);
+    }
+}
+
 
 static void
 threshold_histogram_info_u8 (PixelArea       *src_area,
@@ -702,6 +773,80 @@ threshold_histogram_info_float16 (PixelArea   *src_area,
     }
 }
 
+static void
+threshold_histogram_info_bfp (PixelArea   *src_area,
+			  PixelArea       *mask_area,
+			  HistogramValues  values,
+			  void            *user_data)
+{
+  ThresholdDialog *td;
+  gint             has_alpha;
+  int              w, h;
+  guchar	  *src, *mask = NULL;
+  guint16         *s, *m = NULL;
+  guint16          red, green, blue;
+  gint             value_bin;
+  gint             value;
+  Tag              src_tag = pixelarea_tag (src_area);
+  gint             s_num_channels = tag_num_channels (src_tag);
+  gint             alpha = tag_alpha (src_tag);
+  Tag              mask_tag = pixelarea_tag (mask_area);
+  gint             m_num_channels = tag_num_channels (mask_tag);
+  gint 		   bins = 256; 
+
+  td = (ThresholdDialog *) user_data;
+
+  h = pixelarea_height (src_area);
+
+  has_alpha = (alpha == ALPHA_YES) ? TRUE: FALSE;
+  alpha = has_alpha ? s_num_channels - 1 : s_num_channels;
+
+  src = (guchar*)pixelarea_data (src_area);
+
+  if (mask_area)
+    mask = (guchar*)pixelarea_data (mask_area);
+
+  while (h--)
+    {
+       w = pixelarea_width (src_area);
+       s = (guint16*)src;
+
+       if (mask_area)
+	 m = (guint16*)mask;
+
+       while (w--)
+	 {
+	   if (td->color)
+	     {
+	       red   = s[RED_PIX];
+	       green = s[GREEN_PIX];
+	       blue  = s[BLUE_PIX];
+	       value = MAX (red, green);
+	       value = MAX (value, blue);
+	       value_bin = value/bins;
+	     }
+	   else
+	     value_bin = s[GRAY_PIX]/bins;
+
+	   if (mask_area)
+	     values[HISTOGRAM_VALUE][value_bin] += (double) *m / 65535.0;
+	   else
+	     values[HISTOGRAM_VALUE][value_bin] += 1.0;
+
+	   s += s_num_channels;
+
+	   if (mask_area)
+	     m += m_num_channels;
+	 }
+
+       src += pixelarea_rowstride (src_area);
+ 
+       if (mask_area)
+	 mask += pixelarea_rowstride (mask_area);
+    }
+}
+
+
 
 static void
 threshold_histogram_range_u8 (int              start,
@@ -810,6 +955,30 @@ threshold_histogram_range_float16 (int              start,
 
 /*  threshold action functions  */
 
+static void
+threshold_histogram_range_bfp (int              start,
+			   int              end,
+			   int              bins,
+			   HistogramValues  values,
+			   void            *user_data)
+{
+  ThresholdDialog *td;
+  char text[32];
+
+  td = (ThresholdDialog *) user_data;
+
+  td->low_threshold = start;
+  td->high_threshold = end;
+  td->bins = bins;
+  sprintf (text, "%f", BFP_TO_FLOAT(td->low_threshold));
+  gtk_entry_set_text (GTK_ENTRY (td->low_threshold_text), text);
+  sprintf (text, "%f", BFP_TO_FLOAT(td->high_threshold));
+  gtk_entry_set_text (GTK_ENTRY (td->high_threshold_text), text);
+
+  if (td->preview)
+    threshold_preview (td);
+}
+
 static void
 threshold_button_press (Tool           *tool,
 			GdkEventButton *bevent,
@@ -956,6 +1125,7 @@ threshold_initialize (void *gdisp_ptr)
      start = threshold_dialog->low_threshold;
      end = threshold_dialog->high_threshold;
      break;
+  case PRECISION_BFP:
   case PRECISION_U16:
      start = threshold_dialog->low_threshold/256;
      end = threshold_dialog->high_threshold/256;
@@ -1005,6 +1175,7 @@ threshold_new_dialog (Tag tag)
       td->low_threshold = 255/2;
       td->bins = 256;
      break;
+  case PRECISION_BFP:
   case PRECISION_U16:
       td->high_threshold = 65535;
       td->low_threshold = 65535/2;
@@ -1060,6 +1231,9 @@ threshold_new_dialog (Tag tag)
   case PRECISION_FLOAT16:
      gtk_entry_set_text (GTK_ENTRY (td->low_threshold_text), "0.5");
      break;
+  case PRECISION_BFP:
+     gtk_entry_set_text (GTK_ENTRY (td->low_threshold_text), "16383");
+     break;
   default:
     break;
   }
@@ -1079,6 +1253,7 @@ threshold_new_dialog (Tag tag)
   case PRECISION_U8:
     gtk_entry_set_text (GTK_ENTRY (td->high_threshold_text), "255");
      break;
+  case PRECISION_BFP:
   case PRECISION_U16:
     gtk_entry_set_text (GTK_ENTRY (td->high_threshold_text), "65535");
      break;
@@ -1148,6 +1323,7 @@ threshold_new_dialog (Tag tag)
      start = td->low_threshold;
      end = td->high_threshold;
      break;
+  case PRECISION_BFP:
   case PRECISION_U16:
      start = td->low_threshold/256;
      end = td->high_threshold/256;
@@ -1331,6 +1507,30 @@ threshold_low_threshold_text_update_float16 (GtkWidget *w,
   threshold_low_threshold_text_update_float (w,data);
 }
 
+static void
+threshold_low_threshold_text_update_bfp (GtkWidget *w,
+				     gpointer   data)
+{
+  ThresholdDialog *td;
+  char *str;
+  int value;
+  gint start, end;
+
+  td = (ThresholdDialog *) data;
+  str = gtk_entry_get_text (GTK_ENTRY (w));
+  value = BOUNDS (FLOAT_TO_BFP( atof (str)), 0, td->high_threshold);
+
+  if (value != td->low_threshold)
+    {
+      td->low_threshold = value;
+      start = td->low_threshold/256;
+      end = td->high_threshold/256;
+      histogram_range (td->histogram, start, end);
+      if (td->preview)
+	threshold_preview (td);
+    }
+}
+
 static void
 threshold_high_threshold_text_update_u8 (GtkWidget *w,
 				      gpointer   data)
@@ -1416,6 +1616,32 @@ threshold_high_threshold_text_update_float16 (GtkWidget *w,
   threshold_high_threshold_text_update_float (w,data);
 }
 
+static void
+threshold_high_threshold_text_update_bfp (GtkWidget *w,
+				      gpointer   data)
+{
+  ThresholdDialog *td;
+  char *str;
+  int value;
+  gint start, end;
+  Tag tag;
+
+  td = (ThresholdDialog *) data;
+  tag = drawable_tag (td->drawable);
+  str = gtk_entry_get_text (GTK_ENTRY (w));
+  value = BOUNDS (FLOAT_TO_BFP(atof (str)), td->low_threshold, 65535);
+
+  if (value != td->high_threshold)
+    {
+      td->high_threshold = value;
+      start = td->low_threshold/256;
+      end = td->high_threshold/256;
+      histogram_range (td->histogram, start, end);
+      if (td->preview)
+	threshold_preview (td);
+    }
+}
+
 /*  The threshold procedure definition  */
 ProcArg threshold_args[] =
 {
@@ -1507,6 +1733,7 @@ threshold_invoker (args)
 	  max = bins-1;
 	  break;
 
+      case PRECISION_BFP:
       case PRECISION_U16:
 	  bins = 65536;
 	  max = bins-1;
