@@ -38,6 +38,9 @@
 #include "../layer_pvt.h"
 #include "../channel_pvt.h"
 #include "../tile_manager_pvt.h"
+#include "../spline.h"
+
+#include "../../lib/wire/dl_list.h"
 
 /* #define SWAP_FROM_FILE */
 
@@ -62,7 +65,17 @@ typedef enum
   PROP_OFFSETS = 15,
   PROP_COLOR = 16,
   PROP_COMPRESSION = 17,
-  PROP_GUIDES = 18
+  PROP_GUIDES = 18,
+  PROP_SPLINES_START=19,
+  PROP_SPLINES_CONTINUITY=20,
+  PROP_SPLINES_KEYFRAME=21,
+  PROP_SPLINES_ACTIVE=22,
+  PROP_SPLINES_TYPE=23,
+  PROP_SPLINES_DRAW_POINTS=24,
+  PROP_SPLINES_POINTS_START=25,
+  PROP_SPLINES_SAVE_POINT=26,
+  PROP_SPLINES_POINTS_END=27,
+  PROP_SPLINES_END=28
 } PropType;
 
 typedef enum
@@ -128,7 +141,6 @@ static gint     xcf_load_hierarchy     (XcfInfo     *info,
 					Canvas *tiles);
 static void xcf_load_row 	   (XcfInfo *info, 
 				    PixelRow *row);
-
 #ifdef SWAP_FROM_FILE
 static int      xcf_swap_func          (int          fd,
 					Tile        *tile,
@@ -172,7 +184,7 @@ static guint xcf_write_string (FILE     *fp,
 			       gchar   **data,
 			       gint      count);
 
-
+Spline *xcf_spline=NULL;
 
 static ProcArg xcf_load_args[] =
 {
@@ -610,6 +622,9 @@ xcf_save_layer_props (XcfInfo *info,
 		      GImage  *gimage,
 		      Layer   *layer)
 {
+  Spline *spline;
+  SplinePoint *point;
+
   if (layer == gimage->active_layer)
     xcf_save_prop (info, PROP_ACTIVE_LAYER);
 
@@ -628,7 +643,27 @@ xcf_save_layer_props (XcfInfo *info,
   xcf_save_prop (info, PROP_SHOW_MASK, layer->show_mask);
   xcf_save_prop (info, PROP_OFFSETS, GIMP_DRAWABLE(layer)->offset_x, GIMP_DRAWABLE(layer)->offset_y);
   xcf_save_prop (info, PROP_MODE, layer->mode);
-
+  if(!DL_is_empty(layer->sl.splines))
+    {
+      spline=(Spline*)layer->sl.splines->head;
+      while(spline)
+	{
+	  xcf_save_prop(info, PROP_SPLINES_START, 1);
+	  xcf_save_prop(info, PROP_SPLINES_CONTINUITY, spline->continuity);
+	  xcf_save_prop(info, PROP_SPLINES_KEYFRAME, spline->keyframe);
+	  xcf_save_prop(info, PROP_SPLINES_ACTIVE, spline->active);
+	  xcf_save_prop(info, PROP_SPLINES_TYPE, spline->type);
+	  xcf_save_prop(info, PROP_SPLINES_DRAW_POINTS, spline->draw_points);
+	  xcf_save_prop(info, PROP_SPLINES_POINTS_START, 1);
+	  point=NULL;
+	  if(!DL_is_empty(spline->points))
+	    point=(SplinePoint*)spline->points->head;
+	  xcf_save_prop(info, PROP_SPLINES_SAVE_POINT, point);
+	  xcf_save_prop(info, PROP_SPLINES_POINTS_END, 1);
+	  xcf_save_prop(info, PROP_SPLINES_END, 1);
+	  spline=(Spline*)((DL_node*)spline)->next;
+	}
+    }
   xcf_save_prop (info, PROP_END);
 }
 
@@ -669,11 +704,12 @@ xcf_save_prop (XcfInfo  *info,
 	       PropType  prop_type,
 	       ...)
 {
-  guint32 size;
+  guint32 size, type;
   va_list args;
 
   va_start (args, prop_type);
 
+  type = prop_type;
   switch (prop_type)
     {
     case PROP_COLOR:
@@ -682,7 +718,7 @@ xcf_save_prop (XcfInfo  *info,
     case PROP_END:
       size = 0;
 
-      info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+      info->cp += xcf_write_int32 (info->fp, &type, 1);
       info->cp += xcf_write_int32 (info->fp, &size, 1);
       break;
     case PROP_COLORMAP:
@@ -694,7 +730,7 @@ xcf_save_prop (XcfInfo  *info,
 	colors = va_arg (args, guchar*);
 	size = 4 + ncolors;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &ncolors, 1);
 	info->cp += xcf_write_int8 (info->fp, colors, ncolors * 3);
@@ -705,7 +741,7 @@ xcf_save_prop (XcfInfo  *info,
     case PROP_SELECTION:
       size = 0;
 
-      info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+      info->cp += xcf_write_int32 (info->fp, &type, 1);
       info->cp += xcf_write_int32 (info->fp, &size, 1);
       break;
     case PROP_FLOATING_SELECTION:
@@ -715,7 +751,7 @@ xcf_save_prop (XcfInfo  *info,
 	dummy = 0;
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->floating_sel_offset = info->cp;
 	info->cp += xcf_write_int32 (info->fp, &dummy, 1);
@@ -727,7 +763,7 @@ xcf_save_prop (XcfInfo  *info,
 	opacity = va_arg (args, double);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_float (info->fp, (gfloat*) &opacity, 1);
       }
@@ -739,7 +775,7 @@ xcf_save_prop (XcfInfo  *info,
 	mode = va_arg (args, gint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, (guint32*) &mode, 1);
       }
@@ -751,7 +787,7 @@ xcf_save_prop (XcfInfo  *info,
 	visible = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &visible, 1);
       }
@@ -763,7 +799,7 @@ xcf_save_prop (XcfInfo  *info,
 	linked = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &linked, 1);
       }
@@ -775,7 +811,7 @@ xcf_save_prop (XcfInfo  *info,
 	preserve_trans = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &preserve_trans, 1);
       }
@@ -787,7 +823,7 @@ xcf_save_prop (XcfInfo  *info,
 	apply_mask = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &apply_mask, 1);
       }
@@ -799,7 +835,7 @@ xcf_save_prop (XcfInfo  *info,
 	edit_mask = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &edit_mask, 1);
       }
@@ -811,7 +847,7 @@ xcf_save_prop (XcfInfo  *info,
 	show_mask = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &show_mask, 1);
       }
@@ -823,7 +859,7 @@ xcf_save_prop (XcfInfo  *info,
 	show_masked = va_arg (args, guint32);
 	size = 4;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, &show_masked, 1);
       }
@@ -836,7 +872,7 @@ xcf_save_prop (XcfInfo  *info,
 	offsets[1] = va_arg (args, gint32);
 	size = 8;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int32 (info->fp, (guint32*) offsets, 2);
       }
@@ -848,7 +884,7 @@ xcf_save_prop (XcfInfo  *info,
 	compression = (guint8) va_arg (args, guint32);
 	size = 1;
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 	info->cp += xcf_write_int8 (info->fp, &compression, 1);
       }
@@ -866,7 +902,7 @@ xcf_save_prop (XcfInfo  *info,
 
 	size = nguides * (4 + 1);
 
-	info->cp += xcf_write_int32 (info->fp, (guint32*) &prop_type, 1);
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
 	info->cp += xcf_write_int32 (info->fp, &size, 1);
 
 	while (guides)
@@ -882,6 +918,118 @@ xcf_save_prop (XcfInfo  *info,
 	  }
       }
       break;
+    case PROP_SPLINES_START:
+      {
+	gint32 has_spline;
+	has_spline=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&has_spline, 1);
+      }
+      break;
+    case PROP_SPLINES_CONTINUITY:
+      {
+	gint32 continuity;
+	continuity=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&continuity, 1);
+      }
+      break;
+    case PROP_SPLINES_KEYFRAME:
+      {
+	gint32 keyframe;
+	keyframe=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&keyframe, 1);
+      }
+      break;
+    case PROP_SPLINES_ACTIVE:
+      {
+	gint32 active;
+	active=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&active, 1);
+      }
+      break;
+    case PROP_SPLINES_TYPE:
+      {
+	gint32 type;
+	type=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&type, 1);
+      }
+      break;
+    case PROP_SPLINES_DRAW_POINTS:
+      {
+	gint32 draw_points;
+	draw_points=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&draw_points, 1);
+      }
+      break;
+    case PROP_SPLINES_POINTS_START:
+      break;
+    case PROP_SPLINES_SAVE_POINT:
+      {
+	SplinePoint *point, *start_point;
+	gint32 x, y, cx1, cx2, cy1, cy2;
+	gint32 number_points=0;
+
+	start_point=va_arg(args, SplinePoint*);
+	point=start_point;
+	while(point)
+	  {
+	    number_points++;
+	    point=(SplinePoint*)((DL_node*)point)->next;
+	  }
+	point=start_point;
+	size=28*number_points+4;
+
+	info->cp += xcf_write_int32 (info->fp, &type, 1);
+	info->cp += xcf_write_int32 (info->fp, &size, 1);
+	info->cp += xcf_write_int32 (info->fp, &number_points, 1);
+
+	while (point)
+	  {
+	    x=point->x;
+	    y=point->y;
+	    cx1=point->cx1;
+	    cy1=point->cy1;
+	    cx2=point->cx2;
+	    cy2=point->cy2;
+	    info->cp += xcf_write_int32(info->fp, (guint32*) &x, 1);
+	    info->cp += xcf_write_int32(info->fp, (guint32*) &y, 1);
+	    info->cp += xcf_write_int32(info->fp, (guint32*) &cx1, 1);
+	    info->cp += xcf_write_int32(info->fp, (guint32*) &cy1, 1);
+	    info->cp += xcf_write_int32(info->fp, (guint32*) &cx2, 1);
+	    info->cp += xcf_write_int32(info->fp, (guint32*) &cy2, 1);
+	    point=(SplinePoint*)((DL_node*)point)->next;
+	  }
+      }
+      break;
+    case PROP_SPLINES_POINTS_END:
+      break;
+    case PROP_SPLINES_END:
+      {
+	gint32 has_spline;
+	has_spline=va_arg(args, gint32);
+	size=4;
+	info->cp += xcf_write_int32(info->fp, &type, 1);
+	info->cp += xcf_write_int32(info->fp, &size, 1);
+	info->cp += xcf_write_int32(info->fp, (guint32*)&has_spline, 1);
+      }
+      break;
     }
 
   va_end (args);
@@ -1011,14 +1159,16 @@ xcf_save_hierarchy (XcfInfo     *info,
   Tag tag = canvas_tag (tiles);
   int bytes = canvas_bytes (tiles);
   StorageType storage = canvas_storage (tiles);
+  guint32 stora = storage;
   AutoAlloc auto_alloc = canvas_autoalloc (tiles);
+  guint32 auto_all = auto_alloc;
 
   info->cp += xcf_write_int32 (info->fp, (guint32*) &w, 1);
   info->cp += xcf_write_int32 (info->fp, (guint32*) &h, 1);
   info->cp += xcf_write_int32 (info->fp, (guint32*) &tag, 1);
   info->cp += xcf_write_int32 (info->fp, (guint32*) &bytes, 1);
-  info->cp += xcf_write_int32 (info->fp, (guint32*) &storage, 1);
-  info->cp += xcf_write_int32 (info->fp, (guint32*) &auto_alloc, 1);
+  info->cp += xcf_write_int32 (info->fp, &stora, 1);
+  info->cp += xcf_write_int32 (info->fp, &auto_all, 1);
 
   {
     PixelArea area;
@@ -1063,6 +1213,7 @@ xcf_save_row (XcfInfo *info,
 	info->cp += xcf_write_int8 (info->fp, data, num_channels * w);
       }
       break;
+    case PRECISION_BFP:
     case PRECISION_U16:
     case PRECISION_FLOAT16:
       {
@@ -1307,7 +1458,7 @@ xcf_load_layer_props (XcfInfo *info,
 		      Layer   *layer)
 {
   PropType prop_type;
-  guint32 prop_size;
+  guint32 prop_size, temp;
 
   while (1)
     {
@@ -1353,9 +1504,75 @@ xcf_load_layer_props (XcfInfo *info,
 	case PROP_MODE:
 	  info->cp += xcf_read_int32 (info->fp, (guint32*) &layer->mode, 1);
 	  break;
+	case PROP_SPLINES_START:
+	  if(!xcf_spline)
+	    {
+	      info->cp += xcf_read_int32 (info->fp, (guint32*)&temp, 1);
+	      xcf_spline=(Spline*)malloc(sizeof(Spline));
+	      DL_init((DL_node*)xcf_spline);
+	      xcf_spline->points=NULL;
+	      DL_append(layer->sl.splines, (DL_node*)xcf_spline);
+	    }
+	  else
+	    {
+	      g_message("there is already a spline being init'd.");
+	    }
+	  break;
+	case PROP_SPLINES_CONTINUITY:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&xcf_spline->continuity, 1);
+	  break;
+	case PROP_SPLINES_KEYFRAME:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&xcf_spline->keyframe, 1);
+	  break;
+	case PROP_SPLINES_ACTIVE:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&xcf_spline->active, 1);
+	  break;
+	case PROP_SPLINES_TYPE:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&xcf_spline->type, 1);
+	  break;
+	case PROP_SPLINES_DRAW_POINTS:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&xcf_spline->draw_points, 1);
+	  break;
+	case PROP_SPLINES_POINTS_START:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&temp, 1);
+	  break;
+	case PROP_SPLINES_SAVE_POINT:
+	  {
+	    SplinePoint *point;
+	    gint32 x, y, cx1, cx2, cy1, cy2, number_points;
+	    int i;
+
+	    info->cp += xcf_read_int32(info->fp, (guint32*)&number_points, 1);
+	    xcf_spline->points=(DL_list*)malloc(sizeof(DL_list));
+	    DL_init((DL_node*)xcf_spline->points);
+	    for (i = 0; i < number_points; i++)
+	      {
+		info->cp += xcf_read_int32(info->fp, (guint32*)&x, 1);
+		info->cp += xcf_read_int32(info->fp, (guint32*)&y, 1);
+		info->cp += xcf_read_int32(info->fp, (guint32*)&cx1, 1);
+		info->cp += xcf_read_int32(info->fp, (guint32*)&cy1, 1);
+		info->cp += xcf_read_int32(info->fp, (guint32*)&cx2, 1);
+		info->cp += xcf_read_int32(info->fp, (guint32*)&cy2, 1);
+		point=(SplinePoint*)malloc(sizeof(SplinePoint));
+		point->x=x;
+		point->y=y;
+		point->cx1=cx1;
+		point->cy1=cy1;
+		point->cx2=cx2;
+		point->cy2=cy2;
+		DL_append(xcf_spline->points, (DL_node*)point);
+	      }
+	  }
+	  break;
+	case PROP_SPLINES_POINTS_END:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&temp, 1);
+	  break;
+	case PROP_SPLINES_END:
+	  info->cp += xcf_read_int32(info->fp, (guint32*)&temp, 1);
+	  xcf_spline=NULL;
+	  break;
 	default:
 	  g_message ("unexpected/unknown layer property: %d (skipping)", prop_type);
-
 	  {
 	    guint8 buf[16];
 	    guint amount;
@@ -1683,14 +1900,18 @@ xcf_load_hierarchy (XcfInfo     *info,
   int bytes;
   StorageType storage;
   AutoAlloc auto_alloc;
+  guint32 stora;
+  guint32 auto_all;
 
   info->cp += xcf_read_int32 (info->fp, (guint32*) &w, 1);
   info->cp += xcf_read_int32 (info->fp, (guint32*) &h, 1);
   info->cp += xcf_read_int32 (info->fp, (guint32*) &tag, 1);
   info->cp += xcf_read_int32 (info->fp, (guint32*) &bytes, 1);
-  info->cp += xcf_read_int32 (info->fp, (guint32*) &storage, 1);
-  info->cp += xcf_read_int32 (info->fp, (guint32*) &auto_alloc, 1);
-  
+  info->cp += xcf_read_int32 (info->fp, (guint32*) &stora, 1);
+  info->cp += xcf_read_int32 (info->fp, (guint32*) &auto_all, 1);
+  storage = stora;
+  auto_alloc = auto_all;
+
   {
     void * pag;
     PixelArea area;
@@ -1751,6 +1972,7 @@ xcf_load_row (XcfInfo *info,
 	info->cp += xcf_read_int8 (info->fp, data, num_channels * w);
       }
       break;
+    case PRECISION_BFP:
     case PRECISION_U16:
     case PRECISION_FLOAT16:
       {
