@@ -48,6 +48,10 @@
 #define BUSY_ON    2
 #define BUSY_DOWN  3
 
+#ifdef DEBUG
+#define DISP_DEBUG
+#endif
+
 static guint gdisp_busy = BUSY_OFF;
 
 int middle_mouse_button; 
@@ -111,8 +115,12 @@ gdisplay_menubar_down(
 {
   GDisplay * disp;
   disp = (GDisplay *)user_data;   
-  gdisplay_set_menu_sensitivity (disp);
-  zoom_set_focus(disp);
+
+  if(gdisplay_to_ID(disp) >= -1) {
+    gdisplay_set_menu_sensitivity (disp);
+    zoom_set_focus(disp);
+  } else
+    g_warning("%s:%d %s() gdisplay not found",__FILE__,__LINE__,__func__);
 
   return FALSE;
 }
@@ -508,7 +516,10 @@ gdisplay_canvas_events (GtkWidget *canvas,
             mevent->y = ty;
           }
 
-          (* active_tool->motion_func) (active_tool, mevent, gdisp);
+#         ifndef DISP_DEBUG
+          if(active_tool->gdisp_ptr == gdisp)
+#         endif
+            (* active_tool->motion_func) (active_tool, mevent, gdisp);
         }
       }
       else if ((mevent->state & GDK_BUTTON2_MASK) && scrolled)
@@ -528,7 +539,11 @@ gdisplay_canvas_events (GtkWidget *canvas,
       {
         case GDK_Left: case GDK_Right:
         case GDK_Up: case GDK_Down:
-                     if (active_tool && !gimage_is_empty (gdisp->gimage))
+                     if (active_tool && !gimage_is_empty (gdisp->gimage)
+#                        ifndef DISP_DEBUG
+                         && active_tool->gdisp_ptr == gdisp
+#                        endif
+                        )
                        (* active_tool->arrow_keys_func) (active_tool, kevent, gdisp);
                      return_val = TRUE;
                      break;
@@ -594,7 +609,10 @@ gdisplay_canvas_events (GtkWidget *canvas,
       GdkEventMotion me;
       me.x = tx;  me.y = ty;
       me.state = state;
-      (* active_tool->cursor_update_func) (active_tool, &me, gdisp);
+#     ifndef DISP_DEBUG
+      if(active_tool->gdisp_ptr == gdisp)
+#     endif
+        (* active_tool->cursor_update_func) (active_tool, &me, gdisp);
     }
     else if (gdisp_busy == BUSY_DOWN)
     {
