@@ -927,7 +927,7 @@ file_load (char *filename, char* raw_filename, GDisplay *gdisplay)
   int return_val;
   int i;
 
-  // can't load without a filename
+  /* can't load without a filename */
   if (!filename)
      return FALSE;
 
@@ -975,8 +975,10 @@ file_load (char *filename, char* raw_filename, GDisplay *gdisplay)
 	   (cms_default_image_profile_name != NULL)) 
       {   gimage_set_cms_profile(gimage, cms_get_profile_from_file(cms_default_image_profile_name));
       }
-      else if (cms_open_action == CMS_ASSIGN_PROMPT)
-      {   cms_open_assign_dialog(gimage);
+      /* assign proof profile allways */
+      if (!gimage_get_cms_proof_profile(gimage) &&
+           (cms_default_proof_profile_name != NULL)) 
+      {   gimage_set_cms_proof_profile(gimage, cms_get_profile_from_file(cms_default_proof_profile_name));
       }
  		          
       /* if a workspace profile is given and it's not equal the 
@@ -986,8 +988,8 @@ file_load (char *filename, char* raw_filename, GDisplay *gdisplay)
           if ((gimage_get_cms_profile(gimage) != workspace_profile) &&
 	      ((cms_mismatch_action == CMS_MISMATCH_AUTO_CONVERT)  || 
 	       ((cms_mismatch_action == CMS_MISMATCH_PROMPT) && (cms_convert_on_open_prompt(gimage)==TRUE))))
-	  {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
-	  }
+          {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
+          }
       } 
 
       /*  display the image */
@@ -1053,14 +1055,20 @@ file_load_without_display (char *filename, char* raw_filename, GDisplay *gdispla
       gimage_clean_all (gimage);
       /* assign profile according to cms_open_action */
       if (!gimage_get_cms_profile(gimage) &&
-	   (cms_open_action == CMS_ASSIGN_DEFAULT) &&
+           (cms_open_action == CMS_ASSIGN_DEFAULT) &&
 	   (cms_default_image_profile_name != NULL)) 
       {   gimage_set_cms_profile(gimage, cms_get_profile_from_file(cms_default_image_profile_name));
       }
       else if (cms_open_action == CMS_ASSIGN_PROMPT)
       {   cms_open_assign_dialog(gimage);
       }
- 		          
+
+      /* assign proof profile allways */
+      if (!gimage_get_cms_proof_profile(gimage) &&
+           (cms_default_proof_profile_name != NULL)) 
+      {   gimage_set_cms_proof_profile(gimage, cms_get_profile_from_file(cms_default_proof_profile_name));
+      }
+
       /* if a workspace profile is given and it's not equal the 
 	 image profile, check whether to convert */
       if ((cms_workspace_profile_name != NULL) && (gimage_get_cms_profile(gimage) != NULL))
@@ -1133,23 +1141,28 @@ file_open (char *filename, char* raw_filename)
 
       /* assign profile according to cms_open_action */
       if (!gimage_get_cms_profile(gimage) &&
-	   (cms_open_action == CMS_ASSIGN_DEFAULT) &&
-	   (cms_default_image_profile_name != NULL)) 
+           (cms_open_action == CMS_ASSIGN_DEFAULT) &&
+           (cms_default_image_profile_name != NULL)) 
       {   gimage_set_cms_profile(gimage, cms_get_profile_from_file(cms_default_image_profile_name));
       }
       else if (cms_open_action == CMS_ASSIGN_PROMPT)
       {   cms_open_assign_dialog(gimage);
       }
- 		          
+      /* assign proof profile allways */
+      if (!gimage_get_cms_proof_profile(gimage) &&
+           (cms_default_proof_profile_name != NULL)) 
+      {   gimage_set_cms_proof_profile(gimage, cms_get_profile_from_file(cms_default_proof_profile_name));
+      }
+
       /* if a workspace profile is given and it's not equal the 
-	 image profile, check whether to convert */
+         image profile, check whether to convert */
       if ((cms_workspace_profile_name != NULL) && (gimage_get_cms_profile(gimage) != NULL))
       {   CMSProfile *workspace_profile = cms_get_profile_from_file(cms_workspace_profile_name);
           if ((gimage_get_cms_profile(gimage) != workspace_profile) &&
-	      ((cms_mismatch_action == CMS_MISMATCH_AUTO_CONVERT)  || 
-	       ((cms_mismatch_action == CMS_MISMATCH_PROMPT) && (cms_convert_on_open_prompt(gimage)==TRUE))))
-	  {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
-	  }
+               ((cms_mismatch_action == CMS_MISMATCH_AUTO_CONVERT)  || 
+               ((cms_mismatch_action == CMS_MISMATCH_PROMPT) && (cms_convert_on_open_prompt(gimage)==TRUE))))
+          {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
+          }
       } 
 
       /*  display the image */
@@ -1232,9 +1245,9 @@ file_save (int   image_ID,
  if (gimage_is_layered (gimage) && !save_layer && !save_copy)
   if (strcmp(file_proc->extensions, "rll") && strcmp(file_proc->extensions, "xcf"))
     {
-      //INAGEWORKS Removed RH code aland
-      //g_message ("Can't save layered image except as xcf or rll Choose Flatten first");
-      //return TRUE; 
+      /*INAGEWORKS Removed RH code aland */
+      /*g_message ("Can't save layered image except as xcf or rll Choose Flatten first"); */
+      /*return TRUE;  */
     }
 
   proc = &file_proc->db_info;
@@ -2002,10 +2015,10 @@ file_update_menus (GSList *procs,
 
       if (file_proc->db_info.proc_type != PDB_EXTENSION)
 	{
-      //IMAGEWORKS removes lock aland
-	  //if(is_layered && strcmp(file_proc->db_info.name, "gimp_xcf_save"))
-	  //  menus_set_sensitive (file_proc->menu_path, FALSE);
-	  //else 
+      /*IMAGEWORKS removes lock aland */
+	  /*if(is_layered && strcmp(file_proc->db_info.name, "gimp_xcf_save")) */
+	  /*  menus_set_sensitive (file_proc->menu_path, FALSE); */
+	  /*else  */
 	    menus_set_sensitive (file_proc->menu_path, (file_proc->image_types_val & image_type));
 	}
      
@@ -2019,6 +2032,7 @@ char *
 file_temp_name(char *extension) 
 {   static gint id = 0;
     static gint pid;
+    char *file_name;
 
     GString *s = g_string_new (NULL);
 
@@ -2030,17 +2044,18 @@ file_temp_name(char *extension)
 		      id++, extension);
 
 
-    char *file_name = strdup(s->str);  
+    file_name = strdup(s->str);  
     g_string_free (s, FALSE);
     return file_name;
 }
 
 void
 file_temp_clear() 
-{   DIR *dir;
+{   DIR *dir=0;
     struct dirent *entry;
 
-    dir = opendir (temp_path);
+    if(temp_path)
+        dir = opendir (temp_path);
     if (!dir)
     {   g_warning ("Can not open temp directory %s to clear temp files", temp_path);
 	return;
@@ -2160,7 +2175,7 @@ file_reload_warning_callback (GtkWidget *w,
   Channel *channel;
   Channel *channel2;
 
- int i; 
+  int i; 
   Layer *layer, *layer2; 
   
   menus_set_sensitive ("<Image>/File/Revert to Saved", TRUE);
