@@ -76,7 +76,7 @@ static void	  gdisplay_draw_cursor	    (GDisplay *);
 static void       gdisplay_display_area     (GDisplay *, int, int, int, int);
 static guint      gdisplay_hash             (GDisplay *);
 /* updates the projection */
-static void       gdisplay_project          (GDisplay *, int, int, int, int);
+/*static void       gdisplay_project          (GDisplay *, int, int, int, int);*/
 
 static GHashTable *display_ht = NULL;
 
@@ -137,6 +137,9 @@ gdisplay_new (GImage       *gimage,
   gdisp->proximity = FALSE;
   gdisp->have_cursor = FALSE;
 
+  gdisp->expose = EXPOSE_DEFAULT;
+  gdisp->offset = OFFSET_DEFAULT;
+  gdisp->gamma = GAMMA_DEFAULT;
   gdisp->cms_intent = cms_default_intent;
   gdisp->cms_flags = cms_default_flags;
   gdisp->look_profile_pipe = NULL;
@@ -167,9 +170,15 @@ gdisplay_new (GImage       *gimage,
 
     while (cur_wid_list != NULL) { 
 
-      /* d_printf("Adding factory component: %s\n", 
-       * gtk_type_query(GTK_OBJECT_TYPE(cur_wid_list->data))->type_name); */
-      g_hash_table_insert(display_ht, cur_wid_list->data, gdisp); 
+      if( strcmp("GtkTearoffMenuItem",
+          gtk_type_query(GTK_OBJECT_TYPE(cur_wid_list->data))->type_name) != 0)
+      {
+#       ifdef DEBUG
+        printf("%s:%d %s() Adding factory component: %s\n", __FILE__,__LINE__,__func__,
+          gtk_type_query(GTK_OBJECT_TYPE(cur_wid_list->data))->type_name);
+#       endif
+        g_hash_table_insert(display_ht, cur_wid_list->data, gdisp); 
+      }
       cur_wid_list = cur_wid_list->next;
     }
     cur_slist = cur_slist->next;
@@ -180,22 +189,6 @@ gdisplay_new (GImage       *gimage,
 
   gdisplays_update_title (gimage->ID); 
 
-  /* initialize the projection */
-  Tag t = gimage_tag (gdisp->gimage);
-  t = tag_set_alpha (t, ALPHA_YES);
-  if (tag_format (t) == FORMAT_INDEXED)
-  {   t = tag_set_format (t, FORMAT_RGB);  
-  }
-  gdisp->projection = canvas_new (t, 
-				    gdisp->gimage->width, gdisp->gimage->height,
-#ifdef NO_TILES						  
-				    STORAGE_FLAT);
-#else
-                                    STORAGE_TILED);
-#endif
-
-  /* calculate the projection */
-  gdisplay_project(gdisp, 0, 0, gimage->width, gimage->height);
 
   if (gdisplay_set_colormanaged(gdisp, cms_manage_by_default) == FALSE)	 
   {   g_message("Cannot colormanage this display.\nEither image or display profile are undefined.\n. To avoid this message switch off automatic colormanagement in File/Preferences/Color Management");
@@ -224,24 +217,31 @@ gdisplay_format_title (GImage *gimage,
 {
   int empty = gimage_is_empty (gimage);
   Tag t = gimage_tag (gimage);
+  const char *colour = 0;
   char *alpha_msg = ""; /*"no alpha";*/
+  const char *profile;
+
   if (layer_has_alpha(gimage->active_layer)){
     alpha_msg = "A"; /*"alpha";*/
   }
 
-  const char *profile;
   if (gimage_get_cms_profile(gimage) != NULL)
   {   profile = cms_get_profile_info(gimage_get_cms_profile(gimage))->description;
+      colour = cms_get_profile_info(gimage->cms_profile)->color_space_name;
+      if(strstr(colour,"Cmyk"))
+          alpha_msg = "";
   }
   else 
   {   profile = "[no icc profile assigned]";
   }
 
+
+
   sprintf (title, "%s-%d.%d (%s - %s%s - %s)%s %d%%",
 	   prune_filename (gimage_filename (gimage)),
 	   gimage->ID, gimage->instance_count,
 	   tag_string_precision (tag_precision (t)),
-	   tag_string_format (tag_format (t)),
+	   colour ? colour : tag_string_format (tag_format (t)),
 	   alpha_msg,
 	   profile,
            empty ? " (empty)" : "",
@@ -256,9 +256,53 @@ static void
 gdisplay_delete (GDisplay *gdisp)
 {
   GDisplay *tool_gdisp;
+  GSList *cur_slist;
+  GSList *cur_wid_list;
+  GtkItemFactoryItem * fac_item;
+  int smr=1;
 
   g_hash_table_remove (display_ht, gdisp->shell);
   g_hash_table_remove (display_ht, gdisp->canvas);
+  g_hash_table_remove (display_ht, gdisp->menubar);
+
+  cur_slist = gdisp->menubar_fac->items;
+  while (cur_slist != NULL) {
+    fac_item = (GtkItemFactoryItem *) cur_slist->data; 
+    cur_wid_list = fac_item->widgets;
+
+    while (cur_wid_list != NULL)
+    {
+      char path[64] = "", *ptr=0;
+      const char* type = 0;
+
+      type = gtk_type_query(GTK_OBJECT_TYPE(cur_wid_list->data))->type_name, fac_item->path;
+#     ifdef DEBUG_
+      printf("%s:%d %s() Removing factory component: %s\n",
+      __FILE__,__LINE__,__func__, type?type:"");
+#     endif
+      g_hash_table_remove(display_ht, cur_wid_list->data);
+
+      
+      if(smr && (strstr(type,"MenuItem") || strstr(type,"GtkMenuItem")))
+      {
+        snprintf( path, 64, fac_item->path );
+        ptr = strchr( fac_item->path, '/' );
+        if(ptr) *ptr=0;
+#     ifdef DEBUG
+        printf("%s:%d %s() Removing %s\n",__FILE__,__LINE__,__func__, path);
+#     endif
+       /*menus_destroy (path); */
+        smr = 0;
+      }
+
+      cur_wid_list = cur_wid_list->next;
+    }
+    cur_slist = cur_slist->next;
+  }
+# ifdef DEBUG
+  printf("%s:%d %s() remaining items in display_ht: %d\n",__FILE__,__LINE__,__func__, 
+      g_hash_table_size( display_ht ));
+# endif
 
   /*  stop any active tool  */
   active_tool_control (HALT, (void *) gdisp);
@@ -276,9 +320,6 @@ gdisplay_delete (GDisplay *gdisp)
   /*  free the selection structure  */
   selection_free (gdisp->select);
 
-  /* free the projection */
-  canvas_delete(gdisp->projection); 
-
   /* free the profile pipe */
   g_slist_free(gdisp->look_profile_pipe); 
 
@@ -302,6 +343,10 @@ gdisplay_delete (GDisplay *gdisp)
 
   gtk_widget_unref (gdisp->shell);
 
+# ifdef DEBUG
+  printf("gdisp's in display_list: %d \n",g_slist_length( display_list ));
+# endif
+
   g_free (gdisp);
 
 }
@@ -370,60 +415,13 @@ gdisplay_process_area_list (GSList *list,
   return new_list;
 }
 
-static void 
-gdisplay_project (GDisplay *display, int x, int y, int w, int h) 
-{   /* realloc projection if  necessary */ 
-    if((display->gimage->width != canvas_width(display->projection)) ||
-	    (display->gimage->height != canvas_height(display->projection)) ||
-       (canvas_tag(display->projection) != canvas_tag(gimage_projection(display->gimage))))
-    {   Canvas *new_canvas = canvas_new (canvas_tag(gimage_projection(display->gimage)), 
-					display->gimage->width, display->gimage->height,
-#ifdef NO_TILES						  
-						  STORAGE_FLAT);
-#else
-						  STORAGE_TILED);
-#endif
-     
-        /* if only tag changed, we won't get a full refresh, so copy old data */
-        if (canvas_tag(display->projection) != canvas_tag(gimage_projection(display->gimage)))
-        {   PixelArea src_area;
-            PixelArea dest_area;
-            pixelarea_init (&src_area, display->projection, 0,0,display->gimage->width,
-	  		     display->gimage->height, FALSE);
-	    pixelarea_init (&dest_area, new_canvas, 0,0,display->gimage->width,
-			    display->gimage->height, FALSE);
-	    copy_area(&src_area, &dest_area);
-	}
-        canvas_delete(display->projection);  
-	display->projection = new_canvas;
-    }
-
-
-    /* Stefan Klein: temporary solution, just copy projection calculated in gimage */
-    PixelArea image_projection_area;
-    PixelArea display_projection_area;
-
-    gimage_construct(display->gimage, x,y,w,h);
-    pixelarea_init (&image_projection_area, gimage_projection(display->gimage), x,y,w,h, FALSE);
-    pixelarea_init (&display_projection_area, display->projection, x,y,w,h, TRUE);
-     
-    copy_area (&image_projection_area, &display_projection_area);   
-
-    pixelarea_init (&display_projection_area, display->projection, x,y,w,h, TRUE);
-    /* apply display cms to projection */
-    if (transform_buffer != NULL)
-    {   cms_transform_area(transform_buffer, &display_projection_area, &display_projection_area); 
-    }  
-   
-}
-
-
 
 void
 gdisplay_flush (GDisplay *gdisp)
 {
   GArea  *ga;
   GSList *list;
+  static int flushing = FALSE;
 
   /*  Flush the items in the displays and updates lists--
    *  but only if gdisplay has been mapped and exposed
@@ -431,51 +429,15 @@ gdisplay_flush (GDisplay *gdisp)
   if (!gdisp->select)
     return;
 
+  /*  this prevents multiple recursive calls to this procedure  */
+  if (flushing == TRUE)
+    return;
+
+  flushing = TRUE;
+
   /*  First the updates...  */
   list = gdisp->update_areas;
 
-  /* if colormanaged, get the required transform */
-
-  if ((list !=NULL) && gdisp->colormanaged)
-  {   /* combine profiles */
-      GSList *profiles = NULL;
-      CMSProfile *image_profile = gimage_get_cms_profile(gdisp->gimage);
-      CMSProfile *display_profile = cms_get_display_profile();
-      if (image_profile == NULL)
-      {   g_warning("gdisplay_project: cannot colourmanage, image has no profile assigned");
-          transform_buffer = NULL;
-      }
-      else if (display_profile == NULL)
-      {   g_warning("gdisplay_project: cannot colourmanage, no display profile set");
-          transform_buffer = NULL;
-      }
-      else 
-      {   profiles = g_slist_append(profiles, (gpointer)image_profile);
-          profiles = g_slist_concat(profiles, g_slist_copy(gdisp->look_profile_pipe));
-          profiles = g_slist_append(profiles, (gpointer)display_profile);
-    
-          transform_buffer = cms_get_canvas_transform(profiles,
-                                           gimage_projection(gdisp->gimage),
-	                                   gdisp->cms_intent, gdisp->cms_flags);
-          g_slist_free(profiles);
-      }
-  } 
-  /* if we have got look profiles ... */
-  else if (!gdisp->colormanaged && (gdisp->look_profile_pipe != NULL))
-  {   GSList *profiles = NULL;
-      CMSProfile *image_profile = gimage_get_cms_profile(gdisp->gimage);
-      if (image_profile == NULL)
-      {   image_profile = cms_get_srgb_profile();
-      }
-      profiles = g_slist_append(profiles, (gpointer)image_profile);
-      profiles = g_slist_concat(profiles, g_slist_copy(gdisp->look_profile_pipe));
-      profiles = g_slist_append(profiles, (gpointer)image_profile);
-      transform_buffer = cms_get_canvas_transform(profiles,
-                                           gimage_projection(gdisp->gimage), 
-	                                   gdisp->cms_intent, gdisp->cms_flags);
-      g_slist_free(profiles);
-  }
-
   while (list)
     {
       /*  Paint the area specified by the GArea  */
@@ -533,6 +495,7 @@ gdisplay_flush (GDisplay *gdisp)
   if (gdisp->window_info_dialog)
     info_window_update (gdisp->window_info_dialog,
                         (void *) gdisp);
+  flushing = FALSE;
 }
 
 
@@ -928,9 +891,9 @@ gdisplay_paint_area (GDisplay *gdisp,
   w = (x2 - x1);
   h = (y2 - y1);
 
-  /* re-project the area */
-  gdisplay_project (gdisp, x, y, w, h);
-      
+  /* composite the area */
+  gimage_construct (gdisp->gimage, x, y, w, h);
+
   /*  display the area  */
   gdisplay_transform_coords (gdisp, x, y, &x1, &y1, FALSE);
   gdisplay_transform_coords (gdisp, x + w, y + h, &x2, &y2, FALSE);
@@ -1236,6 +1199,16 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
   Precision precision;
   Tag t;
 
+  if (gdisplay_to_ID (gdisp) < 0)
+  {
+    g_warning("%s:%d %s() Display not found. Maybe it is corrupt: 0x%x", __FILE__,__LINE__,__func__,(int)gdisp);
+    return;
+  }
+#ifdef DEBUG
+  else
+    printf("Display found: %d  0x%x\n", gdisplay_to_ID (gdisp), (int)gdisp);
+#endif
+
   fs = (gimage_floating_sel (gdisp->gimage) != NULL);
   aux = (gimage_get_active_channel (gdisp->gimage) != NULL);
   if ((layer = gimage_get_active_layer (gdisp->gimage)) != NULL)
@@ -1274,6 +1247,7 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
   menus_set_sensitive (buff, (format != FORMAT_RGB));
   g_snprintf(buff, 1024, "<Image%d>/Image/Grayscale", gdisp->unique_id);
   menus_set_sensitive (buff, (format != FORMAT_GRAY));
+
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Threshold", gdisp->unique_id);
   menus_set_sensitive (buff, (format != FORMAT_INDEXED));
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Posterize", gdisp->unique_id);
@@ -1296,12 +1270,15 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
 
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Color Balance", gdisp->unique_id);
   menus_set_sensitive (buff, (format == FORMAT_RGB));
+  g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Color Correction", gdisp->unique_id);
+  menus_set_sensitive (buff, (format == FORMAT_RGB));
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Brightness-Contrast", gdisp->unique_id);
   menus_set_sensitive (buff, (format != FORMAT_INDEXED));
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Hue-Saturation", gdisp->unique_id);
   menus_set_sensitive (buff, (format == FORMAT_RGB));
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Levels", gdisp->unique_id);
   menus_set_sensitive (buff, (format != FORMAT_INDEXED));
+
   g_snprintf(buff, 1024, "<Image%d>/Select", gdisp->unique_id);
   menus_set_sensitive (buff, lp);
   g_snprintf(buff, 1024, "<Image%d>/Edit/Cut", gdisp->unique_id);
@@ -1373,6 +1350,7 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
   menus_set_sensitive ("<Image>/Image/Colors/Invert", (format != FORMAT_INDEXED));
 
   menus_set_sensitive ("<Image>/Image/Colors/Color Balance", (format == FORMAT_RGB));
+  menus_set_sensitive ("<Image>/Image/Colors/Color Correction", (format == FORMAT_RGB));
   menus_set_sensitive ("<Image>/Image/Colors/Brightness-Contrast", (format != FORMAT_INDEXED));
   menus_set_sensitive ("<Image>/Image/Colors/Hue-Saturation", (format == FORMAT_RGB));
 #if 0
@@ -1428,8 +1406,14 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
       menus_set_state ("<Image>/View/Rendering Intent/Absolute Colorimetric", TRUE);
   }
   g_snprintf(buff, 1024, "<Image%d>/View/Colormanage Display", gdisp->unique_id);
-  menus_set_state (buff, gdisp->colormanaged);
-  menus_set_state ("<Image>/View/Colormanage Display", gdisp->colormanaged);
+  menus_set_state (buff, gdisp->colormanaged && format == FORMAT_RGB);
+  menus_set_state ("<Image>/View/Colormanage Display", gdisp->colormanaged && format == FORMAT_RGB);
+  g_snprintf(buff, 1024, "<Image%d>/View/Proof Display", gdisp->unique_id);
+  menus_set_state (buff,  (gdisp->cms_flags & cmsFLAGS_SOFTPROOFING) != 0);
+  menus_set_state ("<Image>/View/Proof Display", (gdisp->cms_flags & cmsFLAGS_SOFTPROOFING) != 0);
+  g_snprintf(buff, 1024, "<Image%d>/View/Gamut Check Display", gdisp->unique_id);
+  menus_set_state (buff, (gdisp->cms_flags & cmsFLAGS_GAMUTCHECK) != 0);
+  menus_set_state ("<Image>/View/Gamut Check Display", (gdisp->cms_flags & cmsFLAGS_GAMUTCHECK) != 0);
 
   menus_set_state ("<Image>/View/Toggle Rulers", GTK_WIDGET_VISIBLE (gdisp->origin) ? 1 : 0);
   menus_set_state ("<Image>/View/Toggle Guides", gdisp->draw_guides);
@@ -1512,11 +1496,14 @@ gdisplay_active ()
 
   if (display_ht) {
     while (cur_widget != NULL) {
-      /*d_printf("%s\n", gtk_type_query(GTK_OBJECT_TYPE(cur_widget))->type_name);*/
       gdisp = g_hash_table_lookup (display_ht, cur_widget);
 
-      if (gdisp)
+      if (gdisp) {
+#       ifdef DEBUG
+        printf("%s:%d %s() %s -> %d  0x%x\n",__FILE__,__LINE__,__func__, gtk_type_query(GTK_OBJECT_TYPE(cur_widget))->type_name, gdisp?gdisp->ID:-1, (int)gdisp);
+#       endif
         return gdisp;
+      }
 
       cur_widget = cur_widget->parent;
     }
@@ -1528,13 +1515,18 @@ gdisplay_active ()
       return gdisp;
     }
 
+# ifdef DEBUG
+  printf("%s:%d %s() could not associate event %s with a gdisplay in display_ht with entries: %d\n",
+  __FILE__,__LINE__,__func__,gtk_type_query(GTK_OBJECT_TYPE(event_widget))->type_name,
+   g_hash_table_size(display_ht));
+# endif
   return NULL;
 }
 
 void
 gdisplays_delete_image (GImage *image)
 {
-  GDisplay *gdisp;
+GDisplay *gdisp;
   GSList *list = display_list;
 
   while (list)
@@ -1570,11 +1562,6 @@ gdisplay_get_unique_id (int ID)
   return NULL;
 }
 
-Canvas * 
-gdisplay_get_projection (GDisplay *display)
-{   return display->projection;
-}
-
 GDisplay *
 gdisplay_get_ID (int ID)
 {
@@ -1595,6 +1582,28 @@ gdisplay_get_ID (int ID)
   return NULL;
 }
 
+int
+gdisplay_to_ID (GDisplay *test_disp)
+{
+  GDisplay *gdisp;
+  GSList *list = display_list;
+
+  /*  Traverse the list of displays, returning the one that matches the ID  */
+  /*  If no display in the list is a match, return -1.                    */
+  while (list)
+    {
+      gdisp = (GDisplay *) list->data;
+#     ifdef DEBUG
+      printf("%s:%d %s() ID = %d  0x%x\n",__FILE__,__LINE__,__func__,gdisp->ID, (int)gdisp);
+#     endif
+      if (gdisp == test_disp)
+        return gdisp->ID;
+
+      list = g_slist_next (list);
+    }
+
+  return -1;
+}
 
 void
 gdisplays_update_title (int ID)
@@ -1929,7 +1938,7 @@ gdisplay_set_cms_flags(GDisplay *display, DWORD flags)
 {   display->cms_flags = flags;
 }
 
-gboolean       
+gboolean
 gdisplay_set_colormanaged(GDisplay *display, gboolean colormanaged)
 {   if (colormanaged != display->colormanaged)
     {   if ((colormanaged == TRUE) && (gimage_get_cms_profile(display->gimage) == NULL))
@@ -1942,11 +1951,6 @@ gdisplay_set_colormanaged(GDisplay *display, gboolean colormanaged)
 	}
 
 	display->colormanaged = colormanaged;
-	/* set the toggle menu */
-	GString *buffer = g_string_new(NULL);
-	g_string_sprintf(buffer, "<Image%d>/View/Colormanage Display", display->unique_id);
-	menus_set_state (buffer->str, colormanaged);
-	g_string_free(buffer, TRUE);
 
 	/* and update */
 	gdisplay_update_full (display);
@@ -1956,18 +1960,71 @@ gdisplay_set_colormanaged(GDisplay *display, gboolean colormanaged)
     return TRUE;
 }
 
+gboolean
+gdisplay_set_proofed(GDisplay *display, gboolean set)
+{   if (set != (display->cms_flags & cmsFLAGS_SOFTPROOFING))
+    {   if(set == TRUE)
+          if(gdisplay_set_colormanaged(display, set) == FALSE)
+            return FALSE;
+        if ((set == TRUE) && (gimage_get_cms_proof_profile(display->gimage) == NULL))
+        {   g_warning("gdisplay_set_proofed: cannot proof. no proof profile defined");
+            return FALSE;
+        }
+        if ((set == TRUE) && (cms_display_profile_name == NULL))
+        {   g_warning("gdisplay_set_proofed: cannot colormanage. no display profile defined");  
+            return FALSE;
+        }
+
+        display->cms_flags = set ? display->cms_flags | cmsFLAGS_SOFTPROOFING :
+                                 display->cms_flags & (~cmsFLAGS_SOFTPROOFING) ;
+
+        /* and update */
+        gdisplay_update_full (display);
+        gdisplay_flush (display);
+    }
+
+    return TRUE;
+}
+
+gboolean
+gdisplay_set_gamutchecked(GDisplay *display, gboolean set)
+{   if (set != (display->cms_flags & cmsFLAGS_SOFTPROOFING))
+    {   if(set == TRUE)
+          if(gdisplay_set_colormanaged(display, set) == FALSE)
+            return FALSE;
+        if ((set == TRUE) && (gimage_get_cms_proof_profile(display->gimage) == NULL))
+        {   g_warning("gdisplay_set_gamutchecked: cannot proof. no proof profile defined");
+            return FALSE;
+        }
+        if ((set == TRUE) && (cms_display_profile_name == NULL))
+        {   g_warning("gdisplay_set_gamutchecked: cannot colormanage. no display profile defined");  
+            return FALSE;
+        }
+
+        display->cms_flags = set ? display->cms_flags | cmsFLAGS_GAMUTCHECK :
+                                 display->cms_flags & (~cmsFLAGS_GAMUTCHECK) ;
+
+        /* and update */
+        gdisplay_update_full (display);
+        gdisplay_flush (display);
+    }
+
+    return TRUE;
+}
+
 gboolean 
 gdisplay_all_set_colormanaged(gboolean colormanaged)
-{   if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
-    {   g_warning("gdisplay_all_set_colormanaged: cannot colormanage. no display profile defined");    
-        return FALSE;
-    }
-    
+{
     gboolean some_without_profile = FALSE; 
     GSList *list = display_list;
     GDisplay *current_display;
     GString *buffer = g_string_new(NULL);
-
+    
+    if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
+    {   g_warning("gdisplay_all_set_colormanaged: cannot colormanage. no display profile defined");    
+        return FALSE;
+    }
+    
     /*  traverse the linked list of displays  */
     while (list)
     {   current_display = (GDisplay *) list->data;
@@ -1978,9 +2035,6 @@ gdisplay_all_set_colormanaged(gboolean colormanaged)
 	}
         
 	current_display->colormanaged = colormanaged;
-	/* set the toggle menu */
-	g_string_sprintf(buffer, "<Image%d>/View/Colormanage Display", current_display->unique_id);
-	menus_set_state (buffer->str, colormanaged);
 
 	/* and update */
 	gdisplay_update_full (current_display);
@@ -1999,16 +2053,17 @@ gdisplay_all_set_colormanaged(gboolean colormanaged)
 
 gboolean
 gdisplay_image_set_colormanaged(int image_id, gboolean colormanaged)
-{   if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
-    {   g_warning("gdisplay_image_set_colormanaged: cannot colormanage. no display profile defined");    
-        return FALSE;
-    }
-    
+{
     gboolean some_without_profile = FALSE; 
     GSList *list = display_list;
     GDisplay *current_display;
     GString *buffer = g_string_new(NULL);
 
+    if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
+    {   g_warning("gdisplay_image_set_colormanaged: cannot colormanage. no display profile defined");    
+        return FALSE;
+    }
+    
     /*  traverse the linked list of displays  */
     while (list)
     {   current_display = (GDisplay *) list->data;
@@ -2020,9 +2075,6 @@ gdisplay_image_set_colormanaged(int image_id, gboolean colormanaged)
 	    }
         
 	    current_display->colormanaged = colormanaged;
-	    /* set the toggle menu */
-	    g_string_sprintf(buffer, "<Image%d>/View/Colormanage Display", current_display->unique_id);
-	    menus_set_state (buffer->str, colormanaged);
 
 	    /* and update */
 	    gdisplay_update_full (current_display);
@@ -2039,6 +2091,88 @@ gdisplay_image_set_colormanaged(int image_id, gboolean colormanaged)
     return TRUE;
 }
 
+gboolean
+gdisplay_image_set_proofed(int image_id, gboolean set)
+{
+    gboolean some_without_profile = FALSE;
+    GSList *list = display_list;
+    GDisplay *current_display;
+    GString *buffer = g_string_new(NULL);
+
+    if (set == TRUE)
+      if (!gdisplay_image_set_colormanaged( image_id, set))
+        return FALSE;
+
+    /*  traverse the linked list of displays  */
+    while (list)
+    {   current_display = (GDisplay *) list->data;
+        list = g_slist_next (list);
+        if (current_display->gimage->ID == image_id)
+        {   if ((set == TRUE) && (gimage_get_cms_proof_profile(current_display->gimage) == NULL))
+            {   some_without_profile = TRUE;
+                continue;
+            }
+        
+            current_display->cms_flags = set ?
+                         current_display->cms_flags | cmsFLAGS_SOFTPROOFING :
+                         current_display->cms_flags & (~cmsFLAGS_SOFTPROOFING) ;
+
+            /* and update */
+            gdisplay_update_full (current_display);
+            gdisplay_flush (current_display);
+        }
+    }
+    g_string_free(buffer, TRUE);
+    
+    if (some_without_profile)
+    {   g_warning("gdisplay_image_set_proofed: cannot colormanage some displays, because the image doesn't have a proof profile assigned.");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+gboolean
+gdisplay_image_set_gamutchecked(int image_id, gboolean set)
+{
+    gboolean some_without_profile = FALSE;
+    GSList *list = display_list;
+    GDisplay *current_display;
+    GString *buffer = g_string_new(NULL);
+
+    if (set == TRUE)
+      if (!gdisplay_image_set_colormanaged( image_id, set))
+        return FALSE;
+
+    /*  traverse the linked list of displays  */
+    while (list)
+    {   current_display = (GDisplay *) list->data;
+        list = g_slist_next (list);
+        if (current_display->gimage->ID == image_id)
+        {   if ((set == TRUE) && (gimage_get_cms_proof_profile(current_display->gimage) == NULL))
+            {   some_without_profile = TRUE;
+                continue;
+            }
+        
+            current_display->cms_flags = set ?
+                           current_display->cms_flags | cmsFLAGS_GAMUTCHECK :
+                           current_display->cms_flags & (~cmsFLAGS_GAMUTCHECK) ;
+
+            /* and update */
+            gdisplay_update_full (current_display);
+            gdisplay_flush (current_display);
+        }
+    }
+    g_string_free(buffer, TRUE);
+    
+    if (some_without_profile)
+    {   g_warning("gdisplay_image_set_gamutchecked: cannot colormanage some displays, because the image doesn't have a proof profile assigned.");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 
 /* look profiles */
 void
