@@ -77,7 +77,6 @@ struct GDisplay
   GtkAdjustment *vsbdata;         /*  vertical data information               */
 
   GImage *gimage;	          /*  pointer to the associated gimage struct */
-  Canvas *projection;             /*  the image's projection -layers+channels */
   int instance;                   /*  the instance # of this gdisplay as      */
                                   /*  taken from the gimage at creation       */
 
@@ -122,11 +121,14 @@ struct GDisplay
   base_frame_manager *bfm; 	  /* link to the frame manager */ 
   LookProfileManager *look_profile_manager;   /* the look profile manager */
 
+  float    expose;                 /* expose float HDR */ 
+  float    offset;                 /* offset float HDR */ 
+  float    gamma;                  /* gamma for float HDR */
   
   gboolean colormanaged;           /* is the display being colourmanaged ? */
-  DWORD  cms_flags;                 /* littlecms flags for display cms */
-  int   cms_intent;                /* littlecms rendering intent */
-  GSList *look_profile_pipe;       /* the pipeline of look profiles image data gets
+  DWORD    cms_flags;              /* littlecms flags for display cms */
+  int      cms_intent;             /* littlecms rendering intent */
+  GSList  *look_profile_pipe;      /* the pipeline of look profiles image data gets
 				      sent through */
   
 };
@@ -169,7 +171,6 @@ GDisplay * gdisplay_get_from_gimage        (GImage *image);
 GDisplay * gdisplay_active                 (void);
 GDisplay * gdisplay_get_ID                 (int);
 GDisplay * gdisplay_get_unique_id          (int);
-Canvas   * gdisplay_get_projection         (GDisplay *);
 void       gdisplays_update_title          (int);
 void       gdisplay_add_update_area  (GDisplay *, int, int, int, int);
 void       gdisplays_update_area           (int, int, int, int, int);
@@ -189,6 +190,7 @@ void 	   gdisplays_delete_image 	   (GImage *image);
 void       gdisplays_flush                 (void);
 
 GDisplay* gdisplay_find_display (int ID);
+int        gdisplay_to_ID                  (GDisplay *);
 
 /* color management getter/setter methods */
 #define    gdisplay_get_cms_intent(display) (display)->cms_intent
@@ -196,11 +198,17 @@ void       gdisplay_set_cms_intent          (GDisplay *, int);
 #define    gdisplay_get_cms_flags(display)  (display)->cms_flags
 void       gdisplay_set_cms_flags           (GDisplay *, DWORD);
 #define    gdisplay_is_colormanaged(display)(display)->colormanaged
+#define    gdisplay_is_proofed(display)(((display)->cms_flags & cmsFLAGS_SOFTPROOFING)?1:0)
+#define    gdisplay_is_gamutchecked(display)(((display)->cms_flags & cmsFLAGS_GAMUTCHECK)?1:0)
 gboolean   gdisplay_set_colormanaged        (GDisplay *, gboolean);
+gboolean   gdisplay_set_proofed        (GDisplay *, gboolean);
+gboolean   gdisplay_set_gamutchecked        (GDisplay *, gboolean);
 /* change color management for all displays */
 gboolean   gdisplay_all_set_colormanaged    (gboolean);
 /* change color management for all displays of a given image */
 gboolean   gdisplay_image_set_colormanaged  (int, gboolean);
+gboolean   gdisplay_image_set_proofed       (int, gboolean);
+gboolean   gdisplay_image_set_gamuchecked   (int, gboolean);
 
 /* look profiles */
 #define    gdisplay_get_look_profile_pipe(display) (display)->look_profile_pipe
