@@ -27,5 +27,13 @@ extern ProcRecord gdisplay_delete_proc;
 extern ProcRecord gdisplays_delete_image_proc;
 extern ProcRecord gdisplay_update_proc;
 extern ProcRecord gdisplays_flush_proc;
+extern ProcRecord gdisplay_get_cms_intent_proc;
+extern ProcRecord gdisplay_set_cms_intent_proc;
+extern ProcRecord gdisplay_get_cms_flags_proc;
+extern ProcRecord gdisplay_set_cms_flags_proc;
+extern ProcRecord gdisplay_is_colormanaged_proc;
+extern ProcRecord gdisplay_set_colormanaged_proc;
+extern ProcRecord gdisplay_all_set_colormanaged_proc;
+extern ProcRecord gdisplay_image_set_colormanaged_proc;
 
 #endif  /*  __GDISPLAY_CMDS_H__  */
