@@ -49,6 +49,10 @@
 #include "layer_pvt.h"
 #include "channel_pvt.h"		/* ick ick. */
 
+#define TEST_ALPHA_COPY 0
+
+#define _(translation_text) translation_text
+
 /*  Local function declarations  */
 static void     gimage_free_projection       (GImage *);
 static GImage * gimage_create                (void);
@@ -165,9 +169,11 @@ gimage_create (void)
   gimage->comp_preview_valid[0] = FALSE;
   gimage->comp_preview_valid[1] = FALSE;
   gimage->comp_preview_valid[2] = FALSE;
+  gimage->comp_preview_valid[3] = TRUE;
   gimage->comp_preview = NULL;
 
   gimage->cms_profile = NULL;  
+  gimage->cms_proof_profile = NULL;  
 
   image_list = g_slist_append (image_list, (void *) gimage);
 
@@ -335,6 +341,7 @@ gimage_copy  (GImage *gimage)
   new_gimage->comp_preview_valid[0] = FALSE;
   new_gimage->comp_preview_valid[1] = FALSE;
   new_gimage->comp_preview_valid[2] = FALSE;
+  new_gimage->comp_preview_valid[3] = (tag_alpha (gimage->tag) == ALPHA_YES);
   new_gimage->comp_preview = NULL;
 
   image_list = g_slist_append (image_list, (void *) new_gimage);
@@ -446,6 +453,7 @@ gimage_create_from_layer  (GImage *gimage)
   new_gimage->comp_preview_valid[0] = FALSE;
   new_gimage->comp_preview_valid[1] = FALSE;
   new_gimage->comp_preview_valid[2] = FALSE;
+  new_gimage->comp_preview_valid[3] = !(tag_alpha (gimage->tag) == ALPHA_YES);
   new_gimage->comp_preview = NULL;
 
   image_list = g_slist_append (image_list, (void *) new_gimage);
@@ -684,6 +692,7 @@ gimage_scale (GImage *gimage, int new_width, int new_height)
   layer_invalidate_previews (gimage->ID);
   gimage_invalidate_preview (gimage);
   gdisplays_update_gimage (gimage->ID);
+  gdisplays_flush();
   gdisplays_shrink_wrap (gimage->ID);
 
   zoom_view_changed(gdisplay_get_from_gimage(gimage));
@@ -805,6 +814,9 @@ gimage_delete (GImage *gimage)
       if (gimage->cms_profile != NULL) 
       {   cms_return_profile(gimage->cms_profile);
       }
+      if (gimage->cms_proof_profile != NULL) 
+      {   cms_return_profile(gimage->cms_proof_profile);
+      }
 
       g_free (gimage);
 
@@ -1135,10 +1147,10 @@ project_intensity (GImage *gimage, Layer *layer,
 {
   gint  affect[4] = {1,1,1,1};
 
-  if (! gimage->construct_flag)
+  if (! gimage->construct_flag) {
     initial_area (src, dest, mask, NULL, layer->opacity,
 		    layer->mode, affect, INITIAL_INTENSITY);
-  else
+  } else
     combine_areas (dest, src, dest, mask, NULL, layer->opacity,
 		     layer->mode, affect, COMBINE_INTEN_A_INTEN);
 }
@@ -1593,6 +1605,10 @@ gimage_get_component_active (GImage *gimage, ChannelType type)
     case Red:     return gimage->active[RED_PIX]; 
     case Green:   return gimage->active[GREEN_PIX]; 
     case Blue:    return gimage->active[BLUE_PIX]; 
+#if TEST_ALPHA_COPY
+    case Matte:   return (tag_alpha(gimage->tag) == ALPHA_YES) ?
+                         gimage->active[ALPHA_PIX] : 0;
+#endif
     case Gray:    return gimage->active[GRAY_PIX]; 
     case Indexed: return gimage->active[INDEXED_PIX]; 
     default:      return 0; 
@@ -1609,6 +1625,10 @@ gimage_get_component_visible (GImage *gimage, ChannelType type)
     case Red:     return gimage->visible[RED_PIX]; 
     case Green:   return gimage->visible[GREEN_PIX]; 
     case Blue:    return gimage->visible[BLUE_PIX]; 
+#if TEST_ALPHA_COPY
+    case Matte:   return (tag_alpha(gimage->tag) == ALPHA_YES) ?
+                         gimage->visible[ALPHA_PIX] : 0;
+#endif
     case Gray:    return gimage->visible[GRAY_PIX]; 
     case Indexed: return gimage->visible[INDEXED_PIX]; 
     default:      return 0; 
@@ -1717,6 +1737,11 @@ gimage_set_active_channel (GImage *gimage, Channel * channel)
       gimage->active[RED_PIX] = 1;
       gimage->active[GREEN_PIX] = 1;
       gimage->active[BLUE_PIX] = 1;
+#if TEST_ALPHA_COPY
+      gimage->active[ALPHA_PIX] = (tag_alpha (gimage->tag) == ALPHA_YES);
+#else
+      gimage->active[ALPHA_PIX] = 1;
+#endif
       break;
     }
 
@@ -1752,10 +1777,15 @@ gimage_set_component_active (GImage *gimage, ChannelType type, int value)
     case Red:     gimage->active[RED_PIX] = value; break;
     case Green:   gimage->active[GREEN_PIX] = value; break;
     case Blue:    gimage->active[BLUE_PIX] = value; break;
+#if TEST_ALPHA_COPY
+    case Matte:   gimage->active[ALPHA_PIX] =tag_alpha(gimage->tag) == ALPHA_YES
+                                             ? value : 0; break;
+#else
+    case Matte:   break;
+#endif
     case Gray:    gimage->active[GRAY_PIX] = value; break;
     case Indexed: gimage->active[INDEXED_PIX] = value; break;
     case Auxillary: break;
-    case Matte:   break;
     }
 
   /*  If there is an active channel and we mess with the components,
@@ -1781,6 +1811,9 @@ gimage_set_component_visible (GImage *gimage, ChannelType type, int value)
     case Red:     gimage->visible[RED_PIX] = value; break;
     case Green:   gimage->visible[GREEN_PIX] = value; break;
     case Blue:    gimage->visible[BLUE_PIX] = value; break;
+#if TEST_ALPHA_COPY
+    case Matte:   gimage->visible[ALPHA_PIX] = value; break;
+#endif
     case Gray:    gimage->visible[GRAY_PIX] = value; break;
     case Indexed: gimage->visible[INDEXED_PIX] = value; break;
     default:      break;
@@ -2064,7 +2097,7 @@ gimage_merge_layers (GImage *gimage, GSList *merge_list, MergeType merge_type)
   int x3, y3, x4, y4;
   int operation;
   int position;
-  int active[MAX_CHANNELS] = {1, 1, 1, 1};
+  int active[MAX_CHANNELS] = {1, 1, 1, 0};
   int off_x, off_y;
 
   layer = NULL;
@@ -3096,6 +3129,7 @@ static int
 gimage_is_flat (GImage *gimage)
 {
   int flat = FALSE;
+  int something_invisible;
 
   /*  What makes a flat image?
    *  1) the solitary layer is exactly gimage-sized and placed
@@ -3130,6 +3164,41 @@ gimage_is_flat (GImage *gimage)
             }
         }
     }
+
+    switch(tag_format (gimage->tag))
+    {
+    case FORMAT_RGB:
+    {
+	int red_vis, green_vis, blue_vis, matte_vis = 1;
+	red_vis = gimage_get_component_visible (gimage, RED_PIX); 
+	green_vis = gimage_get_component_visible (gimage, GREEN_PIX); 
+	blue_vis = gimage_get_component_visible (gimage, BLUE_PIX);
+        if(tag_alpha(gimage->tag) == ALPHA_YES)
+          matte_vis = gimage_get_component_visible (gimage, ALPHA_PIX);
+        else
+          matte_vis = 1;
+	if( !red_vis || !green_vis || !blue_vis
+#if TEST_ALPHA_COPY
+            || !matte_vis
+#endif
+          )
+	  something_invisible = TRUE;
+	else 
+	  something_invisible = FALSE;
+    }
+    break; 
+    case FORMAT_GRAY:
+    {
+	int gray_vis;
+	gray_vis = gimage_get_component_visible (gimage, GRAY_PIX);
+	if( !gray_vis )
+	  something_invisible = TRUE;
+	else 
+	  something_invisible = FALSE;
+    }
+    }
+    if (something_invisible)
+      flat = FALSE;
   
   if ((flat == TRUE) && (gimage->projection))
     {
@@ -3143,6 +3212,18 @@ gimage_is_flat (GImage *gimage)
   return flat;
 }
 
+const char**
+gimage_get_channel_names (GImage *gimage)
+{ 
+  CMSProfile *profile = gimage_get_cms_profile(gimage);
+  static const char* color_space_channel_names_[8] = {_("Red"),_("Green"),_("Blue"),_("Alpha")};
+  const char** color_space_channel_names = color_space_channel_names_;
+  if(profile)
+      color_space_channel_names =
+       cms_get_profile_info(profile)->color_space_channel_names;
+
+  return color_space_channel_names;
+}
 
 int
 gimage_is_layered (GImage *gimage)
@@ -3180,7 +3261,8 @@ gimage_active_drawable (GImage *gimage)
   else
     {
     return NULL;
-}}
+    }
+}
 
 char *
 gimage_filename (GImage *gimage)
@@ -3463,6 +3545,14 @@ gimage_composite_preview (GImage *gimage, ChannelType type,
     case Red:     channel = RED_PIX; break;
     case Green:   channel = GREEN_PIX; break;
     case Blue:    channel = BLUE_PIX; break;
+#if TEST_ALPHA_COPY
+    case Matte:
+                  if(tag_alpha(gimage->tag) == ALPHA_YES) 
+                    channel = ALPHA_PIX;
+                  else
+                    return NULL;
+                  break;
+#endif
     case Gray:    channel = GRAY_PIX; break;
     case Indexed: channel = INDEXED_PIX; break;
     default: return NULL;
@@ -3500,6 +3590,13 @@ gimage_preview_valid (gimage, type)
     case Red:     return gimage->comp_preview_valid [RED_PIX]; 
     case Green:   return gimage->comp_preview_valid [GREEN_PIX]; 
     case Blue:    return gimage->comp_preview_valid [BLUE_PIX]; 
+#if TEST_ALPHA_COPY
+    case Matte: 
+                  if(tag_alpha(gimage->tag) == ALPHA_YES) 
+                    return gimage->comp_preview_valid[ALPHA_PIX];
+                  else
+                    return TRUE;
+#endif
     case Gray:    return gimage->comp_preview_valid [GRAY_PIX]; 
     case Indexed: return gimage->comp_preview_valid [INDEXED_PIX]; 
     default: return TRUE;
@@ -3517,6 +3614,12 @@ gimage_invalidate_preview (GImage *gimage)
   gimage->comp_preview_valid[0] = FALSE;
   gimage->comp_preview_valid[1] = FALSE;
   gimage->comp_preview_valid[2] = FALSE;
+#if TEST_ALPHA_COPY
+  if(tag_alpha(gimage->tag) == ALPHA_YES) 
+    gimage->comp_preview_valid[3] = FALSE;
+  else
+    gimage->comp_preview_valid[3] = TRUE;
+#endif
 }
 
 void
@@ -3541,22 +3644,31 @@ gimage_adjust_background_layer_visibility (GImage *gimage)
   Layer *layer = gimage_get_active_layer (gimage);
   Channel * channel = gimage_get_active_channel (gimage);
   gint something_visible;
-#if 0
-  d_printf ("gimage_adjust_background_layer_visibility\n");	
-  d_printf ("layer is %x\n", (gint)layer);	
+#if DEBUG
+  printf("%s:%d gimage_adjust_background_layer_visibility\n",__FILE__,__LINE__);
+  printf ("layer is %x\n", (gint)layer);	
 #endif
   switch ( f )
   {
     case FORMAT_RGB:
     {
-	int red_vis, green_vis, blue_vis;
+	int red_vis, green_vis, blue_vis, matte_vis = 0;
 	red_vis = gimage_get_component_visible (gimage, RED_PIX); 
 	green_vis = gimage_get_component_visible (gimage, GREEN_PIX); 
 	blue_vis = gimage_get_component_visible (gimage, BLUE_PIX); 
-#if 0
-	d_printf (" red is %d green is %d blue is %d\n", red_vis, green_vis, blue_vis);
+#if TEST_ALPHA_COPY
+        if(tag_alpha(gimage->tag) == ALPHA_YES)
+          matte_vis = gimage_get_component_visible (gimage, ALPHA_PIX);
 #endif
-	if( red_vis || green_vis || blue_vis )
+#if DEBUG
+	printf ("%s:%d red is %d green is %d blue is %d\n",__FILE__,__LINE__,
+            red_vis, green_vis, blue_vis);
+#endif
+	if( red_vis || green_vis || blue_vis
+#if TEST_ALPHA_COPY
+            || matte_vis
+#endif
+          )
 	  something_visible = TRUE;
 	else 
 	  something_visible = FALSE;
@@ -3624,18 +3736,29 @@ gimage_set_cms_profile(GImage *image, CMSProfile *profile)
     gdisplays_update_title(image->ID);
 }
 
+void
+gimage_set_cms_proof_profile(GImage *image, CMSProfile *profile)
+{   image->cms_proof_profile = profile;    
+    gdisplays_update_gimage(image->ID);
+    gdisplays_update_title(image->ID);
+}
+
 void gimage_transform_colors(GImage *image, CMSTransform *transform, gboolean allow_undo)
-{   if (allow_undo)
+{
+    GSList *list = NULL;
+    Layer* floating_layer;
+    PixelArea transform_area;        
+
+    if (allow_undo)
     {   undo_push_group_start (image, GIMAGE_MOD_UNDO);
     }
 
     /*  Relax the floating selection  */
-    Layer* floating_layer = gimage_floating_sel (image);
+    floating_layer = gimage_floating_sel (image);
     if (floating_layer)
     {   floating_sel_relax (floating_layer, TRUE);
     }
   
-    GSList *list = NULL;
     /*  Convert all layers  */
     for (list = image->layers; list; list = g_slist_next (list))
     {   /* get the original layer */
@@ -3653,7 +3776,6 @@ void gimage_transform_colors(GImage *image, CMSTransform *transform, gboolean al
 	}
 
         /* get the transform and convert */
-        PixelArea transform_area;        
         pixelarea_init(&transform_area, tiles,0, 0, 
 	               drawable_width(GIMP_DRAWABLE(layer)), 
  	               drawable_height(GIMP_DRAWABLE(layer)), TRUE);
