@@ -39,6 +39,18 @@
 #define HORIZONTAL_GUIDE 1
 #define VERTICAL_GUIDE   2
 
+/* expose / gamma defaults */
+#define EXPOSE_DEFAULT 0
+#define OFFSET_DEFAULT 0
+#define GAMMA_DEFAULT 1
+#define EXPOSE_MIN -20
+#define EXPOSE_MAX 20
+#define OFFSET_MIN 0
+#define OFFSET_MAX 12
+#define GAMMA_MIN 0.0
+#define GAMMA_MAX 8
+
+
 typedef enum
 {
   ExpandAsNecessary,
@@ -116,13 +128,14 @@ struct GImage
 
                                       /*  Composite preview  */
   Canvas *comp_preview;       /*  the composite preview        */
-  int comp_preview_valid[3];          /*  preview valid-1/channel      */
+  int comp_preview_valid[4];          /*  preview valid-1/channel      */
   float xresolution;                  /*  image x-res, in dpi          */
   float yresolution;                  /*  image y-res, in dpi          */
 
   char onionskin;		      /*  if onion skin is in use      */ 
 
   CMSProfile *cms_profile;           /* the input profile for this image's data */
+  CMSProfile *cms_proof_profile;     /* the proof profile for this image's data */
 };
 
 
@@ -242,9 +255,12 @@ Canvas         *gimage_construct_composite_preview    (GImage *, int, int);
 GImage *        drawable_gimage               (CanvasDrawable*);
 
 /* colour management */
+#define  gimage_get_cms_proof_profile(image) (image)->cms_proof_profile
+void gimage_set_cms_proof_profile(GImage *image, CMSProfile *profile);
 #define  gimage_get_cms_profile(image) (image)->cms_profile
 void gimage_set_cms_profile(GImage *image, CMSProfile *profile);
 void gimage_transform_colors(GImage *image, CMSTransform *transform, 
 			     gboolean allow_undo);
+const char**    gimage_get_channel_names      (GImage *gimage); /* 4 pices */
 
 #endif /* __GIMAGE_H__ */
