@@ -80,6 +80,7 @@ static void                   histogram_tool_value_callback   (GtkWidget *, gpoi
 static void                   histogram_tool_red_callback     (GtkWidget *, gpointer);
 static void                   histogram_tool_green_callback   (GtkWidget *, gpointer);
 static void                   histogram_tool_blue_callback    (GtkWidget *, gpointer);
+static void                   histogram_tool_alpha_callback    (GtkWidget *, gpointer);
 
 static void *histogram_tool_options = NULL;
 static HistogramToolDialog *histogram_tool_dialog = NULL;
@@ -97,6 +98,16 @@ static char * histogram_info_names[7] =
   "Percentile: "
 };
 
+static MenuItem color_option_items[] =
+{
+  { "Value", 0, 0, histogram_tool_value_callback, NULL, NULL, NULL },
+  { "Red", 0, 0, histogram_tool_red_callback, NULL, NULL, NULL },
+  { "Green", 0, 0, histogram_tool_green_callback, NULL, NULL, NULL },
+  { "Blue", 0, 0, histogram_tool_blue_callback, NULL, NULL, NULL },
+  { "Alpha", 0, 0, histogram_tool_alpha_callback, NULL, NULL, NULL },
+  { NULL, 0, 0, NULL, NULL, NULL, NULL }
+};
+
 
 /*  histogram_tool action functions  */
 
@@ -202,6 +213,7 @@ histogram_tool_initialize (void *gdisp_ptr)
 {
   GDisplay *gdisp;
   gint	    bins;
+  int       init = 0;
 
   gdisp = (GDisplay *) gdisp_ptr;
 
@@ -233,9 +245,10 @@ histogram_tool_initialize (void *gdisp_ptr)
   histogram_histogram_funcs (gimage_tag (gdisp->gimage));
 
   /*  The histogram_tool dialog  */
-  if (!histogram_tool_dialog)
+  if (!histogram_tool_dialog) {
     histogram_tool_dialog = histogram_tool_new_dialog ( bins );
-  else if (!GTK_WIDGET_VISIBLE (histogram_tool_dialog->shell))
+    init = 1;
+  } else if (!GTK_WIDGET_VISIBLE (histogram_tool_dialog->shell))
     gtk_widget_show (histogram_tool_dialog->shell);
 
   histogram_tool_dialog->drawable = gimage_active_drawable (gdisp->gimage);
@@ -247,6 +260,66 @@ histogram_tool_initialize (void *gdisp_ptr)
   else
     gtk_widget_hide (histogram_tool_dialog->channel_menu);
 
+  {
+    HistogramToolDialog* dialog = histogram_tool_dialog;
+    const char** color_space_channel_names = gimage_get_channel_names(gdisp->gimage);
+
+    GtkOptionMenu *option_menu = GTK_OPTION_MENU (dialog->channel_menu);
+    GtkMenu *m = GTK_MENU(gtk_option_menu_get_menu(option_menu));
+    GtkWidget *w = gtk_menu_get_active(m);
+    MenuItem *items = color_option_items;
+    int pos = -1;
+    int i;
+    int n_items = get_menu_num_items(items);
+    int is_Lab = 0;
+
+    if(gdisp->gimage->cms_profile)
+      is_Lab = strstr("Lab",cms_get_profile_info(gdisp->gimage->cms_profile)
+                              ->color_space_name) ? 1 : 0;
+
+    pos = update_color_menu_items (items,
+                                   GTK_OPTION_MENU(dialog->channel_menu),
+                                   color_space_channel_names);
+
+    if (is_Lab) {
+      if (pos == 0)
+        ++pos;
+    }
+    else {
+      if (pos == 5)
+        pos = 0;
+    }
+
+    if (is_Lab) {
+      gtk_widget_hide( items[0].widget);
+    } else {
+      gtk_widget_show( items[0].widget);
+    }
+
+    if (!init || is_Lab)
+      items[pos].callback (w, dialog);
+    /* set the current selection */
+    /* the following line is causing segfault for some reason....*/
+    gtk_option_menu_set_history ( GTK_OPTION_MENU (option_menu), pos);
+
+    /* check for alpha channel */
+    if (n_items >= 5)
+    { if (drawable_has_alpha ( (dialog->drawable)))
+        gtk_widget_show( items[4].widget);
+      else
+        gtk_widget_hide( items[4].widget);
+    }
+
+    /*  hide or show the channel menu based on image type  */
+    if (dialog->color)
+      for (i = 0; i < n_items; i++)
+         gtk_widget_set_sensitive( items[i].widget, TRUE);
+    else
+      for (i = 1; i < n_items; i++)
+         gtk_widget_set_sensitive( items[i].widget, FALSE);
+  }
+
+
   histogram_update (histogram_tool_dialog->histogram,
 		    histogram_tool_dialog->drawable,
 		    histogram_histogram_info,
@@ -265,15 +338,6 @@ static ActionAreaItem action_items[] =
   { "Close", histogram_tool_close_callback, NULL, NULL }
 };
 
-static MenuItem color_option_items[] =
-{
-  { "Value", 0, 0, histogram_tool_value_callback, NULL, NULL, NULL },
-  { "Red", 0, 0, histogram_tool_red_callback, NULL, NULL, NULL },
-  { "Green", 0, 0, histogram_tool_green_callback, NULL, NULL, NULL },
-  { "Blue", 0, 0, histogram_tool_blue_callback, NULL, NULL, NULL },
-  { NULL, 0, 0, NULL, NULL, NULL, NULL }
-};
-
 
 static HistogramToolDialog *
 histogram_tool_new_dialog ( gint bins )
@@ -284,7 +348,8 @@ histogram_tool_new_dialog ( gint bins )
   GtkWidget *frame;
   GtkWidget *table;
   GtkWidget *label;
-  GtkWidget *option_menu;
+  GtkWidget *channel_hbox;
+  //GtkWidget *option_menu;
   GtkWidget *menu;
   int i;
   int x, y;
@@ -292,7 +357,7 @@ histogram_tool_new_dialog ( gint bins )
   htd = (HistogramToolDialog *) g_malloc (sizeof (HistogramToolDialog));
   htd->channel = HISTOGRAM_VALUE;
 
-  for (i = 0; i < 4; i++)
+  for (i = 0; i < 5; ++i)
     color_option_items [i].user_data = (gpointer) htd;
 
   /*  The shell and main vbox  */
@@ -306,25 +371,33 @@ histogram_tool_new_dialog ( gint bins )
 		      (GtkSignalFunc) histogram_tool_delete_callback,
 		      htd);
 
-  vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 1);
+  vbox = gtk_vbox_new (FALSE, 2);
+  gtk_container_border_width (GTK_CONTAINER (vbox), 2);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (htd->shell)->vbox), vbox, TRUE, TRUE, 0);
 
   /*  The option menu for selecting channels  */
-  htd->channel_menu = gtk_hbox_new (TRUE, 1);
-  gtk_box_pack_start (GTK_BOX (vbox), htd->channel_menu, FALSE, FALSE, 0);
+  //htd->channel_menu = gtk_hbox_new (TRUE, 1);
+  //gtk_box_pack_start (GTK_BOX (vbox), htd->channel_menu, FALSE, FALSE, 0);
+
+  channel_hbox = gtk_hbox_new (TRUE, 2);
+  gtk_box_pack_start (GTK_BOX (vbox), channel_hbox, FALSE, FALSE, 0);
 
   label = gtk_label_new ("Information on Channel: ");
-  gtk_box_pack_start (GTK_BOX (htd->channel_menu), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (channel_hbox), label, FALSE, FALSE, 0);
+  //gtk_box_pack_start (GTK_BOX (htd->channel_menu), label, FALSE, FALSE, 0);
 
   menu = build_menu (color_option_items, NULL);
-  option_menu = gtk_option_menu_new ();
-  gtk_box_pack_start (GTK_BOX (htd->channel_menu), option_menu, FALSE, FALSE, 2);
+  //option_menu = gtk_option_menu_new ();
+  //gtk_box_pack_start (GTK_BOX (htd->channel_menu), option_menu, FALSE, FALSE, 2);
+  htd->channel_menu = gtk_option_menu_new ();
+  gtk_box_pack_start (GTK_BOX (channel_hbox), htd->channel_menu, FALSE, FALSE, 2);
 
   gtk_widget_show (label);
-  gtk_widget_show (option_menu);
+  //gtk_widget_show (option_menu);
+  //gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
   gtk_widget_show (htd->channel_menu);
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
+  gtk_widget_show (channel_hbox);
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (htd->channel_menu), menu);
 
   /*  The histogram tool histogram  */
   hbox = gtk_hbox_new (TRUE, 1);
@@ -460,6 +533,21 @@ histogram_tool_blue_callback (GtkWidget *w,
     }
 }
 
+static void
+histogram_tool_alpha_callback (GtkWidget *w,
+			      gpointer   client_data)
+{
+  HistogramToolDialog *htd;
+
+  htd = (HistogramToolDialog *) client_data;
+
+  if (htd->channel != HISTOGRAM_ALPHA)
+    {
+      htd->channel = HISTOGRAM_ALPHA;
+      histogram_channel (htd->histogram, htd->channel);
+    }
+}
+
 /*  The histogram procedure definition  */
 ProcArg histogram_args[] =
 {
