@@ -227,6 +227,18 @@ install_help (InstallCallback callback)
 		   "\t\twide " PROGRAM_NAME " brushes installation when searching for\n", -1);
   gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
 		   "\t\tbrushes.\n", -1);
+  gtk_text_insert (GTK_TEXT (text), font_emphasis, NULL, NULL,
+		   "curves\n", -1);
+  gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
+		   "\t\tThis is a subdirectory which can be used to store\n", -1);
+  gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
+		   "\t\tuser defined curves.  The default gimprc file\n", -1);
+  gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
+		   "\t\tchecks this subdirectory in addition to the system-\n", -1);
+  gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
+		   "\t\twide " PROGRAM_NAME " curves installation when searching for\n", -1);
+  gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
+		   "\t\tcurves.\n", -1);
   gtk_text_insert (GTK_TEXT (text), font_emphasis, NULL, NULL,
 		   "gradients\n", -1);
   gtk_text_insert (GTK_TEXT (text), font, NULL, NULL,
