@@ -183,6 +183,27 @@ internal_procs_init ()
   procedural_db_register (&gimage_disable_undo_proc); pcount++;
   procedural_db_register (&gimage_clean_all_proc); pcount++;
   procedural_db_register (&gimage_floating_sel_proc); pcount++;
+  procedural_db_register (&gimage_thumbnail_proc); pcount++;
+  procedural_db_register (&gimage_set_icc_profile_by_name_proc); pcount++;
+  procedural_db_register (&gimage_set_icc_profile_by_mem_proc); pcount++;
+  procedural_db_register (&gimage_set_lab_profile_proc); pcount++;
+  procedural_db_register (&gimage_set_xyz_profile_proc); pcount++;
+  procedural_db_register (&gimage_set_srgb_profile_proc); pcount++;
+  procedural_db_register (&gimage_has_icc_profile_proc); pcount++;
+  procedural_db_register (&gimage_get_icc_profile_by_mem_proc); pcount++;
+  procedural_db_register (&gimage_get_icc_profile_info_proc); pcount++;
+  procedural_db_register (&gimage_get_icc_profile_description_proc); pcount++;
+  procedural_db_register (&gimage_get_icc_profile_pcs_proc); pcount++;
+  procedural_db_register (&gimage_get_icc_profile_color_space_name_proc); pcount++;
+  procedural_db_register (&gimage_get_icc_profile_device_class_name_proc); pcount++;
+  procedural_db_register (&gdisplay_get_cms_intent_proc); pcount++;
+  procedural_db_register (&gdisplay_set_cms_intent_proc); pcount++;
+  procedural_db_register (&gdisplay_get_cms_flags_proc); pcount++;
+  procedural_db_register (&gdisplay_set_cms_flags_proc); pcount++;
+  procedural_db_register (&gdisplay_is_colormanaged_proc); pcount++;
+  procedural_db_register (&gdisplay_set_colormanaged_proc); pcount++;
+  procedural_db_register (&gdisplay_all_set_colormanaged_proc); pcount++;
+  procedural_db_register (&gdisplay_image_set_colormanaged_proc); pcount++;
 
   app_init_update_status(NULL, "GImage mask procedures",
 			 pcount/total_pcount);
