@@ -37,9 +37,9 @@
 #include "store_frame_manager.h"
 
 /* rsr: ld error hack, this is a bug: */
-#define sfm_slide_show 0
+/*#define sfm_slide_show 0*/
 
-#if 0
+#if 1
 #define TEAR_OFF
 #endif
 
@@ -171,16 +171,19 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Select/Shrink", NULL, select_shrink_cmd_callback, 0 },
   { "/Select/Save To Channel", NULL, select_save_cmd_callback, 0 },
   { "/Select/By Color...", NULL, select_by_color_cmd_callback, 0 },
-  { "/View/", "plus", view_zoomin_cmd_callback, 0 },
+  { "/View/", "equal", view_zoomin_cmd_callback, 0 },
 #ifdef TEAR_OFF
   { "/View/tearoff1", NULL, NULL, 0, "<Tearoff>" },
 #endif
   { "/View/Colormanage Display", NULL, view_colormanage_display_cmd_callback, 0, "<ToggleItem>" },
+  { "/View/Proof Display", NULL, view_proof_display_cmd_callback, 0, "<ToggleItem>" },
+  { "/View/Gamut Check Display", NULL, view_gamutcheck_display_cmd_callback, 0, "<ToggleItem>" },
   { "/View/Rendering Intent/Perceptual", NULL, view_rendering_intent_cmd_callback, INTENT_PERCEPTUAL, "<RadioItem>"},
   { "/View/Rendering Intent/Relative Colorimetric", NULL, view_rendering_intent_cmd_callback, INTENT_RELATIVE_COLORIMETRIC, "/View/Rendering Intent/Perceptual" },
   { "/View/Rendering Intent/Saturation", NULL, view_rendering_intent_cmd_callback, INTENT_SATURATION, "/View/Rendering Intent/Relative Colorimetric"},
   { "/View/Rendering Intent/Absolute Colorimetric", NULL, view_rendering_intent_cmd_callback, INTENT_ABSOLUTE_COLORIMETRIC, "/View/Rendering Intent/Saturation"},
   { "/View/Look Profiles...", NULL, view_look_profiles_cmd_callback, 0 },
+  { "/View/Expose", NULL, view_expose_hdr_cmd_callback, 0 },
   { "/View/---", NULL, NULL, 0, "<Separator>" },
 
   { "/View/Pan Zoom Window", NULL, view_pan_zoom_window_cb, 0 },
@@ -194,7 +197,7 @@ static GtkItemFactoryEntry image_entries[] =
   { "/View/Zoom Bookmarks/5", "<control>5", view_zoom_bookmark4_cb, 0},
   { "/View/Zoom Bookmarks/Load", NULL, view_zoom_bookmark_load_cb, 0},
   { "/View/Zoom Bookmarks/Save", NULL, view_zoom_bookmark_save_cb, 0},
-  { "/View/Zoom In", "equal", view_zoomin_cmd_callback, 0 },
+  { "/View/Zoom In", "plus", view_zoomin_cmd_callback, 0 },
   { "/View/Zoom Out", "minus", view_zoomout_cmd_callback, 0 },
 #ifdef TEAR_OFF
   { "/View/Zoom/tearoff1", NULL, NULL, 0, "<Tearoff>" },
@@ -231,7 +234,6 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Image/Colors/Brightness-Contrast", NULL, image_brightness_contrast_cmd_callback, 0 },
   { "/Image/Colors/Hue-Saturation", NULL, image_hue_saturation_cmd_callback, 0 },
   { "/Image/Colors/Gamma-Expose", NULL, image_gamma_expose_cmd_callback, 0 },
-  { "/Image/Colors/Expose-Image", NULL, image_expose_image_cmd_callback, 0 },
 #if 1 
   { "/Image/Colors/Curves", NULL, image_curves_cmd_callback, 0 },
 #endif
@@ -260,7 +262,8 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Image/" PRECISION_FLOAT_STRING, NULL, image_convert_depth_cmd_callback, PRECISION_FLOAT },
   { "/Image/---", NULL, NULL, 0, "<Separator>" },
   { "/Image/Convert using ICC Profile...", NULL, image_convert_colorspace_cmd_callback, 0 },
-  { "/Image/Assign ICC Profile...", NULL, image_assign_cms_profile_cmd_callback, 0 },
+  { "/Image/Assign ICC Profile...", NULL, image_assign_cms_profile_cmd_callback, ICC_IMAGE_PROFILE },
+  { "/Image/Assign ICC Proof Profile...", NULL, image_assign_cms_profile_cmd_callback, ICC_PROOF_PROFILE },
   { "/Image/Resize", NULL, image_resize_cmd_callback, 0 },
   { "/Image/Scale", NULL, image_scale_cmd_callback, 0 },
   { "/Image/---", NULL, NULL, 0, "<Separator>" },
@@ -852,7 +855,13 @@ menus_get_image_menubar(GDisplay *gdisp)
 
   g_snprintf(buff, 256, "<Image%d>", gdisp->unique_id);
   gdisp->menubar_fac = gtk_item_factory_new (GTK_TYPE_MENU_BAR, buff, NULL);
-  gtk_item_factory_create_items_ac (gdisp->menubar_fac , n_image_entries,image_entries, NULL, 2);
+
+# ifdef DEBUG_
+  { int i;
+    for(i = 0 ; i < n_image_entries; ++i)
+      printf("%s:%d %s() %s\n",__FILE__,__LINE__,__func__,image_entries[i]); }
+#endif
+  gtk_item_factory_create_items_ac (gdisp->menubar_fac , n_image_entries, image_entries, NULL, 2);
   plug_in_make_image_menu(gdisp->menubar_fac, buff);
 
   if (!gdisp->menubar_fac->widget) {
