@@ -165,14 +165,15 @@ int       color_visible = FALSE;
 int       palette_visible = FALSE;
 int       gradient_visible = FALSE;
 char *    cms_default_image_profile_name = NULL;
+char *    cms_default_proof_profile_name = NULL;
 char *    cms_workspace_profile_name = NULL;
 char *    cms_display_profile_name = NULL;
 int       cms_default_intent = INTENT_PERCEPTUAL;
 int       cms_default_flags = 0;
 int       cms_open_action = 0;
 int       cms_mismatch_action = 0;
-int       cms_manage_by_default = 0;
-int       enable_rulers_on_start=TRUE;
+int       cms_manage_by_default = TRUE;
+int       enable_rulers_on_start = FALSE;
 
 static int get_next_token (void);
 static int peek_next_token (void);
@@ -241,6 +242,7 @@ static ParseFunc funcs[] =
   { "cms-profile-path",      TT_PATH,       &cms_profile_path, NULL },
   { "default-brush",         TT_STRING,     &default_brush, NULL },
   { "cms-default-image-profile-name",TT_STRING,     &cms_default_image_profile_name, NULL },
+  { "cms-default-proof-profile-name",TT_STRING,     &cms_default_proof_profile_name, NULL },
   { "cms-workspace-profile-name",TT_STRING,     &cms_workspace_profile_name, NULL },
   { "cms-display-profile-name",TT_STRING,     &cms_display_profile_name, NULL },
   { "cms-default-intent",    TT_INT,        &cms_default_intent, NULL },
