@@ -196,7 +196,7 @@ sfm_create_gui (GDisplay *disp)
   gtk_widget_reparent (menubar, GTK_WIDGET (GTK_DIALOG (disp->bfm->sfm->shell)->vbox));
   gtk_widget_show (menubar);
 
-  // gtk_window_add_accel_group (GTK_DIALOG (disp->shell), disp->bfm->sfm->accel_group);
+  /* gtk_window_add_accel_group (GTK_DIALOG (disp->shell), disp->bfm->sfm->accel_group); */
   
   /* check buttons */
   hbox = gtk_hbox_new (FALSE, 1);
@@ -2123,6 +2123,58 @@ sfm_store_add_create_dialog (GDisplay *disp)
     }
 }
 
+void 
+sfm_slide_show (GtkWidget *w, gpointer data)
+{
+  GDisplay *display=NULL;
+  GtkWidget *window;
+  static gint win_x, win_y, win_w, win_h, win_d;
+
+  display = gdisplay_active();
+  window = display->shell;
+
+  if(display)
+    {
+      static int rulers_visible = 0;
+      display->slide_show = display->slide_show ? 0 : 1;
+      if(display->slide_show)
+	{
+          if(GTK_WIDGET_VISIBLE(display->hrule))
+            rulers_visible = 1;
+	  gdk_window_get_geometry(window->window, &win_x, &win_y, &win_w, &win_h, &win_d);
+
+	  gtk_widget_hide(display->menubar);
+	  gtk_widget_hide(display->hsb);
+	  gtk_widget_hide(display->vsb);
+	  gtk_widget_hide(display->hrule);
+	  gtk_widget_hide(display->vrule);
+
+	  gdk_window_move_resize (window->window, -50, -75, gdk_screen_width()+92, 
+				  gdk_screen_height()+105);
+	  gtk_window_set_default_size (GTK_WINDOW (window), gdk_screen_width()+92,
+				       gdk_screen_height()+105);
+	  gtk_widget_set_usize (GTK_WIDGET(window), gdk_screen_width()+92, 
+				gdk_screen_height()+105);
+	}
+      else
+	{
+	  gdk_window_move_resize (window->window, win_x, win_y, win_w, win_h);
+	  gtk_window_set_default_size (GTK_WINDOW (window), win_w, win_h);
+	  gtk_widget_set_usize (GTK_WIDGET(window), win_w, win_h);
+
+	  gtk_widget_show(display->menubar);
+	  gtk_widget_show(display->hsb);
+	  gtk_widget_show(display->vsb);
+          if(rulers_visible)
+          {
+            gtk_widget_show(display->hrule);
+	    gtk_widget_show(display->vrule);
+          }
+	}
+      gdisplays_flush();
+    }
+}
+
 /*
  * CHANGE FRAME NUMBER 
  */
@@ -2153,51 +2205,6 @@ sfm_store_change_frame_to (GDisplay* disp)
       disp->ID = disp->gimage->ID;
       item->gimage = disp->gimage;
       sfm_store_make_cur (disp, fm->fg);
-void 
-sfm_slide_show (GtkWidget *w, gpointer data)
-{
-  GDisplay *display=NULL;
-  GtkWidget *window;
-  static gint win_x, win_y, win_w, win_h, win_d;
-
-  display = gdisplay_active();
-  window = display->shell;
-
-  if(display)
-    {
-      display->slide_show = display->slide_show ? 0 : 1;
-      if(display->slide_show)
-	{ 
-	  gdk_window_get_geometry(window->window, &win_x, &win_y, &win_w, &win_h, &win_d);
-
-	  gtk_widget_hide(display->menubar);
-	  gtk_widget_hide(display->hsb);
-	  gtk_widget_hide(display->vsb);
-	  gtk_widget_hide(display->hrule);
-	  gtk_widget_hide(display->vrule);
-
-	  gdk_window_move_resize (window->window, -50, -75, gdk_screen_width()+92, 
-				  gdk_screen_height()+105);
-	  gtk_window_set_default_size (GTK_WINDOW (window), gdk_screen_width()+92,
-				       gdk_screen_height()+105);
-	  gtk_widget_set_usize (GTK_WIDGET(window), gdk_screen_width()+92, 
-				gdk_screen_height()+105);
-	}
-      else
-	{
-	  gdk_window_move_resize (window->window, win_x, win_y, win_w, win_h);
-	  gtk_window_set_default_size (GTK_WINDOW (window), win_w, win_h);
-	  gtk_widget_set_usize (GTK_WIDGET(window), win_w, win_h);
-
-	  gtk_widget_show(display->menubar);
-	  gtk_widget_show(display->hsb);
-	  gtk_widget_show(display->vsb);
-	  gtk_widget_show(display->hrule);
-	  gtk_widget_show(display->vrule);
-	}
-      gdisplays_flush();
-    }
-}
       gtk_clist_set_text (GTK_CLIST (disp->bfm->sfm->store_list), disp->bfm->sfm->fg, 
 	  5, temp_name);
 
@@ -2242,11 +2249,6 @@ sfm_store_change_frame_dialog_ok (GtkWidget *w, gpointer client_data)
     }
 }
 
-static gint
-sfm_store_change_frame_dialog_delete (GtkWidget *w, GdkEvent *e, gpointer data)
-{
-  store_frame_manager *fm = ((GDisplay*) data)->bfm->sfm;
-
 void 
 sfm_filter_apply (GtkWidget *w, gpointer data)
 {
@@ -2287,6 +2289,12 @@ sfm_filter_apply (GtkWidget *w, gpointer data)
     }
 } 
 
+static gint
+sfm_store_change_frame_dialog_delete (GtkWidget *w, GdkEvent *e, gpointer data)
+{
+  store_frame_manager *fm = ((GDisplay*) data)->bfm->sfm;
+
+
   if (fm)
     {
       if (GTK_WIDGET_VISIBLE (fm->chg_frame_dialog))
