@@ -311,7 +311,7 @@ active_tool_free (void)
       tools_free_gamma_expose (active_tool);
       break;
     case EXPOSE_IMAGE:
-      tools_free_expose_image (active_tool);
+      tools_free_expose_view (active_tool);
       break;
     case SPLINE:
       tools_free_spline(active_tool);
@@ -455,7 +455,7 @@ tools_select (ToolType type)
       active_tool = tools_new_gamma_expose ();
       break;
     case EXPOSE_IMAGE:
-      active_tool = tools_new_expose_image ();
+      active_tool = tools_new_expose_view ();
       break;
     case SPLINE:
       active_tool=tools_new_spline();
@@ -676,8 +676,8 @@ tools_initialize (ToolType type, GDisplay *gdisp_ptr)
       break;
     case EXPOSE_IMAGE:
       if (gdisp) {
-	active_tool = tools_new_expose_image ();
-	expose_image_initialize (gdisp);
+	active_tool = tools_new_expose_view ();
+	expose_view_initialize (gdisp);
       } else {
 	active_tool = tools_new_rect_select ();
       }
