@@ -3,7 +3,7 @@
 #ifndef TRACE_H
 #define TRACE_H
 
-#include "../lib/wire/c_typedefs.h"
+#include "wire/c_typedefs.h"
 
 #ifdef WIN32
 
