@@ -836,7 +836,8 @@ gimp_bfm_set_dir_dest (gint32  disp_ID, char   *name)
 
 void
 gimp_image_set_icc_profile_by_name (gint32  image_ID,
-                                    gchar  *data)
+                                    gchar  *data,
+                                    CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -847,6 +848,7 @@ gimp_image_set_icc_profile_by_name (gint32  image_ID,
                                     PARAM_IMAGE, image_ID,
                                     PARAM_INT32, size,
                                     PARAM_INT8ARRAY, data,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
 
@@ -855,8 +857,9 @@ gimp_image_set_icc_profile_by_name (gint32  image_ID,
 
 void
 gimp_image_set_icc_profile_by_mem  (gint32  image_ID,
-		                    gint    size,
-                                    gchar  *data)
+                                    gint    size,
+                                    gchar  *data,
+                                    CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -867,6 +870,7 @@ gimp_image_set_icc_profile_by_mem  (gint32  image_ID,
                                     PARAM_IMAGE, image_ID,
                                     PARAM_INT32, size,
                                     PARAM_INT8ARRAY, data,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   gimp_destroy_params (return_vals, nreturn_vals);
@@ -918,7 +922,8 @@ gimp_image_set_srgb_profile (gint32  image_ID)
 }
 
 gboolean
-gimp_image_has_icc_profile         (gint32  image_ID)
+gimp_image_has_icc_profile         (gint32  image_ID,
+                                    CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -927,6 +932,7 @@ gimp_image_has_icc_profile         (gint32  image_ID)
   return_vals = gimp_run_procedure ("gimp_image_has_icc_profile",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
@@ -941,7 +947,8 @@ gimp_image_has_icc_profile         (gint32  image_ID)
 
 char*
 gimp_image_get_icc_profile_by_mem  (gint32  image_ID,
-		                    gint   *size)
+                                    gint   *size,
+                                    CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -950,6 +957,7 @@ gimp_image_get_icc_profile_by_mem  (gint32  image_ID,
   return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_by_mem",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
@@ -965,7 +973,8 @@ gimp_image_get_icc_profile_by_mem  (gint32  image_ID,
 
 
 char*
-gimp_image_get_icc_profile_info    (gint32  image_ID)
+gimp_image_get_icc_profile_info    (gint32  image_ID,
+                                    CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -975,6 +984,7 @@ gimp_image_get_icc_profile_info    (gint32  image_ID)
   return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_info",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
@@ -990,7 +1000,8 @@ gimp_image_get_icc_profile_info    (gint32  image_ID)
 
 
 char*
-gimp_image_get_icc_profile_description    (gint32  image_ID)
+gimp_image_get_icc_profile_description    (gint32  image_ID,
+                                           CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -1000,6 +1011,7 @@ gimp_image_get_icc_profile_description    (gint32  image_ID)
   return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_description",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
@@ -1015,7 +1027,8 @@ gimp_image_get_icc_profile_description    (gint32  image_ID)
 
 
 char*
-gimp_image_get_icc_profile_pcs     (gint32  image_ID)
+gimp_image_get_icc_profile_pcs     (gint32  image_ID,
+                                    CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -1025,6 +1038,7 @@ gimp_image_get_icc_profile_pcs     (gint32  image_ID)
   return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_pcs",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
@@ -1040,7 +1054,8 @@ gimp_image_get_icc_profile_pcs     (gint32  image_ID)
 
 
 char*
-gimp_image_get_icc_profile_color_space_name (gint32  image_ID)
+gimp_image_get_icc_profile_color_space_name (gint32  image_ID,
+                                             CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -1050,6 +1065,7 @@ gimp_image_get_icc_profile_color_space_name (gint32  image_ID)
   return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_color_space_name",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
@@ -1064,7 +1080,8 @@ gimp_image_get_icc_profile_color_space_name (gint32  image_ID)
 }
 
 char*
-gimp_image_get_icc_profile_device_class_name (gint32  image_ID)
+gimp_image_get_icc_profile_device_class_name (gint32  image_ID,
+                                              CMSProfileType type)
 {
   GParam *return_vals;
   int nreturn_vals;
@@ -1074,6 +1091,7 @@ gimp_image_get_icc_profile_device_class_name (gint32  image_ID)
   return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_device_class_name",
                                     &nreturn_vals,
                                     PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, type,
                                     PARAM_END);
 
   if (return_vals[0].data.d_status == STATUS_SUCCESS)
