@@ -51,14 +51,15 @@
 #endif
 
 #include "plugin_main.h"
-#include "../lib/wire/protocol.h"
-#include "../lib/wire/wire.h"
+#include "wire/protocol.h"
+#include "wire/wire.h"
 /* rsr: License problem? Files from app in lib! */
 #include "../app/plugin_loader.h"
 #include "../app/plug_in.h"
-#include "../lib/wire/wirebuffer.h"
-#include "../lib/wire/taskswitch.h"
-#include "../lib/wire/iodebug.h"
+#include "wire/wirebuffer.h"
+#include "wire/taskswitch.h"
+#include "wire/iodebug.h"
+#include "version.h"
 
 /* Rowe: workaround for config.h clash when making win32 and Linux simultaneously. 
    Normally, the Linux config.h should first be removed */
@@ -66,6 +67,7 @@
 #undef HAVE_SHM_H
 #endif
 
+
 void gimp_extension_process (guint timeout);
 void gimp_extension_ack     (void);
 
@@ -105,11 +107,11 @@ static guint write_buffer_index = 0;
 static GHashTable *temp_proc_ht = NULL;
 static int is_quitting;
 
-GPlugInInfo PLUG_IN_INFO;
+GPlugInInfo PLUG_IN_INFO_LIB;
 
 void set_gimp_PLUG_IN_INFO (const GPlugInInfo *p)
 {
-  PLUG_IN_INFO = *p;
+  PLUG_IN_INFO_LIB = *p;
 }
 
 int
@@ -124,7 +126,7 @@ gimp_main (int   argc,
 
 	progname = argv[0];
 #ifdef WIN32
-	set_gimp_PLUG_IN_INFO(wire_buffer->plugin->PLUG_IN_INFO);
+	set_gimp_PLUG_IN_INFO(wire_buffer->plugin->PLUG_IN_INFO_LIB);
 	_fmode = _O_BINARY;
 #ifdef _DEBUG
 /* This creates a console window and joins it to a GUI process: */
@@ -163,11 +165,11 @@ gimp_main (int   argc,
   if ((argc == 5) && (strcmp (argv[4], "-query") == 0))
 #endif
     {
-	if (PLUG_IN_INFO.query_proc)
-	{	(* PLUG_IN_INFO.query_proc)();
+	if (PLUG_IN_INFO_LIB.query_proc)
+	{	(* PLUG_IN_INFO_LIB.query_proc)();
 	}
 	else
-	{	e_puts("ERROR: PLUG_IN_INFO.query_proc is <null>");
+	{	e_puts("ERROR: PLUG_IN_INFO_LIB.query_proc is <null>");
 	}
 	gimp_quit ();
 	return 0;
@@ -184,8 +186,8 @@ gimp_main (int   argc,
 void G_GNUC_NORETURN
 gimp_quit ()
 {
-  if (PLUG_IN_INFO.quit_proc)
-    (* PLUG_IN_INFO.quit_proc) ();
+  if (PLUG_IN_INFO_LIB.quit_proc)
+    (* PLUG_IN_INFO_LIB.quit_proc) ();
 
 #ifdef HAVE_SHM_H
   if ((_shm_ID != -1) && _shm_addr)
@@ -1037,9 +1039,9 @@ gimp_proc_run (GPProcRun *proc_run)
   GPProcReturn proc_return;
   GParam *return_vals;
   int nreturn_vals;
-  if (PLUG_IN_INFO.run_proc)
+  if (PLUG_IN_INFO_LIB.run_proc)
     {
-      (* PLUG_IN_INFO.run_proc) (proc_run->name,
+      (* PLUG_IN_INFO_LIB.run_proc) (proc_run->name,
 				 proc_run->nparams,
 				 (GParam*) proc_run->params,
 				 &nreturn_vals,
