@@ -21,13 +21,14 @@
 
 
 #include <glib.h>
-#include "../lib/version.h"
-#include "../lib/wire/wire_types.h"
-#include "../lib/wire/libtile.h"
+
+#include "version.h"
+#include "wire/wire_types.h"
+#include "wire/libtile.h"
 #include "float16.h"
 #include "unit.h"
 #include "pdb.h"
-#include "../libgimp/gimp.h"
+#include "libgimp/gimp.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -199,6 +200,12 @@ DLL_API gint     gimp_use_xshm     (void);
 DLL_API guchar*  gimp_color_cube   (void);
 DLL_API gchar*   gimp_gtkrc        (void);
 
+char*      gimp_get_default_monitor_profile   ();
+char*      gimp_get_default_image_profile     ();
+char*      gimp_get_default_workspace_profile ();
+gint32     gimp_get_default_intent_profile    ();
+gint32     gimp_get_default_flags_profile     ();
+
 
 /****************************************
  *              Images                  *
@@ -213,14 +220,14 @@ DLL_API guint      gimp_image_height                (gint32     image_ID);
 DLL_API GImageType gimp_image_base_type             (gint32     image_ID);
 gint32     gimp_image_floating_selection    (gint32     image_ID);
 DLL_API void       gimp_image_add_channel           (gint32     image_ID,
-					     gint32     channel_ID,
-					     gint       position);
+                                                     gint32     channel_ID,
+                                                     gint       position);
 DLL_API void       gimp_image_add_layer             (gint32     image_ID,
-					     gint32     layer_ID,
-					     gint       position);
+                                                     gint32     layer_ID,
+                                                     gint       position);
 DLL_API void       gimp_image_add_layer_mask        (gint32     image_ID,
-					     gint32     layer_ID,
-					     gint32     mask_ID);
+                                                     gint32     layer_ID,
+                                                     gint32     mask_ID);
 DLL_API void       gimp_image_clean_all             (gint32     image_ID);
 DLL_API void       gimp_image_disable_undo          (gint32     image_ID);
 DLL_API void       gimp_image_enable_undo           (gint32     image_ID);
@@ -228,82 +235,91 @@ DLL_API gint32     gimp_image_flatten               (gint32     image_ID);
 gint       gimp_image_is_layered   	    (gint32     image_ID);
 gint       gimp_image_dirty_flag   	    (gint32     image_ID);
 void       gimp_image_lower_channel         (gint32     image_ID,
-					     gint32     channel_ID);
+                                             gint32     channel_ID);
 void       gimp_image_lower_layer           (gint32     image_ID,
-					     gint32     layer_ID);
+                                             gint32     layer_ID);
 gint32     gimp_image_merge_visible_layers  (gint32     image_ID,
-					     GimpMergeType       merge_type);
+                                             GimpMergeType       merge_type);
 gint32     gimp_image_pick_correlate_layer  (gint32     image_ID,
-					     gint       x,
-					     gint       y);
+                                             gint       x,
+                                             gint       y);
 void       gimp_image_raise_channel         (gint32     image_ID,
-					     gint32     channel_ID);
+                                             gint32     channel_ID);
 void       gimp_image_raise_layer           (gint32     image_ID,
-					     gint32     layer_ID);
+                                             gint32     layer_ID);
 void       gimp_image_remove_channel        (gint32     image_ID,
-					     gint32     channel_ID);
+                                             gint32     channel_ID);
 void       gimp_image_remove_layer          (gint32     image_ID,
-					     gint32     layer_ID);
+                                             gint32     layer_ID);
 void       gimp_image_remove_layer_mask     (gint32     image_ID,
-					     gint32     layer_ID,
-					     gint       mode);
-DLL_API void       gimp_image_resize                (gint32     image_ID,
-					     guint      new_width,
-					     guint      new_height,
-					     gint       offset_x,
-					     gint       offset_y);
+                                             gint32     layer_ID,
+                                             gint       mode);
+DLL_API void       gimp_image_resize        (gint32     image_ID,
+                                             guint      new_width,
+                                             guint      new_height,
+                                             gint       offset_x,
+                                             gint       offset_y);
 gint32     gimp_image_get_active_channel    (gint32     image_ID);
 DLL_API gint32     gimp_image_get_active_layer      (gint32     image_ID);
 DLL_API gint32*    gimp_image_get_channels          (gint32     image_ID,
-					     gint      *nchannels);
+                                                     gint      *nchannels);
 DLL_API guchar*    gimp_image_get_cmap              (gint32     image_ID,
-					     gint      *ncolors);
+                                                     gint      *ncolors);
 gint       gimp_image_get_component_active  (gint32     image_ID,
-					     GimpChannelType       component);
+                                             GimpChannelType       component);
 gint       gimp_image_get_component_visible (gint32     image_ID,
-					     GimpChannelType       component);
+                                             GimpChannelType       component);
 gboolean   gimp_image_get_resolution        (gint32     image_ID,
                                              gdouble    *xresolution,
                                              gdouble    *yresolution);
 
 DLL_API char*      gimp_image_get_filename          (gint32     image_ID);
 DLL_API gint32*    gimp_image_get_layers            (gint32     image_ID,
-					     gint      *nlayers);
+                                                     gint      *nlayers);
 gint32     gimp_image_get_selection         (gint32     image_ID);
 void       gimp_image_set_active_channel    (gint32     image_ID,
-					     gint32     channel_ID);
+                                             gint32     channel_ID);
 void       gimp_image_set_active_layer      (gint32     image_ID,
-					     gint32     layer_ID);
-DLL_API void       gimp_image_set_cmap              (gint32     image_ID,
-					     guchar    *cmap,
-					     gint       ncolors);
+                                             gint32     layer_ID);
+DLL_API void       gimp_image_set_cmap      (gint32     image_ID,
+                                             guchar    *cmap,
+                                             gint       ncolors);
 void       gimp_image_set_component_active  (gint32     image_ID,
-					     gint       component,
-					     gint       active);
+                                             gint       component,
+                                             gint       active);
 void       gimp_image_set_component_visible (gint32     image_ID,
-					     gint       component,
-					     gint       visible);
-DLL_API void       gimp_image_set_filename          (gint32     image_ID,
-					     char      *name);
+                                             gint       component,
+                                             gint       visible);
+DLL_API void       gimp_image_set_filename  (gint32     image_ID,
+                                             char      *name);
 void gimp_bfm_set_dir_src    (gint32     disp_ID, char *filename);
 void gimp_bfm_set_dir_dest    (gint32     disp_ID, char *filename);
 
 void       gimp_image_set_icc_profile_by_name (gint32   image_ID,
-                                               gchar   *data);
+                                               gchar   *name,
+                                               CMSProfileType type);
 void       gimp_image_set_icc_profile_by_mem  (gint32   image_ID,
                                                gint     size,
-                                               gchar   *data);
+                                               gchar   *data,
+                                               CMSProfileType type);
 void       gimp_image_set_lab_profile         (gint32   image_ID);
 void       gimp_image_set_xyz_profile         (gint32   image_ID);
 void       gimp_image_set_srgb_profile        (gint32   image_ID);
-gboolean   gimp_image_has_icc_profile         (gint32   image_ID);
+gboolean   gimp_image_has_icc_profile         (gint32   image_ID,
+                                               CMSProfileType type);
 char*      gimp_image_get_icc_profile_by_mem  (gint32   image_ID,
-                                               gint    *size);
-char*      gimp_image_get_icc_profile_description (gint32  image_ID);
-char*      gimp_image_get_icc_profile_info    (gint32   image_ID);
-char*      gimp_image_get_icc_profile_pcs     (gint32   image_ID);
-char*      gimp_image_get_icc_profile_color_space_name  (gint32   image_ID);
-char*      gimp_image_get_icc_profile_device_class_name (gint32   image_ID);
+                                               gint    *size,
+                                               CMSProfileType type);
+char*      gimp_image_get_icc_profile_description (gint32  image_ID,
+                                                   CMSProfileType type);
+char*      gimp_image_get_icc_profile_info    (gint32   image_ID,
+                                               CMSProfileType type);
+char*      gimp_image_get_icc_profile_pcs     (gint32   image_ID,
+                                               CMSProfileType type);
+char*      gimp_image_get_icc_profile_color_space_name  (gint32   image_ID,
+                                                         CMSProfileType type);
+char*      gimp_image_get_icc_profile_device_class_name (gint32   image_ID,
+                                                         CMSProfileType type);
 
 /****************************************
  *             Displays                 *
