@@ -11,7 +11,7 @@
 #include "../win32/unistd.h"
 #endif
 
-#include "../lib/dll_api.h"
+#include "dll_api.h"
 
 
 DLL_API const char* GetDirDot();
