@@ -10,13 +10,13 @@
 #ifdef WIN32
 #include <win32/gdkwin32.h>
 #endif
-#include "../lib/wire/enums.h"
-#include "../lib/wire/datadir.h"
-#include "../lib/wire/c_typedefs.h"
-#include "../lib/intl.h"
-#include "../lib/dialog.h"
-#include "../lib/export.h"
-#include "../lib/image_convert.h"
+#include "wire/enums.h"
+#include "wire/datadir.h"
+#include "wire/c_typedefs.h"
+#include "intl.h"
+#include "dialog.h"
+#include "export.h"
+#include "image_convert.h"
 
 /*#define N_(x)  x
 #define _(x)   x*/
