@@ -27,6 +27,11 @@
 
 #include "dbbrowser_utils.h"
 
+#ifdef WIN32
+#undef g_free
+#define g_free g2_free
+#endif
+
 GList *proc_table;
 
 GtkWidget*
