@@ -424,7 +424,7 @@ load_image (char *filename)
       hProfile = cmsOpenProfileFromMem(Buffer, Len);
       printf ("%s:%d %s() profile found\n",__FILE__,__LINE__,__func__);
       if (hProfile) {
-        gimp_image_set_icc_profile_by_mem (image_ID, Len, Buffer);
+        gimp_image_set_icc_profile_by_mem (image_ID, Len, Buffer, ICC_IMAGE_PROFILE);
         cmsCloseProfile (hProfile);
         free(Buffer);
         printf ("%s:%d %s() set embedded icc profile\n",__FILE__,__LINE__,__func__);
@@ -604,8 +604,8 @@ save_image (char   *filename,
     char *buffer = NULL;
     int size = 0;
 
-    if (gimp_image_has_icc_profile (image_ID)) { 
-      buffer = gimp_image_get_icc_profile_by_mem (image_ID, &size);
+    if (gimp_image_has_icc_profile (image_ID, ICC_IMAGE_PROFILE)) { 
+      buffer = gimp_image_get_icc_profile_by_mem (image_ID, &size, ICC_IMAGE_PROFILE);
       if (buffer) {
         write_icc_profile (&cinfo, buffer, size);
         printf ("%s:%d %s() embedded icc profile\n",__FILE__,__LINE__,__func__);
