@@ -35,10 +35,12 @@
 #include <ImfRgbaFile.h>
 #include <ImfInputFile.h>
 #include <ImfFrameBuffer.h>
+#include <ImfStandardAttributes.h>
 #include <Iex.h>
 #include <ImathBox.h>
 #include <half.h>
 #include "gtk/gtk.h"
+#include "chroma_icc.h"
 extern "C" {
 #include "lib/plugin_main.h"
 #include "lib/wire/libtile.h"
@@ -179,7 +181,7 @@ load_image (char *filename)
     {
 	Imf::RgbaInputFile exr_full (filename);
 	Imf::InputFile exr (filename);
-
+        
 	// Set up progress meter.
 
 	char progress[PATH_MAX];
@@ -285,6 +287,35 @@ load_image (char *filename)
         g_free(pixels);
         delete [] scanline;
 
+        // colour space description -> ICC profile
+	Imf::Chromaticities image_prim;
+	if(hasChromaticities(exr.header()))
+	{
+	  const Imf::Chromaticities &prims = chromaticities (exr.header());
+          image_prim = prims;
+#if 0
+          printf("%s:%d\nred xy: %f %f\ngreen xy: %f %f\nblue xy: %f %f\nwhite xy: %f %f gamma assumed: 1.0\n",__FILE__,__LINE__,
+                 image_prim.red[0],image_prim.red[1],
+                 image_prim.green[0],image_prim.green[1],
+                 image_prim.blue[0],image_prim.blue[1],
+                 image_prim.white[0],image_prim.white[1]);
+        } else {
+          printf("%s:%d standard chrimaticity  gamma assumed: 1.0\n",
+                 __FILE__,__LINE__);
+#endif
+        }
+
+        {
+          size_t size = 0;
+          char* mem_profile = 0;
+          mem_profile = writeICCprims (image_prim, size);
+          if(mem_profile && size)
+            gimp_image_set_icc_profile_by_mem (image_ID,
+                                (guint32)size, mem_profile, ICC_IMAGE_PROFILE);
+
+          free (mem_profile);
+        }
+
         // Update the display
         gimp_drawable_flush(drawable);
         gimp_drawable_detach(drawable);
@@ -353,10 +384,29 @@ save_image (char   *filename,
     {
 	// write an RGBA OpenEXR file with default settings and type HALF
 	// pixels.
+#   if 1
+    Imf::Header header (width, height);
 
+      if (gimp_image_has_icc_profile(image_ID, ICC_IMAGE_PROFILE))
+      {
+        int size;
+        char *mem_profile=NULL;
+
+        mem_profile = gimp_image_get_icc_profile_by_mem(image_ID, &size, ICC_IMAGE_PROFILE);
+        Imf::Chromaticities xyY = writeEXRprims ( mem_profile,  size );
+        addChromaticities(header, xyY);
+        if (!hasChromaticities (header))
+          g_message("Adding of colour primaries to EXR failed.");
+      }
+
+    Imf::RgbaOutputFile exr (filename, header,
+				 nchan == 4 ? Imf::WRITE_RGBA :
+				 Imf::WRITE_RGB);
+#   else
 	Imf::RgbaOutputFile exr (filename, width, height,
 				 nchan == 4 ? Imf::WRITE_RGBA :
 				 Imf::WRITE_RGB);
+#   endif
 
 	// Write a scanline at a time to save memory.  We do this by
 	// setting the frame buffer y stride to 0.
