@@ -1,5 +1,5 @@
 /*
- * "$Id: png.c,v 1.5 2005/05/24 18:57:49 beku Exp $"
+ * "$Id: png.c,v 1.6 2005/06/15 20:36:19 beku Exp $"
  *
  *   Portable Network Graphics (PNG) plug-in for The GIMP -- an image
  *   manipulation program
@@ -703,7 +703,8 @@ load_image (gchar *filename)	/* I - File to load */
   /* set icc profile */
   if (info->iccp_proflen > 0) {
     gimp_image_set_icc_profile_by_mem (image, info->iccp_proflen,
-                                              info->iccp_profile);
+                                              info->iccp_profile,
+                                              ICC_IMAGE_PROFILE);
     printf ("%s:%d %s() set embedded profile \"%s\"\n",
              __FILE__,__LINE__,__func__,
                                               info->iccp_name);
@@ -958,12 +959,14 @@ save_image (gchar  *filename,	        /* I - File to save to */
 
   // write icc profile
 #if defined(PNG_iCCP_SUPPORTED)
-  if (gimp_image_has_icc_profile (image_ID)) {
+  if (gimp_image_has_icc_profile (image_ID, ICC_IMAGE_PROFILE)) {
     int size;
     char *buffer;
     
-    buffer = gimp_image_get_icc_profile_by_mem (image_ID, &size);
-    png_set_iCCP (pp, info, gimp_image_get_icc_profile_description (image_ID),
+    buffer = gimp_image_get_icc_profile_by_mem (image_ID, &size,
+                                                ICC_IMAGE_PROFILE);
+    png_set_iCCP (pp, info,
+           gimp_image_get_icc_profile_description (image_ID, ICC_IMAGE_PROFILE),
                   0, buffer, size);
     printf ("%s:%d %s() embedd icc profile \"%s\"\n",
              __FILE__,__LINE__,__func__,
