@@ -50,7 +50,7 @@
 #endif
 
 #ifdef DEBUG
- #include "m_debug.h"
+ #define DBG printf("%s:%d %s()\n",__FILE__,__LINE__,__func__);
 #else
  #define DBG 
 #endif
@@ -264,12 +264,13 @@ openInputProfile (void)
             || (strcmp (vals.i_profile,"Lab") == 0)) {
       hIn  = cmsCreateLabProfile(cmsD50_xyY());
     } else {
-      if (gimp_image_has_icc_profile (vals.o_image_ID)) {
-        int size = 0;
+      if (gimp_image_has_icc_profile (vals.o_image_ID, ICC_IMAGE_PROFILE)) {
+        gint size = 0;
         char* buf = NULL;
-        buf = gimp_image_get_icc_profile_by_mem (vals.o_image_ID, &size);
+        buf = gimp_image_get_icc_profile_by_mem (vals.o_image_ID, &size,
+                                                 ICC_IMAGE_PROFILE);
         hIn = cmsOpenProfileFromMem (buf, size);
-        free (buf);
+        if( buf && size) free (buf);
       } else {
         hIn  = cmsOpenProfileFromFile(vals.i_profile, "r");
       }
@@ -495,14 +496,24 @@ convert_image ( void )
           hOut  = cmsOpenProfileFromFile("cmyk.icc", "r");
       }
     } else {
-      fp = fopen ( vals.o_profile, "r");
-      if ( fp == NULL ) {
-        g_message ("Profile %s can not be found or is not readable.",
-                    vals.o_profile);
-        return GIMP_PDB_CALLING_ERROR;
+      char* proof_profile_name = get_proof_name();
+      if (proof_profile_name && (strcmp(vals.o_profile, proof_profile_name)) == 0)
+      {
+        gint size = 0;
+        char* data = gimp_image_get_icc_profile_by_mem ( vals.o_image_ID, &size,
+                     ICC_PROOF_PROFILE);
+        hOut = cmsOpenProfileFromMem(data, size);
+        if(data && size) free(data);
       } else {
-        fclose (fp);
-        hOut  = cmsOpenProfileFromFile(vals.o_profile, "r");
+        fp = fopen ( vals.o_profile, "r");
+        if ( fp == NULL ) {
+          g_message ("Profile %s can not be found or is not readable.",
+                      vals.o_profile);
+          return GIMP_PDB_CALLING_ERROR;
+        } else {
+          fclose (fp);
+          hOut  = cmsOpenProfileFromFile(vals.o_profile, "r");
+        }
       }
     }
     if ( hOut == NULL )
@@ -571,10 +582,14 @@ convert_image ( void )
     // TODO merge the linearisation profile in the printer profile
     //#define MERGE_GAMMALUT
     #ifdef MERGE_GAMMALUT
+    #ifndef cmsFLAGS_PRESERVEBLACK
+    #define cmsFLAGS_PRESERVEBLACK            0x8000
+    #endif
     xform = cmsCreateMultiprofileTransform (multi, nr_of_profiles,
                                             wInput, wOutput,
                                             vals.intent, 
-                                            vals.matrix|vals.precalc|vals.bpc);
+                                            vals.matrix|vals.precalc|vals.bpc|
+                                            cmsFLAGS_PRESERVEBLACK);
     if (0) { //TODO Knopf zum verschmelzen
       cmsHPROFILE merge;
 
@@ -732,7 +747,19 @@ create_separated_image      (   void )
   printf ("%s:%d %s()\n",__FILE__,__LINE__,__func__);
   convert_image ( );
 
-  gimp_image_set_icc_profile_by_name (vals.s_image_ID, &vals.o_profile[0]);
+  char* proof_profile_name = get_proof_name();
+
+  if (proof_profile_name && (strcmp(vals.o_profile, proof_profile_name)) == 0)
+  {
+    gint size = 0;
+    char* data = gimp_image_get_icc_profile_by_mem ( vals.o_image_ID, &size,
+                                                     ICC_PROOF_PROFILE);
+    gimp_image_set_icc_profile_by_mem (vals.s_image_ID, size, data,
+                                                     ICC_IMAGE_PROFILE);
+    if(data && size) free(data);
+  } else
+    gimp_image_set_icc_profile_by_name (vals.s_image_ID, &vals.o_profile[0],
+                                        ICC_IMAGE_PROFILE);
   if (vals.show_image) { // show the print preview
     CMYK_display = gimp_display_new (vals.s_image_ID);
     gimp_display_set_cms_intent (CMYK_display, INTENT_RELATIVE_COLORIMETRIC);
@@ -882,6 +909,52 @@ set_tiff_filename                      (int colour_space)
                        vals.tiff_file);
 }
 
+cmsHPROFILE
+get_print_profile()
+{
+  char *description = NULL;
+  cmsHPROFILE hOut = NULL;
+
+  if ( gimp_image_has_icc_profile (vals.o_image_ID, ICC_PROOF_PROFILE))
+  {
+    description = gimp_image_get_icc_profile_description( vals.o_image_ID,
+                                                             ICC_PROOF_PROFILE);
+  }
+
+  if(description &&
+     (strcmp(vals.o_profile, description)) == 0)
+  {
+    gint size = 0;
+    char* data = gimp_image_get_icc_profile_by_mem ( vals.o_image_ID, &size,
+                   ICC_PROOF_PROFILE);
+    hOut = cmsOpenProfileFromMem(data, size);
+    free( data );
+    free( description );
+  } else
+    hOut = cmsOpenProfileFromFile(vals.o_profile, "r");
+
+  return hOut;
+}
+
+char*
+get_proof_name()
+{
+  static char description[MAX_PATH];
+  description[0] = '0';
+
+  if( gimp_image_has_icc_profile (vals.o_image_ID, ICC_PROOF_PROFILE) )
+  {
+    char* text = gimp_image_get_icc_profile_description( vals.o_image_ID,
+                                                             ICC_PROOF_PROFILE);
+    if(text)
+    {
+      sprintf( description, text );
+      free( text );
+    }
+  }
+  return description;
+}
+
 int
 test_separation_profiles               (void)
 {
@@ -898,7 +971,7 @@ test_separation_profiles               (void)
   } else {
 
     // test for valid profile combination and give some informations if fails
-    hOut = cmsOpenProfileFromFile(vals.o_profile, "r");
+    hOut = get_print_profile();
     if (hOut != NULL) {
       if (strcmp(vals.i_profile, ""        ) == 0 ||
           strcmp(vals.i_profile, "sRGB.icc") == 0 ||
@@ -910,7 +983,7 @@ test_separation_profiles               (void)
         }
         hIn_ok = TRUE;
       } else {
-        if (gimp_image_has_icc_profile (vals.o_image_ID)) {
+        if (gimp_image_has_icc_profile (vals.o_image_ID, ICC_IMAGE_PROFILE)) {
           hIn_ok = TRUE;
         } else {
           hIn = cmsOpenProfileFromFile(vals.i_profile, "r");
