@@ -51,7 +51,7 @@ char version_[128] = "ICC print plug-in v0.4.1-alpha";
 #endif
 
 #ifdef DEBUG
- #include "m_debug.h"
+ #define DBG printf("%s:%d %s()\n",__FILE__,__LINE__,__func__);
 #else
  #define DBG 
 #endif
@@ -1019,6 +1019,9 @@ create_lcms_options_window (void)
 
 void init_lcms_options (void)
 {
+  if ( (strcmp(vals.o_profile, "")) == 0)
+    sprintf (vals.o_profile, get_proof_name());
+
 #ifndef USE_ALL_OPTIONS
   gtk_widget_hide (matrix_vbox);
   gtk_widget_hide (matrix_hseparator);
@@ -1089,8 +1092,10 @@ void init_lcms_options (void)
   if (vals.bpc == cmsFLAGS_BLACKPOINTCOMPENSATION)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (bpc_checkbutton), TRUE);
 
-  if (gimp_image_has_icc_profile (vals.o_image_ID)) {
-    char *color_name = gimp_image_get_icc_profile_color_space_name (vals.o_image_ID);
+  if (gimp_image_has_icc_profile (vals.o_image_ID, ICC_IMAGE_PROFILE)) {
+    char *color_name =
+                   gimp_image_get_icc_profile_color_space_name (vals.o_image_ID,
+                                                             ICC_IMAGE_PROFILE);
     gtk_entry_set_text (GTK_ENTRY (select_iprofile_entry), "intern");
     gtk_widget_hide (wksp_profile_label);
     gtk_widget_hide (select_iprofile_entry);
@@ -1107,6 +1112,7 @@ void init_lcms_options (void)
       gtk_widget_set_sensitive (color_mode_cmyk_radiobutton, FALSE);
 ;
     }
+    if(color_name) free(color_name);
   } else {
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (direct_checkbutton), TRUE);
   }
@@ -1120,10 +1126,7 @@ void init_lcms_options (void)
 
 void
 on_select_iprofile_entry_insert_text   (GtkEditable     *editable,
-/*                                        gchar           *new_text,
-                                        gint             new_text_length,
-                                        gint            *position,
-*/                                        gpointer         user_data)
+                                        gpointer         user_data)
 {
 
   sprintf (vals.i_profile, gtk_editable_get_chars(editable,0,
@@ -1134,10 +1137,7 @@ on_select_iprofile_entry_insert_text   (GtkEditable     *editable,
 
 void
 on_select_oprofile_entry_insert_text   (GtkEditable     *editable,
-/*                                        gchar           *new_text,
-                                        gint             new_text_length,
-                                        gint            *position,
-*/                                        gpointer         user_data)
+                                        gpointer         user_data)
 {
 
   sprintf (vals.o_profile, "%s", gtk_entry_get_text (GTK_ENTRY (editable)));
@@ -1185,7 +1185,8 @@ set_profile_selection (gpointer user_data, char *profile)
   if ((strcmp (profile,"")) == 0)
     sprintf (profile, "/usr/share/color/icc/");
 
-  if ((strcmp (profile,"c")) == 0)
+  if ((strcmp (profile,"h")) == 0 ||
+      (strcmp (profile,"u")) == 0)
     sprintf (profile, "~/.color/icc/");
 
   gtk_file_selection_set_filename (GTK_FILE_SELECTION(fileselection1),
