@@ -1,5 +1,5 @@
 /*
- * "$Id: print.c,v 1.3 2005/05/09 20:43:52 beku Exp $"
+ * "$Id: print.c,v 1.7 2005/11/05 06:51:43 beku Exp $"
  *
  *   Print plug-in for the GIMP.
  *
@@ -56,7 +56,7 @@
 
 #undef DDBUG
 #ifdef DEBUG
- #include "m_debug.h"
+ #define DBG printf("%s:%d %s()\n",__FILE__,__LINE__,__func__);
 #else
  #define DBG 
 #endif
@@ -230,11 +230,11 @@ run (char   *name,		/* I - Name of print program. */
   vals.o_drawable_ID = vals.e_drawable_ID = vals.s_drawable_ID = param[2].data.d_int32;
 
   {
-        char *text, *optr;
+        char *text, *optr, *filename;
 
         optr = text = g_new (char,MAX_PATH);
-
-        sprintf(text,"%s", gimp_image_get_filename (vals.o_image_ID));
+        filename = gimp_image_get_filename (vals.o_image_ID);
+        sprintf(text,"%s", filename);
         if (strchr(text, G_DIR_SEPARATOR))
           text = strrchr(text, G_DIR_SEPARATOR) + 1;
 
@@ -242,6 +242,7 @@ run (char   *name,		/* I - Name of print program. */
         sprintf (vals.tiff_file,"%s",text);
         //g_print ("%s:%d vals.image_filename %s|%s\n",__FILE__,__LINE__,vals.image_filename,vals.tiff_file);
         g_free (optr);
+        g_free (filename);
   }
 
   /*
@@ -329,9 +330,10 @@ run (char   *name,		/* I - Name of print program. */
          || linear.use_lin) {
           alpha_capable = GIMP_EXPORT_CAN_HANDLE_ALPHA;
         }
-        if (gimp_image_has_icc_profile (vals.o_image_ID)) {
+        if (gimp_image_has_icc_profile (vals.o_image_ID, ICC_IMAGE_PROFILE)) {
           char *csp;
-          csp = gimp_image_get_icc_profile_color_space_name  (vals.o_image_ID);
+          csp = gimp_image_get_icc_profile_color_space_name  (vals.o_image_ID,
+                                                             ICC_IMAGE_PROFILE);
           if (strstr( csp, "Cmyk" ) != 0) {
             vals.color_space = PT_CMYK;
             alpha_capable = GIMP_EXPORT_CAN_HANDLE_ALPHA;
@@ -353,10 +355,6 @@ run (char   *name,		/* I - Name of print program. */
           vals.e_image_ID = vals.s_image_ID;
           vals.e_drawable_ID = vals.s_drawable_ID;
           drawable_type = gimp_drawable_type(vals.e_drawable_ID);
-          if (vals.show_image) {
-            //gimp_display_new (vals.e_image_ID);
-          }
-          //g_print ("%s:%d %s() export done to drawable %d of type %d.\n",__FILE__,__LINE__,__func__, vals.e_drawable_ID, drawable_type);
         }
       }
       break;
@@ -367,18 +365,10 @@ run (char   *name,		/* I - Name of print program. */
   // convert_image   ...  do all colour conversions and report status
   if (vals.icc || linear.use_lin) {
     values[0].data.d_status = create_separated_image ();
-    //g_print ("%s:%d %s() Conversion done for %d|%d.\n",__FILE__,__LINE__,__func__,vals.s_image_ID,vals.s_drawable_ID);
   } else {
     values[0].data.d_status = GIMP_PDB_SUCCESS;
     vals.s_image_ID = vals.e_image_ID;
     vals.s_drawable_ID = vals.e_drawable_ID;
-    //g_print ("%s:%d %s() no Separation.\n",__FILE__,__LINE__,__func__);
-  }
-
-  if (vals.icc == TRUE) {
-    ;//gimp_procedural_db_set_data("print_use_icc","TRUE",4);
-  } else {
-    ;//gimp_procedural_db_set_data("print_use_icc","FALSE",5);
   }
 
   // p_gimp_save_tiff
@@ -386,6 +376,16 @@ run (char   *name,		/* I - Name of print program. */
     char *profile=NULL;
 
     if (vals.icc) {
+      if(gimp_image_has_icc_profile (vals.o_image_ID, ICC_PROOF_PROFILE))
+      {
+        char *description = gimp_image_get_icc_profile_description( vals.o_image_ID,
+                                                             ICC_PROOF_PROFILE);
+        if(description)
+        {
+          sprintf (vals.o_profile, description);
+          free( description );
+        }
+      }
       profile = vals.o_profile;
     } else {
       profile = malloc (MAX_PATH);
@@ -406,9 +406,9 @@ run (char   *name,		/* I - Name of print program. */
 
     // initialise libgimpprint
 
-  if (getenv("STP_DEBUG_STARTUP"))
-    while (SDEBUG)
-      ;
+    if (getenv("STP_DEBUG_STARTUP"))
+      while (SDEBUG)
+        ;
 
     stp_init();
 
@@ -515,10 +515,8 @@ run (char   *name,		/* I - Name of print program. */
 
     if (values[0].data.d_status == GIMP_PDB_SUCCESS) {
       drawable = gimp_drawable_get ( vals.s_drawable_ID );
-      //g_print ("%s:%d %s() %d|%d|%d\n",__FILE__,__LINE__,__func__,(int)drawable,vals.s_image_ID, drawable->id);
     } else {
       drawable = NULL;
-      //g_print ("%s:%d %s() \n",__FILE__,__LINE__,__func__);
       return;
     }
 
@@ -546,28 +544,6 @@ run (char   *name,		/* I - Name of print program. */
       }
 
     if (vals.color_space == PT_CMYK) {
-      #if 0
-      if (vals.icc) {
-        char *label   = g_new(char, 128);
-        char *l_param = g_new(char, 32);
-        char *tooltip = g_new(char, 512);
-
-        sprintf (label, _("Tweak Output...\n(Experts only)"));
-        sprintf (l_param, "calibrated_warning_label");
-        sprintf (tooltip, _("You are in calibrated mode. This is very sensible to new color adjustments.\nIt is likely that changes here decalibrate the printer.\nSettings will be not saved!"));
-
-        g_print ("%s:%d %s() %s /%d\n",__FILE__,__LINE__,__func__,label, strlen (label)+1);
-        stp_set_string_parameter (gimp_vars.v, l_param, label); 
-        //stp_set_string_parameter_active(gimp_vars.v, l_param,
-          //                              STP_PARAMETER_ACTIVE);
-        g_print ("%s:%d %s() %s /%d\n",__FILE__,__LINE__,__func__,tooltip, strlen (tooltip)+1);
-        stp_set_string_parameter (gimp_vars.v, "calibrated_warning_tooltip", tooltip);
-        //stp_set_string_parameter_active(gimp_vars.v, t_param,
-          //                              STP_PARAMETER_ACTIVE);
- 
-        g_print ("%s:%d %s() %s\n",__FILE__,__LINE__,__func__,stp_get_string_parameter (gimp_vars.v, (char*)"calibrated_warning_label") );
-      }
-      #endif
       if (drawable->bpp > 4 ) {
         sprintf (vals.color_type , "CMYK");
       }
