@@ -299,7 +299,7 @@ static void   run        (gchar   *name,
                           gint    *nreturn_vals,
                           GimpParam **return_vals);
 
-static GImageType  psd_type_to_gimp_type      (psd_imagetype  psdtype);
+static GDrawableType  psd_type_to_gimp_type      (psd_imagetype  psdtype);
 static GImageType  psd_type_to_gimp_base_type (psd_imagetype  psdtype);
 static GLayerMode  psd_lmode_to_gimp_lmode    (gchar          modekey[4]);
 static GimpUnit    psd_unit_to_gimp_unit      (gint           psdunit);
@@ -1294,9 +1294,11 @@ seek_to_and_unpack_pixeldata(FILE* fd, gint layeri, gint channeli)
     }
 	  
   if (tmpline)
-    g_free(tmpline);
+  {  g_free(tmpline);
+  }
   else
-    IFDBG {printf("\nTRIED TO FREE NULL!");fflush(stdout);}
+  {  IFDBG {printf("\nTRIED TO FREE NULL!");fflush(stdout);}
+  }
 }
 
 
