@@ -44,7 +44,7 @@
 #endif
 
 #ifdef DEBUG
- #include "m_debug.h"
+ #define DBG(text) printf("%s:%d %s() %s\n",__FILE__,__LINE__,__func__,text);
 #else
  #define DBG(text) 
 #endif
@@ -157,10 +157,12 @@ getProfileInfo                   ( cmsHPROFILE  hProfile)
       sprintf (profile_info,"%s%s %s",profile_info,text,
                               cmsTakeProductDesc(hProfile));
       if (!first) {
-        len = min_len - strlen (profile_info);
+        int old_len = strlen (profile_info);
+        if (old_len < min_len)
+          len = min_len - old_len;
 
         for (i=0; i < len * 2.2; i++) {
-          sprintf (profile_info,"%s ",profile_info);
+          snprintf (profile_info,2048,"%s ",profile_info);
         }
       }
       sprintf (profile_info,"%s\n",profile_info);
@@ -267,6 +269,10 @@ get_ifd_info                (   TIFF           *tif,
       TIFFSetField(tif, TIFFTAG_SGILOGDATAFMT, ci->logDataFMT);
     }
   }
+  if (ci->photomet == 32803) {/* Bayer pattern */
+    ci->photomet = PHOTOMETRIC_MINISBLACK;
+    printf("%s:%d %s() Bayer pattern detected.\n",__FILE__,__LINE__,__func__);
+  }
 
   if (!TIFFGetField (tif, TIFFTAG_FILLORDER, &ci->save_vals.fillorder))
     ci->save_vals.fillorder = 0;
@@ -591,9 +597,10 @@ get_ifd_info                (   TIFF           *tif,
         ci->worst_case = 1;
   #endif
     }
+#   ifdef DEBUG
     printf ("%s:%d IFD %ld: bps = %d\n",__FILE__,__LINE__,
              ci->aktuelles_dir, ci->bps);
-
+#   endif
     switch (ci->image_type) {
       case GRAY:
         ci->layer_type = (ci->alpha) ? GRAYA_IMAGE : GRAY_IMAGE;
@@ -733,10 +740,10 @@ get_ifd_info                (   TIFF           *tif,
           ci->icc_profile_info = getProfileInfo (hIn);
           g_print ("%s\n",ci->icc_profile_info);
           cmsCloseProfile (hIn);
-          #ifdef DEBUG
+          #ifdef DEBUG_
           saveProfileToFile ("/tmp/cinepaint_tiff_load.icc", ci->icc_profile,
                              ci->profile_size);
-          DBG ("profile saved to /tmp/cinepaint_tiff_load.icc\n")
+          DBG ("profile saved to /tmp/cinepaint_tiff_load.icc")
           #endif
         } else {
           ci->profile_size = 0;
@@ -756,16 +763,16 @@ get_ifd_info                (   TIFF           *tif,
     && (ci->filetype & FILETYPE_MASK) )
   {  /*viewport definitions */
       ci->top->image_height = 
-	( ci->rows * ci->top->yres / ci->yres);
+        ( ci->rows * ci->top->yres / ci->yres);
       ci->top->image_width = 
-	( ci->cols * ci->top->xres / ci->xres);
+        ( ci->cols * ci->top->xres / ci->xres);
       ci->top->viewport_offx = ci->offx;
       ci->top->viewport_offy = ci->offy;
   } else if (!ci->worst_case) {
         /* show all IFD s */
       /* actual IFD more bottom? */
     if ( ( ci->rows + (guint16)(ci->ypos * ci->yres) )
-	  >  ci->top->image_height ) 
+          >  ci->top->image_height ) 
         ci->top->image_height = ci->rows
 		 + (guint16)(ci->ypos * ci->yres);
       /* actual IFD more right? */
@@ -1165,7 +1172,7 @@ get_layer_info               (  gint32           image_ID,
       }
 
 #ifndef GIMP_MAJOR_VERSION
-  if (gimp_image_has_icc_profile(image_ID)) {
+  if (gimp_image_has_icc_profile(image_ID, ICC_IMAGE_PROFILE)) {
     int size, ink;
     cmsHPROFILE profile=NULL;
     char *mem_profile=NULL;
@@ -1173,11 +1180,11 @@ get_layer_info               (  gint32           image_ID,
     info->icc_profile = NULL;
     info->profile_size = 0;
     info->icc_profile_info = NULL;
-    mem_profile = gimp_image_get_icc_profile_by_mem(image_ID, &size);
+    mem_profile = gimp_image_get_icc_profile_by_mem(image_ID, &size, ICC_IMAGE_PROFILE);
     if (mem_profile && size) {
       profile = cmsOpenProfileFromMem (mem_profile, size);
       sprintf (color_space_name_,
-               gimp_image_get_icc_profile_color_space_name (image_ID));
+               gimp_image_get_icc_profile_color_space_name (image_ID, ICC_IMAGE_PROFILE));
       DBG (color_space_name_)
     
       ink = photometFromICC(color_space_name_, info);
@@ -1185,17 +1192,19 @@ get_layer_info               (  gint32           image_ID,
       info->profile_size = size;
       info->icc_profile = mem_profile;
       info->icc_profile_info = calloc (sizeof (char),
-                             strlen(gimp_image_get_icc_profile_info(image_ID)));
+          strlen(gimp_image_get_icc_profile_info(image_ID, ICC_IMAGE_PROFILE)));
       sprintf (info->icc_profile_info,
-                             gimp_image_get_icc_profile_info(image_ID));
+                  gimp_image_get_icc_profile_info(image_ID, ICC_IMAGE_PROFILE));
       memcpy (&info->colorspace[0], color_space_name_, 4);
       info->colorspace[4] = '\000';
       DBG (info->icc_profile_info)
     } else {
+#     ifdef DEBUG
       printf ("%s:%d could not get an profile\n",__FILE__,__LINE__);
+#     endif
       info->save_vals.use_icc_profile = FALSE;
     }
-    if (strcmp (gimp_image_get_icc_profile_color_space_name(image_ID) , "XYZ")
+    if (strcmp (gimp_image_get_icc_profile_color_space_name(image_ID, ICC_IMAGE_PROFILE) , "XYZ")
         == 0) {
           if (info->bps == 32 && info->spp >= 3) {
             info->photomet = PHOTOMETRIC_LOGLUV;
